@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -13,11 +13,13 @@ import logging
 import os
 import shlex
 import sys
+from typing import Optional
 
 import click
 
 from spsdk import SPSDKError
 from spsdk.apps.blhost_helper import (
+    PROPERTIES_OVERRIDE,
     OemGenMasterShareHelp,
     OemSetMasterShareHelp,
     parse_key_prov_key_type,
@@ -26,6 +28,7 @@ from spsdk.apps.blhost_helper import (
     parse_trust_prov_oem_key_type,
     progress_bar,
 )
+from spsdk.apps.utils import spsdk_logger
 from spsdk.apps.utils.common_cli_options import (
     CommandsTreeGroup,
     isp_interfaces,
@@ -37,6 +40,7 @@ from spsdk.apps.utils.utils import (
     catch_spsdk_error,
     format_raw_data,
     get_interface,
+    get_key,
     parse_file_and_size,
     parse_hex_data,
 )
@@ -46,7 +50,7 @@ from spsdk.utils.images import BinaryImage
 
 
 @click.group(name="blhost", no_args_is_help=True, cls=CommandsTreeGroup)
-@isp_interfaces(uart=True, usb=True, lpcusbsio=True, buspal=True)
+@isp_interfaces(uart=True, usb=True, sdio=True, lpcusbsio=True, buspal=True)
 @spsdk_apps_common_options
 @click.option(
     "-s",
@@ -59,6 +63,7 @@ def main(
     ctx: click.Context,
     port: str,
     usb: str,
+    sdio: str,
     buspal: str,
     lpcusbsio: str,
     use_json: bool,
@@ -68,7 +73,7 @@ def main(
 ) -> int:
     """Utility for communication with the bootloader on target."""
     log_level = log_level or logging.WARNING
-    logging.basicConfig(level=log_level)
+    spsdk_logger.install(level=log_level)
 
     # print help for get-property if property tag is 0 or 'list-properties'
     if ctx.invoked_subcommand == "get-property":
@@ -87,6 +92,7 @@ def main(
                 module="mboot",
                 port=port,
                 usb=usb,
+                sdio=sdio,
                 timeout=timeout,
                 buspal=buspal,
                 lpcusbsio=lpcusbsio,
@@ -115,8 +121,6 @@ def batch(ctx: click.Context, command_file: str) -> None:
     \b
     COMMAND_FILE    - path to blhost command file
     """
-    click.secho("This is an experimental command. Use at your own risk!", fg="yellow")
-
     with open(command_file) as f:
         for line in f.readlines():
             tokes = shlex.split(line, comments=True)
@@ -324,7 +328,6 @@ def flash_image(ctx: click.Context, image_file_path: str, erase: str, memory_id:
     with McuBoot(ctx.obj["interface"]) as mboot:
         if erase == "erase":
             for segment in bin_image.sub_images:
-
                 mboot.flash_erase_region(
                     address=segment.aligned_start(ALIGNMENT),
                     length=segment.aligned_length(ALIGNMENT),
@@ -609,8 +612,19 @@ def load_image(ctx: click.Context, boot_file: click.File) -> None:
 @main.command()
 @click.argument("property_tag", type=str, required=True)
 @click.argument("index", type=INT(), default="0")
+@click.option(
+    "-f",
+    "--family",
+    type=click.Choice(list(PROPERTIES_OVERRIDE.keys())),
+    help="Provide family. So far supported 'kw45xx' and 'k32w1xx'",
+)
 @click.pass_context
-def get_property(ctx: click.Context, property_tag: str, index: int) -> None:
+def get_property(
+    ctx: click.Context,
+    property_tag: str,
+    index: int,
+    family: str,
+) -> None:
     """Queries various bootloader properties and settings.
 
     Each supported property has a unique <PROPERTY_TAG>.
@@ -652,14 +666,24 @@ def get_property(ctx: click.Context, property_tag: str, index: int) -> None:
     28 or 'irq-notify-pin'              Interrupt notifier pin
     29 or 'pfr-keystore_update-opt'     PFR key store update option
     30 or 'byte-write-timeout-ms'       Byte write timeout in ms
+    31 or 'fuse-locked-status'          Fuse Locked Status
+    for kw45xx/k32w1xx devices:
+    10 or 'verify-erases'               Verify Erases flag
+    20 or 'boot status'                 Value of Boot Status Register
+    21 or 'loadable-fw-version'         LoadableFWVersion
+    22 or 'fuse-program-voltage'        Fuse Program Voltage
 
     \b
     Note: Not all the properties are available for all devices.
     """
-    property_tag_int = parse_property_tag(property_tag)
+    property_tag_int = parse_property_tag(property_tag, family)
     with McuBoot(ctx.obj["interface"]) as mboot:
         response = mboot.get_property(property_tag_int, index=index)  # type: ignore
-        property_text = str(parse_property_value(property_tag_int, response)) if response else None
+        property_text = (
+            str(parse_property_value(property_tag_int, response, None, family))
+            if response
+            else None
+        )
         display_output(
             response, mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"], property_text
         )
@@ -668,8 +692,19 @@ def get_property(ctx: click.Context, property_tag: str, index: int) -> None:
 @main.command()
 @click.argument("property_tag", type=str, required=True)
 @click.argument("value", type=INT(), required=True)
+@click.option(
+    "-f",
+    "--family",
+    type=click.Choice(list(PROPERTIES_OVERRIDE.keys())),
+    help="Provide family. So far supported 'kw45xx' and 'k32w1xx'",
+)
 @click.pass_context
-def set_property(ctx: click.Context, property_tag: str, value: int) -> None:
+def set_property(
+    ctx: click.Context,
+    property_tag: str,
+    value: int,
+    family: str,
+) -> None:
     """Changes properties and options in the bootloader.
 
     Accepts the same <PROPERTY_TAG> used with the get-property sub-command;
@@ -686,11 +721,14 @@ def set_property(ctx: click.Context, property_tag: str, value: int) -> None:
     28 or 'irq-notify-pin'              Interrupt notifier pin
     29 or 'pfr-keystore_update-opt'     PFR key store update option
     30 or 'byte-write-timeout-ms'       Byte write timeout in ms
+    for kw45xx/k32w1xx devices:
+    10 or 'verify-erases'               Verify Erases flag
+    22 or 'fuse-program-voltage'        Fuse Program Voltage
 
     \b
     Note: Not all properties can be set on all devices.
     """
-    property_tag_int = parse_property_tag(property_tag)
+    property_tag_int = parse_property_tag(property_tag, family)
     with McuBoot(ctx.obj["interface"]) as mboot:
         mboot.set_property(prop_tag=property_tag_int, value=value)  # type: ignore
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
@@ -926,16 +964,33 @@ def program_aeskey(ctx: click.Context, file: click.File) -> None:
 
 
 @key_provisioning.command(name="set_user_key")
+@click.option(
+    "-s",
+    "--key-size",
+    type=INT(),
+    required=False,
+    help=(
+        "Key size in bits. If this field is defined,"
+        " the command could load as text as binary form of key."
+    ),
+)
 @click.argument("key_type", metavar="TYPE", type=str, required=True)
 @click.argument("file_and_size", metavar="FILE[,SIZE]", type=str, required=True)
 @click.pass_context
-def set_user_key(ctx: click.Context, key_type: str, file_and_size: str) -> None:
+def set_user_key(ctx: click.Context, key_type: str, file_and_size: str, key_size: int) -> None:
     """Sends the user key specified by type to the bootloader.
 
     <FILE> is the binary file containing user key plain text.
     If <SIZE> is not specified, the entire <FILE> will be sent.
     Otherwise, blhost only sends the first <SIZE> bytes.
 
+    \b
+    If the '--key-size' option is defined, the argument 'file_and_size' accepts those formats:
+     - filename to binary file
+     - filename to text file
+     - string with key
+    The size part is ignored in this case.
+
     \b
     TYPE  - Type of user key
     FILE  - Binary file containing user key plaintext
@@ -959,8 +1014,11 @@ def set_user_key(ctx: click.Context, key_type: str, file_and_size: str) -> None:
     """
     file_path, size = parse_file_and_size(file_and_size)
     key_type_int = parse_key_prov_key_type(key_type)
-    with open(file_path, "rb") as key_file:
-        key_data = key_file.read(size)
+
+    if not key_size:
+        key_size = (size if size > 0 else os.path.getsize(file_path)) * 8
+
+    key_data = get_key(file_path, expected_size=key_size // 8)
 
     with McuBoot(ctx.obj["interface"]) as mboot:
         mboot.kp_set_user_key(key_type=key_type_int, key_data=key_data)  # type: ignore
@@ -1419,7 +1477,7 @@ def oem_get_cust_cert_dice_puk(
     """Creates the initial trust provisioning keys.
 
     \b
-    OEM_RKTH_INPUT_ADDR                 - The input buffer address where the OEM RKTH locates at
+    OEM_RKTH_INPUT_ADDR                - The input buffer address where the OEM RKTH locates at
     OEM_RKTH_INPUT_SIZE                - The byte count of the OEM RKTH
     OEM_CUST_CERT_DICE_PUK_OUTPUT_ADDR - The output buffer address where ROM writes the OEM Customer
                                          Certificate Public Key for DICE to
@@ -1461,6 +1519,41 @@ def update_life_cycle(ctx: click.Context, life_cycle: int) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
+@main.command()
+@click.argument("cmd-msg-addr", metavar="COMMAND MESSAGE ADDRESS", type=INT(), required=True)
+@click.argument("cmd-msg-cnt", metavar="COMMAND MESSAGE COUNT", type=INT(), required=True)
+@click.argument("resp-msg-addr", metavar="RESPONSE MESSAGE ADDRESS", type=INT(), required=True)
+@click.argument("resp-msg-cnt", metavar="RESPONSE MESSAGE COUNT", type=INT(), required=True)
+@click.pass_context
+def ele_message(
+    ctx: click.Context, cmd_msg_addr: int, cmd_msg_cnt: int, resp_msg_addr: int, resp_msg_cnt: int
+) -> None:
+    """Send message to EdgeLock Enclave.
+
+    This command is designed to be, as general, as is possible to work with EdgeLock Enclave.
+    EdgeLock Enclave message is prepared in PC and stored in target RAM (for example by 'blhost write-memory').
+    The response of ELE command is stored also in target memory on place that is defined by 'resp-msg-addr
+    and could be read back (for example by 'blhost read-memory').
+
+
+    Size of command message and response is in count of 32-bit words.
+
+    \b
+    COMMAND MESSAGE ADDRESS     - Address in target where is stored the command words.
+    COMMAND MESSAGE COUNT       - Count of the stored command words.
+    RESPONSE MESSAGE ADDRESS    - Address in target memory space where the ELE store response.
+    RESPONSE MESSAGE COUNT      - Maximal count of words reserved for response.
+    """
+    with McuBoot(ctx.obj["interface"]) as mboot:
+        mboot.ele_message(
+            cmdMsgAddr=cmd_msg_addr,
+            cmdMsgCnt=cmd_msg_cnt,
+            respMsgAddr=resp_msg_addr,
+            respMsgCnt=resp_msg_cnt,
+        )
+        display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
+
+
 @trust_provisioning.command(name="prove_genuinity")
 @click.argument("address", type=INT())
 @click.argument("buffer_size", type=INT())
@@ -1478,7 +1571,9 @@ def prove_genuinity(ctx: click.Context, address: int, buffer_size: int) -> None:
             [tp_response_length],
             mboot.status_code,
             use_json=ctx.obj["use_json"],
-            extra_output=f"TP response will be {tp_response_length} bytes long.",
+            extra_output=f"TP response will be {tp_response_length} bytes long."
+            if tp_response_length
+            else None,
         )
 
 
@@ -1501,11 +1596,11 @@ def set_wrap_data(ctx: click.Context, address: int, control: int, stage: int) ->
 
 
 def display_output(
-    response: list = None,
+    response: Optional[list] = None,
     status_code: int = 0,
     use_json: bool = False,
     suppress: bool = False,
-    extra_output: str = None,
+    extra_output: Optional[str] = None,
 ) -> None:
     """Displays response and status code.
 
