@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -15,12 +15,14 @@ from click_option_group import RequiredMutuallyExclusiveOptionGroup, optgroup
 
 from spsdk import __version__ as spsdk_version
 from spsdk.apps.utils.utils import SPSDKAppError, catch_spsdk_error
+from spsdk.crypto.signature_provider import get_signature_provider
 from spsdk.exceptions import SPSDKError
-from spsdk.image import SB3_SCH_FILE, TrustZone, get_mbi_class
+from spsdk.image import TrustZone, get_mbi_class
 from spsdk.image.mbimg import mbi_generate_config_templates, mbi_get_supported_families
 from spsdk.sbfile.sb2.images import generate_SB21
-from spsdk.sbfile.sb31.images import SecureBinary31
+from spsdk.sbfile.sb31.images import SB3_SCH_FILE, SecureBinary31
 from spsdk.utils.misc import load_configuration, write_file
+from spsdk.utils.plugins import load_plugin_from_source
 from spsdk.utils.schema_validator import ValidationSchemas, check_config
 
 SUPPORTED_FAMILIES = mbi_get_supported_families()
@@ -33,7 +35,7 @@ def generate_trustzone_binary(tzm_conf: str) -> None:
     check_config(config_data, TrustZone.get_validation_schemas(config_data["family"]))
     trustzone = TrustZone.from_config(config_data)
     tz_data = trustzone.export()
-    output_file = config_data["tzpOutputFile"]
+    output_file = os.path.abspath(config_data["tzpOutputFile"])
     write_file(tz_data, output_file, mode="wb")
     click.echo(f"Success. (Trustzone binary: {output_file} created.)")
 
@@ -53,18 +55,17 @@ def generate_config_templates(family: str, output_folder: str) -> None:
 
     # And generate all config templates files
     for key, val in templates.items():
-        file_name = f"{key}.yml"
+        file_name = f"{key}.yaml"
         if os.path.isfile(output_folder):
             raise SPSDKError(f"The specified path {output_folder} is file.")
         if not os.path.isdir(output_folder):
             os.mkdir(output_folder)
-        full_file_name = os.path.join(output_folder, file_name)
+        full_file_name = os.path.abspath(os.path.join(output_folder, file_name))
         if not os.path.isfile(full_file_name):
             click.echo(f"Creating {file_name} template file.")
-            with open(full_file_name, "w") as f:
-                f.write(val)
+            write_file(val, full_file_name)
         else:
-            click.echo(f"Skip creating {file_name}, this file already exists.")
+            click.echo(f"Skip creating {full_file_name}, this file already exists.")
 
 
 def generate_master_boot_image(image_conf: str) -> None:
@@ -79,7 +80,7 @@ def generate_master_boot_image(image_conf: str) -> None:
     mbi.load_from_config(config_data)
     mbi_data = mbi.export()
 
-    mbi_output_file_path = config_data["masterBootOutputFile"]
+    mbi_output_file_path = os.path.abspath(config_data["masterBootOutputFile"])
     write_file(mbi_data, mbi_output_file_path, mode="wb")
 
     click.echo(f"Success. (Master Boot Image: {mbi_output_file_path} created.)")
@@ -93,6 +94,7 @@ def generate_secure_binary_21(
     signing_certificate_file_paths: List[str],
     root_key_certificate_paths: List[str],
     hoh_out_path: str,
+    plugin: str,
     external_files: List[str],
 ) -> None:
     """Generate SecureBinary image from BD command file.
@@ -109,23 +111,32 @@ def generate_secure_binary_21(
     passed in signing_certificate_file_paths.
     :param hoh_out_path: output path to hash of hashes of root keys. If set to
     None, 'hash.bin' is created under working directory.
+    :param plugin: External python file containing a custom SignatureProvider implementation.
     :param external_files: external files referenced from BD file.
 
     :raises SPSDKAppError: If incorrect bf file is provided
     """
+    if plugin:
+        load_plugin_from_source(plugin)
     if output_file_path is None:
         raise SPSDKAppError("Error: no output file was specified")
+    signature_provider = (
+        get_signature_provider(local_file_key=private_key_file_path)
+        if os.path.isfile(private_key_file_path)
+        else get_signature_provider(sp_cfg=private_key_file_path)
+    )
     try:
         sb2_data = generate_SB21(
             bd_file_path=str(bd_file_path),
             key_file_path=str(key_file_path),
-            private_key_file_path=str(private_key_file_path),
+            signature_provider=signature_provider,
             signing_certificate_file_paths=[str(x) for x in signing_certificate_file_paths],
             root_key_certificate_paths=[str(x) for x in root_key_certificate_paths],
             hoh_out_path=str(hoh_out_path),
             external_files=[str(x) for x in external_files],
         )
-        write_file(sb2_data, str(output_file_path), mode="wb")
+        output_file_path = os.path.abspath(output_file_path)
+        write_file(sb2_data, output_file_path, mode="wb")
     except SPSDKError as exc:
         raise SPSDKAppError(f"The SB2.1 file generation failed: ({str(exc)}).") from exc
     else:
@@ -135,17 +146,22 @@ def generate_secure_binary_21(
 def generate_secure_binary_31(container_conf: str) -> None:
     """Generate SecureBinary image from json configuration file.
 
+    RoTKTH/SB3KDK is printed out in verbose mode
+
     :param container_conf: configuration file
     :raises SPSDKError: Raised when there is no signing key
     """
     config_data = load_configuration(container_conf)
-    schemas = SecureBinary31.get_validation_schemas(include_test_configuration=True)
+    check_config(config_data, SecureBinary31.get_validation_schemas_family())
+    schemas = SecureBinary31.get_validation_schemas(
+        config_data["family"], include_test_configuration=True
+    )
     schemas.append(ValidationSchemas.get_schema_file(SB3_SCH_FILE)["sb3_output"])
     check_config(config_data, schemas)
     sb3 = SecureBinary31.load_from_config(config_data)
-    sb3_data = sb3.export()
+    sb3_data = sb3.export(cert_block=None)
 
-    sb3_output_file_path = config_data["containerOutputFile"]
+    sb3_output_file_path = os.path.abspath(config_data["containerOutputFile"])
     write_file(sb3_data, sb3_output_file_path, mode="wb")
 
     click.echo(f"Success. (Secure binary 3.1: {sb3_output_file_path} created.)")
@@ -223,6 +239,12 @@ with -S/--cert arg.",
     help="Path to output hash of hashes of root keys. If argument is not \
 provided, then by default the tool creates hash.bin in the working directory.",
 )
+@click.option(
+    "--plugin",
+    type=click.Path(exists=True, dir_okay=False),
+    required=False,
+    help="External python file containing a custom SignatureProvider implementation.",
+)
 @click.version_option(spsdk_version, "--version")
 @click.help_option("--help")
 @click.argument("external", type=click.Path(), nargs=-1)
@@ -239,12 +261,14 @@ def main(
     tzm_conf: str,
     config_template: str,
     hash_of_hashes: str,
+    plugin: str,
     external: List[str],
 ) -> None:
     """Tool for generating TrustZone, MasterBootImage and SecureBinary images.
 
     !!! The ELFTOSB tool is deprecated, use new NXPIMAGE tool from SPSDK for new projects !!!
     """
+    click.secho("Deprecated tool! Use npximage instead", fg="yellow")
     if command:
         if output is None:
             raise SPSDKAppError("Error: no output file was specified")
@@ -256,6 +280,7 @@ def main(
             signing_certificate_file_paths=cert,
             root_key_certificate_paths=root_key_cert,
             hoh_out_path=hash_of_hashes,
+            plugin=plugin,
             external_files=external,
         )
 
