@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2021-2022 NXP
+# Copyright 2021-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,19 +11,15 @@ import os
 import sys
 
 import click
-from click_option_group import RequiredMutuallyExclusiveOptionGroup, optgroup
+from click_option_group import optgroup
 
 from spsdk import SPSDK_DATA_FOLDER, SPSDKError
+from spsdk.apps.utils import spsdk_logger
 from spsdk.apps.utils.common_cli_options import (
     CommandsTreeGroupAliasedGetCfgTemplate,
     spsdk_apps_common_options,
 )
-from spsdk.apps.utils.utils import (
-    SPSDKAppError,
-    catch_spsdk_error,
-    check_destination_dir,
-    check_file_exists,
-)
+from spsdk.apps.utils.utils import SPSDKAppError, catch_spsdk_error, check_file_exists
 from spsdk.crypto import (
     Encoding,
     ec,
@@ -34,7 +30,7 @@ from spsdk.crypto import (
 )
 from spsdk.crypto.certificate_management import generate_name
 from spsdk.crypto.loaders import extract_public_key, load_certificate
-from spsdk.utils.misc import find_file, load_configuration
+from spsdk.utils.misc import find_file, load_configuration, load_text, write_file
 
 NXPCERTGEN_DATA_FOLDER: str = os.path.join(SPSDK_DATA_FOLDER, "nxpcertgen")
 
@@ -64,8 +60,12 @@ class CertificateParametersConfig:  # pylint: disable=too-few-public-methods
 @click.group(name="nxpcertgen", no_args_is_help=True, cls=CommandsTreeGroupAliasedGetCfgTemplate)  # type: ignore
 @spsdk_apps_common_options
 def main(log_level: int) -> None:
-    """Utility for certificate generation."""
-    logging.basicConfig(level=log_level or logging.WARNING)
+    """Utility for certificate generation.
+
+    !!! The NXPCERTGEN tool is deprecated, use new NXPCRYPTO tool from SPSDK for new projects !!!
+    """
+    click.secho("Deprecated tool! Use npxcrypto instead", fg="yellow")
+    spsdk_logger.install(level=log_level, logger=logger)
 
 
 @main.command(name="generate", no_args_is_help=True)
@@ -80,7 +80,7 @@ def main(log_level: int) -> None:
 @click.option(
     "-o",
     "--output",
-    type=click.Path(),
+    type=click.Path(resolve_path=True),
     required=True,
     help="Path where certificate will be stored.",
 )
@@ -103,7 +103,6 @@ def generate(config: str, output: str, encoding: str, force: bool) -> None:
     logger.info("Generating Certificate...")
     logger.info("Loading configuration from yml file...")
 
-    check_destination_dir(output, force)
     check_file_exists(output, force)
 
     config_data = load_configuration(config)
@@ -129,11 +128,11 @@ def generate(config: str, output: str, encoding: str, force: bool) -> None:
     encoding_type = Encoding.PEM if encoding.lower() == "pem" else Encoding.DER
     save_crypto_item(certificate, output, encoding_type=encoding_type)
     logger.info("Certificate generated successfully...")
-    click.echo(f"The certificate file has been created: {os.path.abspath(output)}")
+    click.echo(f"The certificate file has been created: {output}")
 
 
 @main.command(name="get-template", no_args_is_help=True)
-@click.argument("output", metavar="PATH", type=click.Path())
+@click.argument("output", metavar="PATH", type=click.Path(resolve_path=True))
 @click.option(
     "-f",
     "--force",
@@ -148,16 +147,11 @@ def get_template(output: str, force: bool) -> None:
     PATH    - file name path to write template config file
     """
     logger.info("Creating Certificate template...")
-    check_destination_dir(output, force)
     check_file_exists(output, force)
 
-    with open(os.path.join(NXPCERTGEN_DATA_FOLDER, "certgen_config.yml"), "r") as file:
-        template = file.read()
+    write_file(load_text(os.path.join(NXPCERTGEN_DATA_FOLDER, "certgen_config.yaml")), output)
 
-    with open(output, "w") as file:
-        file.write(template)
-
-    click.echo(f"The configuration template file has been created: {os.path.abspath(output)}")
+    click.echo(f"The configuration template file has been created: {output}")
 
 
 @main.command(name="verify", no_args_is_help=True)
@@ -204,8 +198,8 @@ def verify(certificate: str, sign: str, puk: str) -> None:
                 ec.ECDSA(cert.signature_hash_algorithm),
             )
             click.echo("Signature is OK")
-        except:
-            raise SPSDKAppError("Invalid signature")
+        except Exception as e:
+            raise SPSDKAppError("Invalid signature") from e
     if puk:
         logger.info("Performing public key verification")
         cert_puk = cert.public_key()
