@@ -1,19 +1,19 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """NXP USB Device Scanner."""
 
-import logging
 import sys
 from typing import IO
 
 import click
 from click_option_group import MutuallyExclusiveOptionGroup, optgroup
 
+from spsdk.apps.utils import spsdk_logger
 from spsdk.apps.utils.common_cli_options import spsdk_apps_common_options
 from spsdk.apps.utils.utils import catch_spsdk_error
 from spsdk.utils import nxpdevscan
@@ -45,6 +45,13 @@ from spsdk.utils import nxpdevscan
     flag_value="usb",
     help="Search only for USB devices",
 )
+@optgroup.option(
+    "-sd",
+    "--sdio",
+    "scope",
+    flag_value="sdio",
+    help="Search only for SDIO devices",
+)
 @optgroup.option(
     "-p",
     "--port",
@@ -62,10 +69,17 @@ from spsdk.utils import nxpdevscan
 @spsdk_apps_common_options
 def main(extend_vids: str, out: IO[str], scope: str, log_level: int) -> None:
     """Utility listing all connected NXP USB and UART devices."""
-    logging.basicConfig(level=log_level or logging.WARNING)
-
+    spsdk_logger.install(level=log_level)
     additional_vids = [int(vid, 16) for vid in extend_vids]
 
+    if scope in ["all", "sdio"]:
+        nxp_sdio_devices = nxpdevscan.search_nxp_sdio_devices()
+        if out.name == "<stdout>":
+            click.echo(8 * "-" + " Connected NXP SDIO Devices " + 8 * "-" + "\n", out)
+        for sdio_dev in nxp_sdio_devices:
+            click.echo(sdio_dev.info(), out)
+            click.echo("", out)
+
     if scope in ["all", "usb"]:
         nxp_usb_devices = nxpdevscan.search_nxp_usb_devices(additional_vids)
         if out.name == "<stdout>":
