@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -13,8 +13,9 @@ from typing import List
 
 import click
 
+from spsdk.apps.utils import spsdk_logger
 from spsdk.apps.utils.common_cli_options import spsdk_apps_common_options
-from spsdk.apps.utils.utils import catch_spsdk_error, check_destination_dir, check_file_exists
+from spsdk.apps.utils.utils import catch_spsdk_error, check_file_exists
 from spsdk.crypto import (
     ec,
     generate_ecc_private_key,
@@ -72,7 +73,7 @@ def get_list_of_supported_keys() -> List[str]:
     help="Password with which the output file will be encrypted. "
     "If not provided, the output will be unencrypted.",
 )
-@click.argument("path", type=click.Path(file_okay=True))
+@click.argument("path", type=click.Path(file_okay=True, resolve_path=True))
 @click.option(
     "--force",
     is_flag=True,
@@ -82,16 +83,17 @@ def get_list_of_supported_keys() -> List[str]:
 def main(log_level: int, key_type: str, path: str, password: str, force: bool) -> int:
     """NXP Key Generator Tool.
 
+    !!! The NXPKEYGEN tool is deprecated, use new NXPCRYPTO tool from SPSDK for new projects !!!
+
     \b
     PATH    - output file path, where the key pairs (private and public key) will be stored.
               Each key will be stored in separate file (.pub and .pem).
     """
-    logging.basicConfig(level=log_level or logging.WARNING)
+    spsdk_logger.install(level=log_level)
 
     key_param = key_type.lower().strip()
     is_rsa = "rsa" in key_param
 
-    check_destination_dir(path, force)
     check_file_exists(path, force)
     pub_key_path = os.path.splitext(path)[0] + ".pub"
     check_file_exists(pub_key_path, force)
@@ -112,6 +114,7 @@ def main(log_level: int, key_type: str, path: str, password: str, force: bool) -
         logger.info("Saving ECC key pair...")
         save_ecc_private_key(priv_key_ec, path, password if password else None)
         save_ecc_public_key(pub_key_ec, pub_key_path)
+    click.echo(f"The key pair has been created: {(pub_key_path)}, {(path)}")
     return 0
 
 
