@@ -1,24 +1,23 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Console script for pfr."""
 
-import io
 import logging
 import os
 import sys
-from typing import Optional, Tuple, Type, Union
+from typing import Callable, Optional, Tuple, Type, Union
 
 import click
 from click_option_group import MutuallyExclusiveOptionGroup, optgroup
-from ruamel.yaml import YAML
 
-from spsdk import SPSDK_YML_INDENT, pfr
+from spsdk import pfr
 from spsdk.apps.elftosb_utils.sb_31_helper import RootOfTrustInfo
+from spsdk.apps.utils import spsdk_logger
 from spsdk.apps.utils.common_cli_options import (
     FC,
     CommandsTreeGroupAliasedGetCfgTemplate,
@@ -30,17 +29,23 @@ from spsdk.crypto.loaders import extract_public_keys
 from spsdk.mboot import McuBoot
 from spsdk.pfr.exceptions import SPSDKError, SPSDKPfrConfigError, SPSDKPfrError
 from spsdk.pfr.pfrc import Pfrc
-from spsdk.utils.misc import find_file, load_configuration, size_fmt, write_file
+from spsdk.utils.misc import find_file, load_binary, load_configuration, size_fmt, write_file
+from spsdk.utils.schema_validator import ConfigTemplate
 
 PFRArea = Union[Type[pfr.CMPA], Type[pfr.CFPA]]
 logger = logging.getLogger(__name__)
 
 
-def _store_output(data: Union[str, bytes], path: Optional[str], mode: str = "w") -> None:
+def _store_output(
+    data: Union[str, bytes], path: Optional[str], mode: str = "w", msg: Optional[str] = None
+) -> None:
     """Store the output data; either on stdout or into file if it's provided."""
+    if msg:
+        click.echo(msg)
     if path is None:
         click.echo(data)
     else:
+        click.echo(f"Result has been stored in: {path}")
         write_file(data, path=path, mode=mode)
 
 
@@ -49,45 +54,55 @@ def _get_pfr_class(area_name: str) -> PFRArea:
     return getattr(pfr, area_name.upper())
 
 
-def pfr_device_type_options(options: FC) -> FC:
-    """Setup PFR options for device, revision and area (PFR page type)."""
-    options = click.option(
-        "-r",
-        "--revision",
-        help="Chip revision; if not specified, most recent one will be used",
-    )(options)
-    options = click.option(
-        "-d",
-        "--device",
-        type=click.Choice(pfr.CMPA.devices()),
-        help="Device to use",
-        required=True,
-    )(options)
-    options = click.option(
-        "-t",
-        "--type",
-        "area",
-        required=True,
-        type=click.Choice(["cmpa", "cfpa"]),
-        help="Select PFR partition",
-    )(options)
-    return options
+def pfr_device_type_options(no_type: bool = False) -> Callable:
+    """PFR common device click options.
+
+    :param no_type: If true, the type option is not added, defaults to False
+    :return: Click decorator
+    """
+
+    def decorator(options: Callable[[FC], FC]) -> Callable[[FC], FC]:
+        """Setup PFR options for device, revision and area (PFR page type)."""
+        options = click.option(
+            "-r",
+            "--revision",
+            help="Chip revision; if not specified, most recent one will be used",
+        )(options)
+        options = click.option(
+            "-d",
+            "--device",
+            type=click.Choice(pfr.CMPA.devices()),
+            help="Device to use",
+            required=True,
+        )(options)
+        if not no_type:
+            options = click.option(
+                "-t",
+                "--type",
+                "area",
+                required=True,
+                type=click.Choice(["cmpa", "cfpa"]),
+                help="Select PFR partition",
+            )(options)
+        return options
+
+    return decorator
 
 
 @click.group(name="pfr", no_args_is_help=True, cls=CommandsTreeGroupAliasedGetCfgTemplate)
 @spsdk_apps_common_options
 def main(log_level: int) -> int:
     """Utility for generating and parsing Protected Flash Region data (CMPA, CFPA)."""
-    logging.basicConfig(level=log_level or logging.WARNING)
+    spsdk_logger.install(level=log_level)
     return 0
 
 
 @main.command(name="get-template", no_args_is_help=True)
-@pfr_device_type_options
+@pfr_device_type_options()
 @click.option(
     "-o",
     "--output",
-    type=click.Path(),
+    type=click.Path(resolve_path=True),
     required=False,
     help="Save the output into a file instead of console",
 )
@@ -95,17 +110,13 @@ def main(log_level: int) -> int:
 def get_template(device: str, revision: str, area: str, output: str, full: bool) -> None:
     """Generate user configuration template file."""
     pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
-    yaml = YAML(pure=True)
-    yaml.indent(sequence=SPSDK_YML_INDENT * 2, offset=SPSDK_YML_INDENT)
     data = pfr_obj.get_yaml_config(not full)
-    stream = io.StringIO()
-    yaml.dump(data, stream)
-    yaml_data = stream.getvalue()
-    _store_output(yaml_data, output)
+    yaml_data = ConfigTemplate.convert_cm_to_yaml(data)
+    _store_output(yaml_data, output, msg=f"PFR {area} configuration template has been created.")
 
 
 @main.command(name="parse-binary", no_args_is_help=True)
-@pfr_device_type_options
+@pfr_device_type_options()
 @click.option(
     "-o",
     "--output",
@@ -116,7 +127,7 @@ def get_template(device: str, revision: str, area: str, output: str, full: bool)
 @click.option(
     "-b",
     "--binary",
-    type=click.Path(exists=True, dir_okay=False),
+    type=click.Path(exists=True, dir_okay=False, resolve_path=True),
     required=True,
     help="Binary to parse",
 )
@@ -142,8 +153,7 @@ def parse_binary(
     show_diff: bool,
 ) -> None:
     """Parse binary and extract configuration."""
-    with open(binary, "rb") as f:
-        data = f.read()
+    data = load_binary(binary)
     yaml_data = _parse_binary_data(
         data=data,
         device=device,
@@ -152,14 +162,14 @@ def parse_binary(
         show_calc=show_calc,
         show_diff=show_diff,
     )
-    _store_output(yaml_data, output)
+    _store_output(yaml_data, output, msg=f"Success. (PFR: {binary} has been parsed.")
 
 
 def _parse_binary_data(
     data: bytes,
     device: str,
     area: str,
-    revision: str = None,
+    revision: Optional[str] = None,
     show_calc: bool = False,
     show_diff: bool = False,
 ) -> str:
@@ -176,11 +186,7 @@ def _parse_binary_data(
     pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
     pfr_obj.parse(data)
     parsed = pfr_obj.get_yaml_config(exclude_computed=not show_calc, diff=show_diff)
-    yaml = YAML()
-    yaml.indent(sequence=4, offset=2)
-    stream = io.StringIO()
-    yaml.dump(parsed, stream)
-    yaml_data = stream.getvalue()
+    yaml_data = ConfigTemplate.convert_cm_to_yaml(parsed)
     return yaml_data
 
 
@@ -210,7 +216,7 @@ def _parse_binary_data(
 @click.option(
     "-o",
     "--output",
-    type=click.Path(dir_okay=False),
+    type=click.Path(dir_okay=False, resolve_path=True),
     required=True,
     help="Save the output into a file instead of console",
 )
@@ -250,12 +256,15 @@ def generate_binary(
         raise SPSDKPfrConfigError(
             f"The configuration file is not valid. The reason is: {invalid_reason}"
         )
-    assert pfr_config.type
+    area = pfr_config.type
+    assert area
+    pfr_obj = _get_pfr_class(area)(device=pfr_config.device, revision=pfr_config.revision)
+    pfr_obj.set_config(pfr_config, raw=not calc_inverse)
     if pfr_config.device in Pfrc.get_supported_families():
         try:
             pfrc = Pfrc(
-                cmpa=pfr_config if pfr_config.type.upper() == "CMPA" else None,
-                cfpa=pfr_config if pfr_config.type.upper() == "CFPA" else None,
+                cmpa=pfr_obj if area.lower() == "cmpa" else None,  # type: ignore
+                cfpa=pfr_obj if area.lower() == "cfpa" else None,  # type: ignore
             )
             rules = pfrc.validate_brick_conditions()
         except (SPSDKPfrConfigError, SPSDKPfrError) as e:
@@ -279,24 +288,20 @@ def generate_binary(
         root_of_trust = tuple((find_file(x, search_paths=[elf2sb_config_dir]) for x in public_keys))
     if secret_file:
         root_of_trust = secret_file
-    area = pfr_config.type
     if area.lower() == "cmpa" and root_of_trust:
         keys = extract_public_keys(root_of_trust, password)
-    pfr_obj = _get_pfr_class(area)(device=pfr_config.device, revision=pfr_config.revision)
     if not pfr_config.revision:
         pfr_config.revision = pfr_obj.revision
-    pfr_obj.set_config(pfr_config, raw=not calc_inverse)
-
     data = pfr_obj.export(add_seal=add_seal, keys=keys)
-    _store_output(data, output, "wb")
+    _store_output(data, output, "wb", msg="Success. (PFR binary has been generated)")
 
 
 @main.command(name="info", no_args_is_help=True)
-@pfr_device_type_options
+@pfr_device_type_options()
 @click.option(
     "-o",
     "--output",
-    type=click.Path(dir_okay=False),
+    type=click.Path(dir_okay=False, resolve_path=True),
     required=True,
     help="Save the output into a file instead of console",
 )
@@ -316,7 +321,7 @@ def info(device: str, revision: str, area: str, output: str, open_result: bool)
         regs_exclude=["SHA256_DIGEST"],
         fields_exclude=["FIELD"],
     )
-    _store_output(html_output, output)
+    _store_output(html_output, output, msg="Success. (PFR info HTML page has been generated)")
     if open_result:  # pragma: no cover # can't test opening the html document
         click.launch(f"{output}")
 
@@ -328,8 +333,15 @@ def devices() -> None:
 
 
 @main.command(name="write", no_args_is_help=True)
-@isp_interfaces(uart=True, usb=True, lpcusbsio=True, buspal=True, json_option=False)
-@pfr_device_type_options
+@isp_interfaces(
+    uart=True,
+    usb=True,
+    lpcusbsio=True,
+    buspal=True,
+    json_option=False,
+    use_long_timeout_option=True,
+)
+@pfr_device_type_options()
 @click.option(
     "-b",
     "--binary",
@@ -349,19 +361,13 @@ def write(
     binary: str,
 ) -> None:
     """Write PFR page to the device."""
-    # silence warning about missing revision in 'normal' logging mode
-    if not logger.isEnabledFor(logging.INFO):
-        pfr_logger = logging.getLogger("spsdk.pfr")
-        pfr_logger.setLevel(level=logging.ERROR)
-
     pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
     pfr_page_address = pfr_obj.config.get_address(device)
     pfr_page_length = pfr_obj.BINARY_SIZE
 
     click.echo(f"{pfr_obj.__class__.__name__} page address on {device} is {pfr_page_address:#x}")
 
-    with open(binary, "rb") as f:
-        data = f.read()
+    data = load_binary(binary)
     if len(data) != pfr_page_length:
         raise SPSDKError(
             f"PFR page length is {pfr_page_length}. Provided binary has {size_fmt(len(data))}."
@@ -379,19 +385,26 @@ def write(
 
 
 @main.command(name="read", no_args_is_help=True)
-@isp_interfaces(uart=True, usb=True, lpcusbsio=True, buspal=True, json_option=False)
-@pfr_device_type_options
+@isp_interfaces(
+    uart=True,
+    usb=True,
+    lpcusbsio=True,
+    buspal=True,
+    json_option=False,
+    use_long_timeout_option=True,
+)
+@pfr_device_type_options()
 @click.option(
     "-o",
     "--output",
-    type=click.Path(dir_okay=False),
+    type=click.Path(dir_okay=False, resolve_path=True),
     help="Store PFR data into a file. If not specified hexdump data into stdout.",
 )
 @click.option(
     "-y",
     "--yaml",
     "yaml_output",
-    type=click.Path(dir_okay=False),
+    type=click.Path(dir_okay=False, resolve_path=True),
     help="Parse data read from device into YAML config.",
 )
 @click.option(
@@ -422,11 +435,6 @@ def read(
     show_calc: bool,
 ) -> None:
     """Read PFR page from the device."""
-    # silence warning about missing revision in 'normal' logging mode
-    if not logger.isEnabledFor(logging.INFO):
-        pfr_logger = logging.getLogger("spsdk.pfr")
-        pfr_logger.setLevel(level=logging.ERROR)
-
     pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
     pfr_page_address = pfr_obj.config.get_address(device)
     pfr_page_length = pfr_obj.BINARY_SIZE
@@ -461,6 +469,41 @@ def read(
         click.echo(format_raw_data(data=data, use_hexdump=True))
 
 
+@main.command(name="erase-cmpa", no_args_is_help=True)
+@isp_interfaces(
+    uart=True,
+    usb=True,
+    lpcusbsio=True,
+    buspal=True,
+    json_option=False,
+    use_long_timeout_option=True,
+)
+@pfr_device_type_options(no_type=True)
+def erase_cmpa(
+    port: str,
+    usb: str,
+    buspal: str,
+    lpcusbsio: str,
+    timeout: int,
+    device: str,
+    revision: str,
+) -> None:
+    """Erase CMPA PFR page in the device if is not sealed."""
+    pfr_obj = _get_pfr_class("cmpa")(device=device, revision=revision)
+    pfr_page_address = pfr_obj.config.get_address(device)
+    pfr_page_length = pfr_obj.BINARY_SIZE
+
+    click.echo(f"CMPA page address on {device} is {pfr_page_address:#x}")
+
+    interface = get_interface(
+        module="mboot", port=port, usb=usb, buspal=buspal, lpcusbsio=lpcusbsio, timeout=timeout
+    )
+    assert isinstance(interface, MBootInterface)
+    with McuBoot(device=interface) as mboot:
+        mboot.write_memory(address=pfr_page_address, data=bytes(pfr_page_length))
+    click.echo(f"CMPA page {'has been erased.' if mboot.status_code == 0 else 'erase failed!'}")
+
+
 @catch_spsdk_error
 def safe_main() -> None:
     """Call the main function."""
