@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -20,7 +20,9 @@ from spsdk import __version__ as spsdk_version
 from spsdk.apps.utils.common_cli_options import CommandsTreeGroup
 
 from .blhost import main as blhost_main
+from .dk6prog import main as dk6prog_main
 from .elftosb import main as elftosb_main
+from .ifr import main as ifr_main
 from .nxpcertgen import main as nxpcertgen_main
 from .nxpcrypto import main as nxpcrypto_main
 from .nxpdebugmbox import main as nxpdebugmbox_main
@@ -51,6 +53,7 @@ def main() -> int:
 
 main.add_command(blhost_main, name="blhost")
 main.add_command(elftosb_main, name="elftosb")
+main.add_command(ifr_main, name="ifr")
 main.add_command(nxpcertgen_main, name="nxpcertgen")
 main.add_command(nxpcrypto_main, name="nxpcrypto")
 main.add_command(nxpdebugmbox_main, name="nxpdebugmbox")
@@ -62,6 +65,8 @@ main.add_command(pfr_main, name="pfr")
 main.add_command(sdphost_main, name="sdphost")
 main.add_command(sdpshost_main, name="sdpshost")
 main.add_command(shadowregs_main, name="shadowregs")
+main.add_command(dk6prog_main, name="dk6prog")
+
 if TP:
     main.add_command(tpconfig_main, name="tpconfig")
     main.add_command(tphost_main, name="tphost")
