@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -48,6 +48,19 @@ _usb_option = click.option(
 """,
 )
 
+_sdio_option = click.option(
+    "-sd",
+    "--sdio",
+    metavar="SDIO_PATH|DEV_NAME",
+    help="""SDIO device identifier.
+
+    \b
+    Following formats are supported: device/instance path, device name.
+    device/instance path: device string; e.g. /dev/mcu-sdio.
+    Use 'nxpdevscan' utility to list connected device names.
+""",
+)
+
 _lpcusbsio_option = click.option(
     "-l",
     "--lpcusbsio",
@@ -94,13 +107,23 @@ _json_option = click.option(
     help="Use JSON output",
 )
 
-_timeout_option = click.option(
-    "-t",
-    "--timeout",
-    metavar="<ms>",
-    help="""Sets timeout when waiting on data over a serial line. The default is 5000 milliseconds.""",
-    default=5000,
-)
+
+def _timeout_option(use_long_option: bool, timeout: int) -> Callable[[FC], FC]:
+    """Get the timeout option.
+
+    :param use_long_option: Use long version only
+    :param timeout: Default timeout in miliseconds
+
+    :return: click decorator
+    """
+    options = [] if use_long_option else ["-t"]
+    options.append("--timeout")
+    return click.option(
+        *options,
+        metavar="<ms>",
+        help=f"""Sets timeout when waiting on data over a serial line. The default is {timeout} milliseconds.""",
+        default=timeout,
+    )
 
 
 def spsdk_apps_common_options(options: FC) -> FC:
@@ -129,24 +152,42 @@ def spsdk_apps_common_options(options: FC) -> FC:
     return options
 
 
+def spsdk_plugin_option(options: FC) -> FC:
+    """Plugin click option decorator.
+
+    :return: Click decorator
+    """
+    return click.option(
+        "--plugin",
+        required=False,
+        help="External python file/package containing a custom plugin implementation.",
+    )(options)
+
+
 def isp_interfaces(
     uart: bool = False,
     usb: bool = False,
+    sdio: bool = False,
     lpcusbsio: bool = False,
     buspal: bool = False,
     json_option: bool = True,
     timeout_option: bool = True,
     is_sdp: bool = False,
+    use_long_timeout_option: bool = False,
+    default_timeout: int = 5000,
 ) -> Callable:
     """Interfaces Click CLI options.
 
     :param uart: UART interface, defaults to False
     :param usb: USB interface, defaults to False
+    :param sdio: SDIO interface, defaults to False
     :param lpcusbsio: LPCUSBSIO interface, defaults to False
     :param buspal: BUSPAL interface, defaults to False
     :param json_option: add -j option, defaults to True
     :param timeout_option: add timeout option, defaults to True
     :param is_sdp: Specifies whether the ISP interface is meant for SDP(S) protocol
+    :param use_long_timeout_option: Use only the long form for timeout (--timeout)
+    :param default_timeout: Default timeout to be set when getting the timeout_option
     :return: click decorator
     """
 
@@ -157,6 +198,8 @@ def isp_interfaces(
             options.append(_sdp_uart_option if is_sdp else _uart_option)
         if usb:
             options.append(_usb_option)
+        if sdio:
+            options.append(_sdio_option)
         if lpcusbsio:
             options.append(_lpcusbsio_option)
         if buspal:
@@ -164,8 +207,8 @@ def isp_interfaces(
         if json_option:
             options.append(_json_option)
         if timeout_option:
-            options.append(_timeout_option)
-
+            option = _timeout_option(use_long_timeout_option, default_timeout)
+            options.append(option)
         for option in reversed(options):
             func = option(func)
 
@@ -203,6 +246,7 @@ class GroupAliasedGetCfgTemplate(click.Group):
     better user experience.
     """
 
+    # pylint: disable=inconsistent-return-statements    # ctx.fail at the end terminates the command call
     def get_command(self, ctx: click.Context, cmd_name: str) -> Optional[click.Command]:
         """Override original click get_command function to implement alias for get-cfg-template obsolete command.
 
@@ -210,9 +254,9 @@ class GroupAliasedGetCfgTemplate(click.Group):
         :param cmd_name: Requested command name
         :return: Suitable command representation.
         """
-        rv = click.Group.get_command(self, ctx, cmd_name)
-        if rv is not None:
-            return rv
+        command = click.Group.get_command(self, ctx, cmd_name)
+        if command is not None:
+            return command
         if cmd_name == "get-cfg-template":
             click.secho(
                 "The 'get-cfg-template' is deprecated command, use 'get-template' instead of.",
