@@ -1,19 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-"""Module with Debugcredential class."""
+"""Module with DebugCredential class."""
 
+import math
 from struct import calcsize, pack, unpack_from
-from typing import Any, List, Type
+from typing import Any, List, Optional, Type
 
 from spsdk import SPSDKError, crypto
-from spsdk.crypto import SignatureProvider
 from spsdk.crypto.loaders import extract_public_key
+from spsdk.crypto.signature_provider import SignatureProvider, get_signature_provider
 from spsdk.dat.utils import ecc_key_to_bytes, ecc_public_numbers_to_bytes, rsa_key_to_bytes
+from spsdk.exceptions import SPSDKValueError
+from spsdk.image.ahab.ahab_container import SRKRecord, SRKTable
 from spsdk.utils.crypto.backend_internal import internal_backend
 from spsdk.utils.misc import find_file
 
@@ -24,9 +27,21 @@ class DebugCredential:
     # Subclasses override the following invalid class member values
     FORMAT = "INVALID_FORMAT"
     FORMAT_NO_SIG = "INVALID_FORMAT"
+    SOCC_FORMAT = ""
     VERSION = "0.0"
     HASH_LENGTH = 32
 
+    SOCC_LIST = {
+        0x0000: "i.MXRT595, i.MXRT685",
+        0x0001: "LPC550x, LPC55s0x, LPC551x, LPC55s1x, LPC552x, LPC55s2x, LPC55s6",
+        0x0004: "LPC55s3",
+        0x0005: "KW45xx/K32W1xx",
+        0x0006: "MCXN9xx (A0)",
+        0x0007: "MCXN9xx (A1)",
+        0x000A: "RW61x (A2)",
+        0x5254049C: "i.MXRT118x",
+    }
+
     def __init__(
         self,
         socc: int,
@@ -37,8 +52,8 @@ class DebugCredential:
         cc_vu: int,
         cc_beacon: int,
         rot_pub: bytes,
-        signature: bytes = None,
-        signature_provider: SignatureProvider = None,
+        signature: Optional[bytes] = None,
+        signature_provider: Optional[SignatureProvider] = None,
     ) -> None:
         """Initialize the DebugCredential object.
 
@@ -88,17 +103,29 @@ class DebugCredential:
         )
         return data
 
+    @staticmethod
+    def get_socc_description(version: str, socc: int) -> str:
+        """Get SOCC family name description.
+
+        :param version: Protocol version
+        :param socc: SOCC number
+        :return: SOCC string representation
+        """
+        cls = DebugCredential._get_class(version, socc)
+        return f"{socc:{cls.SOCC_FORMAT}}, " + cls.SOCC_LIST.get(socc, "Unknown SOCC")
+
     def info(self) -> str:
         """String representation of DebugCredential.
 
         :return: binary representation of the debug credential
         """
         msg = f"Version : {self.VERSION}\n"
-        msg += f"SOCC    : {self.socc}\n"
+        msg += f"SOCC    : {self.get_socc_description(self.VERSION, self.socc)}\n"
         msg += f"UUID    : {self.uuid.hex().upper()}\n"
         msg += f"CC_SOCC : {hex(self.cc_socu)}\n"
         msg += f"CC_VU   : {hex(self.cc_vu)}\n"
         msg += f"BEACON  : {self.cc_beacon}\n"
+        msg += f"RoTKH   : {self.get_rotkh().hex()}\n"
         return msg
 
     def sign(self) -> None:
@@ -129,8 +156,8 @@ class DebugCredential:
     def __eq__(self, other: Any) -> bool:
         return isinstance(other, DebugCredential) and vars(self) == vars(other)
 
-    @staticmethod
-    def _get_rot_meta(used_root_cert: int, rot_pub_keys: List[str]) -> bytes:
+    @classmethod
+    def _get_rot_meta(cls, used_root_cert: int, rot_pub_keys: List[str]) -> bytes:
         """Creates the RoT meta-data required by the device to corroborate.
 
         :return: binary representing the rot-meta data
@@ -156,15 +183,23 @@ class DebugCredential:
         """
         raise NotImplementedError("Derived class has to implement this method.")
 
+    def get_rotkh(self) -> bytes:
+        """Get Root Of Trust Keys Hash.
+
+        :return: RoTKH in bytes
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
+
     @classmethod
     def _get_class(cls, version: str, socc: int) -> "Type[DebugCredential]":
-        if socc == 4:
-            return _lpc55s3x_version_mapping[version]
+        if socc in [0x5254049C]:
+            return _edge_lock_version_mapping[version]
         return _version_mapping[version]
 
     @classmethod
     def create_from_yaml_config(
-        cls, version: str, yaml_config: dict, search_paths: List[str] = None
+        cls, version: str, yaml_config: dict, search_paths: Optional[List[str]] = None
     ) -> "DebugCredential":
         """Create a debug credential object out of yaml configuration.
 
@@ -182,6 +217,11 @@ class DebugCredential:
         if "rotk" in yaml_config.keys():
             yaml_config["rotk"] = find_file(yaml_config["rotk"], search_paths=search_paths)
         yaml_config["dck"] = find_file(yaml_config["dck"], search_paths=search_paths)
+        signature_provider = get_signature_provider(
+            sp_cfg=yaml_config.get("sign_provider"),
+            local_file_key=yaml_config.get("rotk"),
+            search_paths=search_paths,
+        )
         dc_obj = klass(
             socc=yaml_config["socc"],
             uuid=bytes.fromhex(yaml_config["uuid"]),
@@ -196,11 +236,7 @@ class DebugCredential:
             rot_pub=klass._get_rot_pub(  # pylint: disable=protected-access
                 yaml_config["rot_id"], yaml_config["rot_meta"]
             ),
-            signature_provider=SignatureProvider.create(
-                # if the yaml_config doesn't contain 'sign_provider' assume file-type
-                yaml_config.get("sign_provider")
-                or f'type=file;file_path={yaml_config["rotk"]}'
-            ),
+            signature_provider=signature_provider,
         )
         return dc_obj
 
@@ -234,8 +270,8 @@ class DebugCredentialRSA(DebugCredential):
     FORMAT_NO_SIG = "<2HL16s128s260s3L260s"
     FORMAT = FORMAT_NO_SIG + "256s"
 
-    @staticmethod
-    def _get_rot_meta(used_root_cert: int, rot_pub_keys: List[str]) -> bytes:
+    @classmethod
+    def _get_rot_meta(cls, used_root_cert: int, rot_pub_keys: List[str]) -> bytes:
         """Creates the RoT meta-data required by the device to corroborate.
 
         The meta-data is created by getting the public numbers (modulus and exponent)
@@ -258,7 +294,7 @@ class DebugCredentialRSA(DebugCredential):
 
         :return: binary representing the DCK key
         """
-        dck_key = crypto.load_public_key(file_path=dck_key_path)
+        dck_key = extract_public_key(file_path=dck_key_path)
         assert isinstance(dck_key, crypto.RSAPublicKey)
         return rsa_key_to_bytes(key=dck_key, exp_length=4)
 
@@ -276,14 +312,22 @@ class DebugCredentialRSA(DebugCredential):
         assert isinstance(pub_key, crypto.RSAPublicKey)
         return rsa_key_to_bytes(key=pub_key, exp_length=4)
 
+    def get_rotkh(self) -> bytes:
+        """Get Root Of Trust Keys Hash.
+
+        :return: RoTKH in bytes
+        """
+        return internal_backend.hash(data=self.rot_meta[:])
+
 
 class DebugCredentialECC(DebugCredential):
     """Class for ECC specific of DebugCredential."""
 
-    FORMAT_NO_SIG = "<2HL16s528s132s3L4s"
-    FORMAT = FORMAT_NO_SIG + "132s"
-    CURVE: Any = crypto.ec.SECP256R1()
-    CORD_LENGTH = 66
+    HASH_LENGTH = 0
+    KEY_LENGTH = 0
+    CORD_LENGTH = 0
+    HASH_SIZES = {32: 256, 48: 384, 66: 512}
+    CURVE: crypto.ec.EllipticCurve = crypto.ec.SECP256R1()
 
     def sign(self) -> None:
         """Sign the DC data using SignatureProvider."""
@@ -296,104 +340,14 @@ class DebugCredentialECC(DebugCredential):
             public_numbers=public_numbers, length=self.CORD_LENGTH
         )
 
-    @staticmethod
-    def _get_rot_meta(used_root_cert: int, rot_pub_keys: List[str]) -> bytes:
-        """Creates the RoT meta-data required by the device to corroborate.
-
-        The meta-data is created by getting the public numbers (modulus and exponent)
-        from each of the RoT public keys, hashing them and combing together.
-
-        :return: binary representing the rot-meta data
-        """
-        rot_meta = bytearray(528)
-        for index, rot_key in enumerate(rot_pub_keys):
-            rot = extract_public_key(file_path=rot_key, password=None)
-            assert isinstance(rot, crypto.EllipticCurvePublicKey)
-            data = ecc_key_to_bytes(key=rot, length=66)
-            rot_meta[index * 132 : (index + 1) * 132] = data
-        return bytes(rot_meta)
-
-    @staticmethod
-    def _get_dck(dck_key_path: str) -> bytes:
-        """Loads the Debugger Public Key (DCK).
-
-        :return: binary representing the DCK key
-        """
-        dck_key = crypto.load_public_key(file_path=dck_key_path)
-        assert isinstance(dck_key, crypto.EllipticCurvePublicKey)
-        return ecc_key_to_bytes(key=dck_key, length=66)
-
-    @staticmethod
-    def _get_rot_pub(rot_pub_id: int, rot_pub_keys: List[str]) -> bytes:
-        """Creates RoTKey_Pub (2 element 16-bit array (little endian).
-
-        CTRK table index (RoT meta-data) of the public key used by the vendor to sign the DC.
-        Curve identifier:
-        - Secp256r1: 0x0001
-        - Secp384r1: 0x0002
-        - Secp521r1: 0x0003
-
-        :return: binary representation
-        """
-        pub_key_path = rot_pub_keys[rot_pub_id]
-        pub_key = extract_public_key(file_path=pub_key_path, password=None)
-        assert isinstance(pub_key, crypto.EllipticCurvePublicKey)
-        curve_index = {256: 1, 384: 2, 521: 3}[pub_key.curve.key_size]
-        return pack("<2H", rot_pub_id, curve_index)
-
-
-class DebugCredentialRSA2048(DebugCredentialRSA):
-    """DebugCredential class for RSA 2048."""
-
-    FORMAT_NO_SIG = "<2HL16s128s260s3L260s"
-    FORMAT = FORMAT_NO_SIG + "256s"
-    VERSION = "1.0"
-
-
-class DebugCredentialRSA4096(DebugCredentialRSA):
-    """DebugCredential class for RSA 4096."""
-
-    FORMAT_NO_SIG = "<2HL16s128s516s3L516s"
-    FORMAT = FORMAT_NO_SIG + "512s"
-    VERSION = "1.1"
-
-
-class DebugCredentialECC256(DebugCredentialECC):
-    """DebugCredential class for ECC 256."""
-
-    VERSION = "2.0"
-    CURVE = crypto.ec.SECP256R1()
-
-
-class DebugCredentialECC384(DebugCredentialECC):
-    """DebugCredential class for ECC 384."""
-
-    VERSION = "2.1"
-    CURVE = crypto.ec.SECP384R1()
-
-
-class DebugCredentialECC521(DebugCredentialECC):
-    """DebugCredential class for ECC 521."""
-
-    VERSION = "2.2"
-    CURVE = crypto.ec.SECP521R1()
-
-
-class Lpc55s3xMixin(DebugCredentialECC):
-    """LPC55s3x Class."""
-
-    HASH_LENGTH = 0
-    KEY_LENGTH = 0
-    CORD_LENGTH = 0
-
-    @staticmethod
-    def _get_rot_meta(used_root_cert: int, rot_pub_keys: List[str]) -> bytes:
+    @classmethod
+    def _get_rot_meta(cls, used_root_cert: int, rot_pub_keys: List[str]) -> bytes:
         """Creates the RoT meta-data required by the device to corroborate.
 
         :return: binary representing the rot-meta data
         """
-        ctrk_hash_table = Lpc55s3xMixin.create_ctrk_table(rot_pub_keys)
-        flags = Lpc55s3xMixin.calculate_flags(used_root_cert, rot_pub_keys)
+        ctrk_hash_table = DebugCredentialECC.create_ctrk_table(rot_pub_keys)
+        flags = DebugCredentialECC.calculate_flags(used_root_cert, rot_pub_keys)
         return flags + ctrk_hash_table
 
     @staticmethod
@@ -402,8 +356,8 @@ class Lpc55s3xMixin(DebugCredentialECC):
 
         :return: binary representing the DCK key
         """
-        dck_key = crypto.load_public_key(file_path=dck_key_path)
-        length = dck_key.key_size // 8
+        dck_key = extract_public_key(file_path=dck_key_path)
+        length = math.ceil(dck_key.key_size / 8)
         assert isinstance(dck_key, crypto.EllipticCurvePublicKey)
         data = ecc_key_to_bytes(dck_key, length=length)
         return data
@@ -416,7 +370,7 @@ class Lpc55s3xMixin(DebugCredentialECC):
         """
         root_key = rot_pub_keys[rot_pub_id]
         root_public_key = extract_public_key(file_path=root_key, password=None)
-        length = root_public_key.key_size // 8
+        length = math.ceil(root_public_key.key_size / 8)
         assert isinstance(root_public_key, crypto.EllipticCurvePublicKey)
         data = ecc_key_to_bytes(root_public_key, length=length)
         return data
@@ -427,7 +381,7 @@ class Lpc55s3xMixin(DebugCredentialECC):
         :return: binary representation of the debug credential
         """
         msg = f"Version : {self.VERSION}\n"
-        msg += f"SOCC    : {self.socc}\n"
+        msg += f"SOCC    : {self.get_socc_description(self.VERSION, self.socc)}\n"
         msg += f"UUID    : {self.uuid.hex().upper()}\n"
         msg += f"CC_SOCC : {hex(self.cc_socu)}\n"
         msg += f"CC_VU   : {hex(self.cc_vu)}\n"
@@ -437,10 +391,7 @@ class Lpc55s3xMixin(DebugCredentialECC):
             msg += "CRTK table not present \n"
         else:
             msg += f"CRTK table has {ctrk_records_num} entries\n"
-            # Compute and show RKTH HASH
-            key_length = 256 if (len(self.rot_meta) - 4) == 32 else 384
-            ctrk_hash = internal_backend.hash(data=self.rot_meta[4:], algorithm=f"sha{key_length}")
-            msg += f"CRTK Hash: {ctrk_hash.hex()}"
+            msg += f"CRTK Hash: {self.get_rotkh().hex()}"
         return msg
 
     @property
@@ -462,9 +413,9 @@ class Lpc55s3xMixin(DebugCredentialECC):
         for pub_key_path in rot_pub_keys:
             pub_key = extract_public_key(file_path=pub_key_path, password=None)
             assert isinstance(pub_key, crypto.EllipticCurvePublicKey)
-            key_length = pub_key.key_size
-            data = ecc_key_to_bytes(key=pub_key, length=key_length // 8)
-            ctrk_hash = internal_backend.hash(data=data, algorithm=f"sha{key_length}")
+            hash_size = DebugCredentialECC.HASH_SIZES[math.ceil(pub_key.key_size / 8)]
+            data = ecc_key_to_bytes(key=pub_key, length=math.ceil(pub_key.key_size / 8))
+            ctrk_hash = internal_backend.hash(data=data, algorithm=f"sha{hash_size}")
             ctrk_table += ctrk_hash
         return ctrk_table
 
@@ -566,6 +517,17 @@ class Lpc55s3xMixin(DebugCredentialECC):
             signature=signature,
         )
 
+    def get_rotkh(self) -> bytes:
+        """Get Root Of Trust Keys Hash.
+
+        :return: RoTKH in bytes
+        """
+        srk_records_num = self.rot_meta[0] >> 4
+        if srk_records_num == 1:
+            return b""
+        key_length = 256 if ((len(self.rot_meta) - 4) // srk_records_num) == 32 else 384
+        return internal_backend.hash(data=self.rot_meta[4:], algorithm=f"sha{key_length}")
+
     @classmethod
     def get_instance_from_challenge(cls, data: bytes) -> "DebugCredential":
         """Returns instance of class from DAP authentication challenge data.
@@ -575,22 +537,260 @@ class Lpc55s3xMixin(DebugCredentialECC):
         return cls.parse(data, 0)
 
 
-class DebugCredentialECC256Lpc55s3x(Lpc55s3xMixin):
+class DebugCredentialEdgeLockEnclave(DebugCredentialECC):
+    """EdgeLock Class."""
+
+    HASH_LENGTH = 0
+    KEY_LENGTH = 0
+    CORD_LENGTH = 0
+    SOCC_FORMAT = "08X"
+
+    @classmethod
+    def _get_rot_meta(cls, used_root_cert: int, rot_pub_keys: List[str]) -> bytes:
+        """Creates the RoT meta-data required by the device to corroborate.
+
+        :return: binary representing the rot-meta data
+        """
+        srk_hash_table = DebugCredentialEdgeLockEnclave.create_srk_table(rot_pub_keys)
+        flags = DebugCredentialECC.calculate_flags(used_root_cert, rot_pub_keys)
+        return flags + srk_hash_table
+
+    @staticmethod
+    def _get_dck(dck_key_path: str) -> bytes:
+        """Loads the Debugger Public Key (DCK).
+
+        :return: binary representing the DCK key
+        """
+        dck_key = extract_public_key(file_path=dck_key_path)
+        length = math.ceil(dck_key.key_size / 8)
+        assert isinstance(dck_key, crypto.EllipticCurvePublicKey)
+        data = ecc_key_to_bytes(dck_key, length=length)
+        return data
+
+    @staticmethod
+    def _get_rot_pub(rot_pub_id: int, rot_pub_keys: List[str]) -> bytes:
+        """Loads the vendor RoT Public key that corresponds to the private key used for singing.
+
+        :return: binary representing the rotk public key
+        """
+        return DebugCredentialECC._get_rot_pub(rot_pub_id=rot_pub_id, rot_pub_keys=rot_pub_keys)
+
+    def get_rotkh(self) -> bytes:
+        """Get Root Of Trust Keys Hash.
+
+        :return: RoTKH in bytes
+        """
+        srk = SRKTable.parse(self.rot_meta[4:])
+        srk.update_fields()
+        return srk.compute_srk_hash()
+
+    def info(self) -> str:
+        """String representation of DebugCredential.
+
+        :return: binary representation of the debug credential
+        """
+        msg = f"Version : {self.VERSION}\n"
+        msg += f"SOCC    : {self.get_socc_description(self.VERSION, self.socc)}\n"
+        msg += f"UUID    : {self.uuid.hex().upper()}\n"
+        msg += f"CC_SOCC : {hex(self.cc_socu)}\n"
+        msg += f"CC_VU   : {hex(self.cc_vu)}\n"
+        msg += f"BEACON  : {self.cc_beacon}\n"
+        srk_records_num = self.rot_meta[0] >> 4
+        if srk_records_num != 4:
+            msg += "Invalid count of SRK records \n"
+        else:
+            msg += f"SRK table has {srk_records_num} entries\n"
+            msg += f"SRK Hash: {self.get_rotkh().hex()}"
+        return msg
+
+    @property
+    def FORMAT(self) -> str:  # type: ignore # pylint: disable=invalid-name
+        """Formatting string."""
+        return f"<2HL16s3L{len(self.rot_meta)}s{self.HASH_LENGTH * 2}s{self.HASH_LENGTH * 2}s"
+
+    @property
+    def FORMAT_NO_SIG(self) -> str:  # type: ignore # pylint: disable=invalid-name
+        """Formatting string without signature."""
+        return f"<2HL16s3L{len(self.rot_meta)}s{self.HASH_LENGTH * 2}s"
+
+    @staticmethod
+    def create_srk_table(rot_pub_keys: List[str]) -> bytes:
+        """Creates ctrk table."""
+        if len(rot_pub_keys) != 4:
+            raise SPSDKValueError("Invalid count of Super Root keys!")
+
+        srk_table = SRKTable(
+            [SRKRecord.create_from_key(extract_public_key(x)) for x in rot_pub_keys]
+        )
+        srk_table.update_fields()
+        srk_table.validate({})
+        return srk_table.export()
+
+    def export(self) -> bytes:
+        """Export to binary form (serialization)."""
+        data = pack(
+            self.FORMAT,
+            *[int(v) for v in self.VERSION.split(".")],
+            self.socc,
+            self.uuid,
+            self.cc_socu,
+            self.cc_vu,
+            self.cc_beacon,
+            self.rot_meta,
+            self.dck_pub,
+            self.signature,
+        )
+        return data
+
+    def _get_data_to_sign(self) -> bytes:
+        """Collects data meant for signing."""
+        data = pack(
+            self.FORMAT_NO_SIG,
+            *[int(v) for v in self.VERSION.split(".")],
+            self.socc,
+            self.uuid,
+            self.cc_socu,
+            self.cc_vu,
+            self.cc_beacon,
+            self.rot_meta,
+            self.dck_pub,
+        )
+        return data
+
+    def __eq__(self, other: Any) -> bool:
+        self_vars = vars(self)
+        del self_vars["signature_provider"]
+        other_vars = vars(other)
+        del other_vars["signature_provider"]
+        return isinstance(other, DebugCredential) and other_vars == self_vars
+
+    @classmethod
+    def parse(cls, data: bytes, offset: int = 0) -> "DebugCredential":
+        """Parse the debug credential.
+
+        :param data: Raw data as bytes
+        :param offset: Offset of input data
+        :return: DebugCredential object
+        :raises SPSDKError: When flag is invalid
+        """
+        format_head = "<2HL16s4L"
+        (
+            version_major,  # pylint: disable=unused-variable
+            version_minor,  # pylint: disable=unused-variable
+            socc,
+            uuid,
+            cc_socu,
+            cc_vu,
+            beacon,
+            flags,
+        ) = unpack_from(format_head, data)
+        if not flags & 0x8000_0000:
+            raise SPSDKError("Invalid flag")
+
+        srk_table = SRKTable.parse(data, offset=offset + calcsize(format_head))
+        srk_table.update_fields()
+        srk_table.validate({})
+        rot_meta = int.to_bytes(flags, 4, "little") + srk_table.export()
+        format_tail = f"<{cls.HASH_LENGTH * 2}s{cls.HASH_LENGTH * 2}s"
+        dck_pub, signature = unpack_from(
+            format_tail, data, offset + calcsize(format_head) + len(srk_table)
+        )
+
+        return cls(
+            socc=socc,
+            uuid=uuid,
+            rot_meta=rot_meta,
+            dck_pub=dck_pub,
+            rot_pub=bytes(),
+            cc_socu=cc_socu,
+            cc_vu=cc_vu,
+            cc_beacon=beacon,
+            signature=signature,
+        )
+
+    @classmethod
+    def get_instance_from_challenge(cls, data: bytes) -> "DebugCredential":
+        """Returns instance of class from DAP authentication challenge data.
+
+        :return: Instance of this class.
+        """
+        return cls.parse(data, 0)
+
+
+class DebugCredentialRSA2048(DebugCredentialRSA):
+    """DebugCredential class for RSA 2048."""
+
+    FORMAT_NO_SIG = "<2HL16s128s260s3L260s"
+    FORMAT = FORMAT_NO_SIG + "256s"
+    VERSION = "1.0"
+
+
+class DebugCredentialRSA4096(DebugCredentialRSA):
+    """DebugCredential class for RSA 4096."""
+
+    FORMAT_NO_SIG = "<2HL16s128s516s3L516s"
+    FORMAT = FORMAT_NO_SIG + "512s"
+    VERSION = "1.1"
+
+
+class DebugCredentialECC256(DebugCredentialECC):
     """DebugCredential class for LPC55s3x for version 2.0 (p256)."""
 
+    VERSION = "2.0"
+    CURVE = crypto.ec.SECP256R1()
     HASH_LENGTH = 32
     CORD_LENGTH = 32
     KEY_LENGTH = 256
-    VERSION = "2.0"
 
 
-class DebugCredentialECC384Lpc55s3x(Lpc55s3xMixin):
+class DebugCredentialECC384(DebugCredentialECC):
     """DebugCredential class for LPC55s3x for version 2.1 (p384)."""
 
+    VERSION = "2.1"
+    CURVE = crypto.ec.SECP384R1()
     HASH_LENGTH = 48
     CORD_LENGTH = 48
     KEY_LENGTH = 384
+
+
+class DebugCredentialECC521(DebugCredentialECC):
+    """DebugCredential class for LPC55s3x for version 2.1 (p384)."""
+
+    VERSION = "2.2"
+    CURVE = crypto.ec.SECP521R1()
+    HASH_LENGTH = 66
+    CORD_LENGTH = 66
+    KEY_LENGTH = 521
+
+
+class DebugCredentialEdgeLockEnclaveECC256(DebugCredentialEdgeLockEnclave):
+    """Debug Credential class for device using EdgeLock peripheral for ECC256 keys."""
+
+    VERSION = "2.0"
+    CURVE = crypto.ec.SECP256R1()
+    HASH_LENGTH = 32
+    CORD_LENGTH = 32
+    KEY_LENGTH = 256
+
+
+class DebugCredentialEdgeLockEnclaveECC384(DebugCredentialEdgeLockEnclave):
+    """Debug Credential class for device using EdgeLock peripheral for ECC384 keys."""
+
     VERSION = "2.1"
+    CURVE = crypto.ec.SECP384R1()
+    HASH_LENGTH = 48
+    CORD_LENGTH = 48
+    KEY_LENGTH = 384
+
+
+class DebugCredentialEdgeLockEnclaveECC521(DebugCredentialEdgeLockEnclave):
+    """Debug Credential class for device using EdgeLock peripheral for ECC521 keys."""
+
+    VERSION = "2.2"
+    CURVE = crypto.ec.SECP521R1()
+    HASH_LENGTH = 66
+    CORD_LENGTH = 66
+    KEY_LENGTH = 521
 
 
 _version_mapping = {
@@ -601,7 +801,9 @@ _version_mapping = {
     "2.2": DebugCredentialECC521,
 }
 
-_lpc55s3x_version_mapping = {
-    "2.0": DebugCredentialECC256Lpc55s3x,
-    "2.1": DebugCredentialECC384Lpc55s3x,
+
+_edge_lock_version_mapping = {
+    "2.0": DebugCredentialEdgeLockEnclaveECC256,
+    "2.1": DebugCredentialEdgeLockEnclaveECC384,
+    "2.2": DebugCredentialEdgeLockEnclaveECC521,
 }
