@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -40,7 +40,7 @@ class DebugMailboxCommand:
         self.name = name
         self.delay = delay
 
-    def run(self, params: List[int] = None) -> List[Any]:
+    def run(self, params: Optional[List[int]] = None) -> List[Any]:
         """Run DebugMailboxCommand."""
         paramslen = len(params) if params else 0
         if paramslen != self.paramlen:
@@ -71,11 +71,12 @@ class DebugMailboxCommand:
         ret = self.dm.spin_read(self.dm.registers["RETURN"]["address"])
         logger.debug(f"-> spin_read:  {format_value(ret, 32)}")
 
-        new_protocol = bool(ret >> 31)
+        # bit 31 is flag in new protocol version
+        # new_protocol = bool(ret >> 31)
+        error_indication = bool(ret >> 20) and not bool(self.resplen)
         # solve the case that response is in legacy protocol and there is some
         # unwanted bits in none expected data. In this case return valid read data.
-        if not new_protocol and not ret & ~self.STATUS_IS_DATA_MASK:
-            logger.debug("The data is returned by legacy protocol.")
+        if not self.resplen and not error_indication:
             return [ret]
 
         resplen = (ret >> 16) & 0x7FFF
@@ -112,7 +113,7 @@ class DebugMailboxCommand:
             return self.run(**args)
         except (SPSDKTimeoutError, TimeoutError) as error:
             if raise_if_failure:
-                raise error
+                raise SPSDKTimeoutError("Timeout occurred") from error
             logger.error(str(error))
         return None
 
@@ -171,9 +172,9 @@ class EnterBlankDebugAuthentication(DebugMailboxCommand):
 class SetFaultAnalysisMode(DebugMailboxCommand):
     """Class for SetFaultAnalysisMode."""
 
-    def __init__(self, dm: DebugMailbox) -> None:
+    def __init__(self, dm: DebugMailbox, paramlen: int = 0) -> None:
         """Initialize."""
-        super().__init__(dm, id=6, name="SET_FA_MODE")
+        super().__init__(dm, id=6, name="SET_FA_MODE", paramlen=paramlen)
 
 
 class StartDebugSession(DebugMailboxCommand):
@@ -202,6 +203,24 @@ class DebugAuthenticationResponse(DebugMailboxCommand):
         super().__init__(dm, id=17, name="DBG_AUTH_RESP", paramlen=paramlen)
 
 
+class NxpDebugAuthenticationStart(DebugMailboxCommand):
+    """Class for DebugAuthenticationStart."""
+
+    def __init__(self, dm: DebugMailbox, resplen: int = 26) -> None:
+        """Initialize."""
+        # 26 words == 104 bytes (SHA256 - 32 Bytes)
+        # 30 words == 120 bytes (SHA384 - 48 Bytes)
+        super().__init__(dm, id=18, name="NXP_DBG_AUTH_START", resplen=resplen)
+
+
+class NxpDebugAuthenticationResponse(DebugMailboxCommand):
+    """Class for DebugAuthenticationResponse."""
+
+    def __init__(self, dm: DebugMailbox, paramlen: int) -> None:
+        """Initialize."""
+        super().__init__(dm, id=19, name="NXP_DBG_AUTH_RESP", paramlen=paramlen)
+
+
 class StartDebugSessions(DebugMailboxCommand):
     """Class for StartDebugSessions."""
 
