@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,5 +11,6 @@ import os
 from spsdk import SPSDK_DATA_FOLDER
 
 AHAB_DATA_FOLDER: str = os.path.join(SPSDK_DATA_FOLDER, "ahab")
-AHAB_SCH_FILE: str = os.path.join(AHAB_DATA_FOLDER, "sch_ahab.yml")
-AHAB_DATABASE_FILE: str = os.path.join(AHAB_DATA_FOLDER, "database.yml")
+AHAB_SCH_FILE: str = os.path.join(AHAB_DATA_FOLDER, "sch_ahab.yaml")
+AHAB_DATABASE_FILE: str = os.path.join(AHAB_DATA_FOLDER, "database.yaml")
+SIGNED_MSG_SCH_FILE: str = os.path.join(AHAB_DATA_FOLDER, "sch_signed_msg.yaml")
