@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -17,6 +17,7 @@ from asn1crypto import cms, util, x509
 
 from spsdk import SPSDKError
 from spsdk.crypto import Certificate, Encoding
+from spsdk.exceptions import SPSDKValueError
 from spsdk.utils.crypto import crypto_backend, matches_key_and_cert
 from spsdk.utils.easy_enum import Enum
 from spsdk.utils.misc import DebugInfo
@@ -267,7 +268,7 @@ class CmdWriteData(CmdBase):
     @ops.setter
     def ops(self, value: int) -> None:
         if value not in EnumWriteOps:
-            raise SPSDKError
+            raise SPSDKValueError("Value not defined")
         self._header.param &= ~(0x3 << 3)
         self._header.param |= int(value) << 3
 
@@ -275,7 +276,7 @@ class CmdWriteData(CmdBase):
         self,
         numbytes: int = 4,
         ops: int = EnumWriteOps.WRITE_VALUE,
-        data: Iterable[Tuple[int, int]] = None,
+        data: Optional[Iterable[Tuple[int, int]]] = None,
     ) -> None:
         """Initialize Write Data command.
 
@@ -626,7 +627,7 @@ class CmdInitialize(CmdBase):
             raise SPSDKError("Incorrect value of engine")
         self._header.param = value
 
-    def __init__(self, engine: int = EnumEngine.ANY, data: List[int] = None) -> None:
+    def __init__(self, engine: int = EnumEngine.ANY, data: Optional[List[int]] = None) -> None:
         """Initialize the initialize command."""
         if engine not in EnumEngine:
             raise SPSDKError("Incorrect value of engine")
@@ -1123,7 +1124,7 @@ class CmdInstallKey(CmdBase):
             return False
         return True
 
-    @property
+    @property  # type: ignore
     def cmd_data_reference(self) -> Optional[Union[CertificateImg, SrkTable]]:
         """Reference to an additional data (such as certificate, signature, etc).
 
@@ -1326,7 +1327,7 @@ class CmdAuthData(CmdBase):
         """
         self.location = value
 
-    @property
+    @property  # type: ignore
     def cmd_data_reference(self) -> Optional[SignatureOrMAC]:
         """Reference to an additional data (such as certificate, signature, etc).
 
@@ -1342,14 +1343,14 @@ class CmdAuthData(CmdBase):
         By default, the command does not support cmd_data_reference
 
         :param value: to be set
-        :raise ExpectedSignatureOrMACError: if unsupported data object is provided
+        :raises ExpectedSignatureOrMACError: if unsupported data object is provided
         """
         if self.sig_format == EnumCertFormat.AEAD:
             assert isinstance(value, MAC)
         elif self.sig_format == EnumCertFormat.CMS:
             assert isinstance(value, Signature)
         else:
-            raise ExpectedSignatureOrMACError()
+            raise ExpectedSignatureOrMACError("Unsupported data object is provided")
         self._signature = value
 
     def parse_cmd_data(self, data: bytes, offset: int) -> SignatureOrMAC:
@@ -1358,7 +1359,7 @@ class CmdAuthData(CmdBase):
         :param data: to be parsed
         :param offset: start position in data to parse
         :return: parsed data object; command-specific: Signature or MAC
-        :raise ExpectedSignatureOrMACError: if unsupported data object is provided
+        :raises ExpectedSignatureOrMACError: if unsupported data object is provided
         """
         header = Header.parse(data, offset)
         if header.tag == SegTag.MAC:
