@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -12,6 +12,7 @@ from struct import calcsize, pack, unpack_from
 from typing import Optional
 
 from spsdk import SPSDKError
+from spsdk.exceptions import SPSDKParsingError
 from spsdk.utils.easy_enum import Enum
 
 ########################################################################################################################
@@ -53,19 +54,6 @@ class CmdTag(Enum):
     UNLK = (0xB2, "UNLK", "Unlock")
 
 
-########################################################################################################################
-# Exceptions
-########################################################################################################################
-
-
-class UnparsedException(Exception):
-    """Unparsed Exception."""
-
-
-class CorruptedException(Exception):
-    """Corrupted Exception."""
-
-
 ########################################################################################################################
 # Classes
 ########################################################################################################################
@@ -137,11 +125,11 @@ class Header:
         :param offset: Offset of input data
         :param required_tag: Check header TAG if specified value or ignore if is None
         :return: Header object
-        :raise UnparsedException: if required header tag does not match
+        :raises SPSDKParsingError: if required header tag does not match
         """
         tag, length, param = unpack_from(cls.FORMAT, data, offset)
         if required_tag is not None and tag != required_tag:
-            raise UnparsedException(
+            raise SPSDKParsingError(
                 f" Invalid header tag: '0x{tag:02X}' expected '0x{required_tag:02X}' "
             )
 
@@ -176,7 +164,7 @@ class CmdHeader(Header):
         :param offset: to start reading binary data
         :param required_tag: CmdTag, None if not required
         :return: parsed instance
-        :raises UnparsedException: if required header tag does not match
+        :raises SPSDKParsingError: If required header tag does not match
         :raises SPSDKError: If invalid tag
         """
         if required_tag is not None:
@@ -195,18 +183,18 @@ class Header2(Header):
         return pack(self.FORMAT, self.param, self.length, self.tag)
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0, required_tag: int = None) -> "Header":
+    def parse(cls, data: bytes, offset: int = 0, required_tag: Optional[int] = None) -> "Header":
         """Parse header.
 
         :param data: Raw data as bytes or bytearray
         :param offset: Offset of input data
         :param required_tag: Check header TAG if specified value or ignore if is None
-        :raises UnparsedException: Raises an error if required tag is empty or not valid
+        :raises SPSDKParsingError: Raises an error if required tag is empty or not valid
         :return: Header2 object
         """
         param, length, tag = unpack_from(cls.FORMAT, data, offset)
         if required_tag is not None and tag != required_tag:
-            raise UnparsedException(
+            raise SPSDKParsingError(
                 f" Invalid header tag: '0x{tag:02X}' expected '0x{required_tag:02X}' "
             )
 
