@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -18,7 +18,7 @@ from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.ciphers.aead import AESCCM
 from cryptography.hazmat.primitives.serialization import Encoding
 
-from spsdk import SPSDKError
+from spsdk.exceptions import SPSDKError, SPSDKParsingError
 from spsdk.utils.crypto import crypto_backend
 from spsdk.utils.easy_enum import Enum
 from spsdk.utils.misc import DebugInfo, align, align_block, extend_block
@@ -32,7 +32,7 @@ from .commands import (
     EnumEngine,
     EnumInsKey,
 )
-from .header import Header, Header2, UnparsedException
+from .header import Header, Header2
 from .misc import NotEnoughBytesException, read_raw_data, read_raw_segment
 from .secret import MAC, CertificateImg, Signature, SrkTable
 from .segments import (
@@ -146,6 +146,7 @@ class BootImgBase:
 # Boot Image V2 (i.MX-RT)
 ########################################################################################################################
 
+
 # pylint: disable=too-many-public-methods
 class BootImgRT(BootImgBase):
     """IMX Boot Image v2."""
@@ -365,16 +366,36 @@ class BootImgRT(BootImgBase):
         """
         return self._bee
 
+    @bee.setter
+    def bee(self, bee: SegBEE) -> None:
+        """Setter.
+
+        :param bee: BEE instance to be set
+        """
+        assert isinstance(bee, SegBEE)
+        self._bee = bee
+
     @property
     def app_offset(self) -> int:
         """:return: offset in the binary image, where the application starts.
 
+        Please mind: the offset include FCB block (even the FCB block is not exported)
+        The offset is 0x2000 for XIP images and 0x1000 for non-XIP images
+        """
+        return self.get_app_offset(self.ivt_offset)
+
+    @staticmethod
+    def get_app_offset(ivt_offset: int) -> int:
+        """:return: offset in the binary image, where the application starts.
+
+        :param ivt_offset: Offset of IVT segment
+
         Please mind: the offset include FCB block (even the FCB block is not exported)
         The offset is 0x2000 for XIP images and 0x1000 for non-XIP images
         """
         return (
             BootImgRT.XIP_APP_OFFSET
-            if (self.ivt_offset == self.IVT_OFFSET_NOR_FLASH)
+            if (ivt_offset == BootImgRT.IVT_OFFSET_NOR_FLASH)
             else BootImgRT.NON_XIP_APP_OFFSET
         )
 
@@ -844,12 +865,11 @@ class BootImgRT(BootImgBase):
         #
         self.csf = csf
 
-    def _export_fcb_bee(self, dbg_info: DebugInfo) -> bytes:
-        """Export FCB and BEE segments.
+    def export_fcb(self, dbg_info: DebugInfo) -> bytes:
+        """Export FCB segment.
 
         :param dbg_info: optional instance to provide info about exported data
-        :return: binary FCB segment and BEE regions
-        :raises ValueError: if any BEE region is configured for images not located in the FLASH
+        :return: binary FCB segment
         :raises SPSDKError: If invalid length of data
         """
         if not self.fcb.enabled:
@@ -857,12 +877,55 @@ class BootImgRT(BootImgBase):
         data = self.fcb.export(dbg_info=dbg_info)
         if len(data) != self.fcb.space:
             raise SPSDKError("Invalid length of data")
+        return data
+
+    def export_bee(self, dbg_info: DebugInfo) -> bytes:
+        """Export BEE segment.
+
+        :param dbg_info: optional instance to provide info about exported data
+        :return: binary BEE segment
+        :raises SPSDKError: if any BEE region is configured for images not located in the FLASH
+        """
+        data = b""
         if self.ivt_offset == self.IVT_OFFSET_NOR_FLASH:
-            data += self.bee.export(dbg_info=dbg_info)
+            data = self.bee.export(dbg_info=dbg_info)
         elif self.bee.space > 0:
             raise SPSDKError("BEE can be configured only for XIP images located in FLASH")
         return data
 
+    def export_dcd(self, dbg_info: DebugInfo) -> bytes:
+        """Export DCD segment.
+
+        :param dbg_info: optional instance to provide info about exported data
+        :return: binary DCD segment
+        :raises SPSDKError: If DCD padding is not set
+        """
+        dcd_data = b""
+        if (self.dcd is not None) and self.dcd.enabled:
+            if self.dcd.padding_len != 0:
+                raise SPSDKError("Padding can not be present")
+            dcd_data = self.dcd.export()
+            dbg_info.append_binary_section("DCD", dcd_data)
+        return dcd_data
+
+    def export_csf(
+        self, dbg_info: DebugInfo, data: bytes, zulu: datetime = datetime.now(timezone.utc)
+    ) -> bytes:
+        """Export CSF segment.
+
+        :param dbg_info: optional instance to provide info about exported data
+        :param data: generated binary data used for creating of signature
+        :param zulu: current UTC datetime
+        :return: binary CFD segment
+        """
+        csf_data = b""
+        if self.enabled_csf:
+            dbg_info.append_section("CSF")
+            base_data_addr = self.address if self.fcb.enabled else self.address + self.ivt_offset
+            self.enabled_csf.update_signatures(zulu, data, base_data_addr)
+            csf_data = self.enabled_csf.export(dbg_info=dbg_info)
+        return csf_data
+
     def _bee_encrypt_img_data(self, data: bytes) -> bytes:
         """Encrypt data located in BEE regions.
 
@@ -906,9 +969,11 @@ class BootImgRT(BootImgBase):
 
         self._update()
         dbg_info.append_section("RT10xxBootableImage")
-        # FCB + BEE
-        data = self._export_fcb_bee(dbg_info)
-
+        # FCB
+        data = self.export_fcb(dbg_info)
+        # BEE
+        bee_data = self.export_bee(dbg_info)
+        data += bee_data
         # IVT
         ivt_data = self.ivt.export()
         data += ivt_data
@@ -918,12 +983,8 @@ class BootImgRT(BootImgBase):
         data += bdt_data
         dbg_info.append_binary_section("BDT", bdt_data)
         # DCD
-        if (self.dcd is not None) and self.dcd.enabled:
-            if self.dcd.padding_len != 0:
-                raise SPSDKError("Padding can not be present")
-            dcd_data = self.dcd.export()
-            data += dcd_data
-            dbg_info.append_binary_section("DCD", dcd_data)
+        dcd_data = self.export_dcd(dbg_info)
+        data += dcd_data
         # padding before APP
         app_alignment = self.app_offset if self.fcb.enabled else self.app_offset - self.ivt_offset
         if not app_alignment >= len(data):
@@ -934,13 +995,8 @@ class BootImgRT(BootImgBase):
         data += app_data
         dbg_info.append_binary_section("APP", app_data)
         # CSF
-        if csf:
-            if dbg_info:
-                dbg_info.append_section("CSF")
-            base_data_addr = self.address if self.fcb.enabled else self.address + self.ivt_offset
-            csf.update_signatures(zulu, data, base_data_addr)
-            data += csf.export(dbg_info=dbg_info)
-
+        csf_data = self.export_csf(dbg_info, data=data, zulu=zulu)
+        data += csf_data
         return self._bee_encrypt_img_data(data)
 
     @classmethod
@@ -961,7 +1017,6 @@ class BootImgRT(BootImgBase):
             end_pos = min(start_pos + size, end_pos)
 
         for ivt_ofs in cls.IVT_OFFSETS:
-
             if start_pos + ivt_ofs > end_pos:
                 break
             strm.seek(start_pos + ivt_ofs)
@@ -970,7 +1025,7 @@ class BootImgRT(BootImgBase):
                 header = Header.parse(header_data, required_tag=SegTag.IVT2)
                 if (header.length == SegIVT2.SIZE) and (header.param in cls.VERSIONS):
                     return header, start_pos + ivt_ofs, end_pos
-            except UnparsedException:  # ignore different header tags
+            except SPSDKParsingError:  # ignore different header tags
                 pass
 
         raise SPSDKError("IVT not found")
@@ -1039,7 +1094,7 @@ class BootImgRT(BootImgBase):
             xmcd_header = XMCDHeader.parse(read_raw_data(stream, XMCDHeader.SIZE))
             xmcd_data = read_raw_data(stream, xmcd_header.config_data_size)
             obj.xmcd = SegXMCD(header=xmcd_header, config_data=xmcd_data)
-        except UnparsedException:
+        except SPSDKParsingError:
             # No XMCD found
             pass
         # Parse BDT
@@ -1264,7 +1319,7 @@ class BootImg2(BootImgBase):
             if address != 0:
                 self.address = address
         else:
-            raise Exception("Unknown data type !")
+            raise SPSDKError("Unknown data type !")
 
     def export(self) -> bytes:
         """Export image as bytes array.
@@ -1285,7 +1340,7 @@ class BootImg2(BootImgBase):
         cls,
         stream: Union[bytes, bytearray, BufferedReader, BytesIO],
         step: int = 0x100,
-        size: int = None,
+        size: Optional[int] = None,
     ) -> "BootImg2":
         """Parse image from stream buffer or bytes array.
 
@@ -1550,7 +1605,7 @@ class BootImg8m(BootImgBase):
             if address != 0:
                 self.address = address
         else:
-            raise Exception("Unknown data type !")
+            raise SPSDKError("Unknown data type !")
 
     def export(self) -> bytes:
         """Export Image as bytes array.
@@ -1571,7 +1626,7 @@ class BootImg8m(BootImgBase):
         cls,
         stream: Union[bytes, bytearray, BufferedReader, BytesIO],
         step: int = 0x100,
-        size: int = None,
+        size: Optional[int] = None,
     ) -> BootImgBase:
         """Parse image from stream buffer or bytes array.
 
@@ -1911,7 +1966,7 @@ class BootImg3a(BootImgBase):
 
         elif img_type == EnumAppType.SCD:
             if self._sdc_address == 0:
-                raise Exception("SCFW have to be define before SCD!")
+                raise SPSDKError("SCFW have to be define before SCD!")
             image_index = self.bdt[0].images_count
             self.bdt[0].images[image_index].image_destination = self._sdc_address
             self.bdt[0].images[image_index].image_entry = 0
@@ -1925,7 +1980,7 @@ class BootImg3a(BootImgBase):
             self.app[0][image_index].padding = self._compute_padding(len(data), self.SECTOR_SIZE)
 
         else:
-            raise Exception("Unknown data type!")
+            raise SPSDKError("Unknown data type!")
 
     def export(self) -> bytes:
         """Export Image as binary blob."""
@@ -1953,7 +2008,7 @@ class BootImg3a(BootImgBase):
         cls,
         stream: Union[bytes, bytearray, BufferedReader, BytesIO],
         step: int = 0x100,
-        size: int = None,
+        size: Optional[int] = None,
     ) -> BootImgBase:
         """Parse image from stream buffer or bytes array.
 
@@ -2319,7 +2374,7 @@ class BootImg3b(BootImgBase):
 
         elif img_type == EnumAppType.SCD:
             if self._scd_address == 0:
-                raise Exception("SCFW have to be define before SCD!")
+                raise SPSDKError("SCFW have to be define before SCD!")
             self.scd.data = data
             self.scd.padding = self._compute_padding(len(data), self.SECTOR_SIZE)
             self.bdt[0].scd.image_destination = self._scd_address
@@ -2328,7 +2383,7 @@ class BootImg3b(BootImgBase):
             self.ivt[0].scd_address = self.bdt[0].scd.image_destination
 
         else:
-            raise Exception("Unknown image type!")
+            raise SPSDKError("Unknown image type!")
 
     def export(self) -> bytes:
         """Export."""
@@ -2362,7 +2417,7 @@ class BootImg3b(BootImgBase):
         cls,
         stream: Union[bytes, bytearray, BufferedReader, BytesIO],
         step: int = 0x100,
-        size: int = None,
+        size: Optional[int] = None,
     ) -> BootImgBase:
         """Parse image from stream buffer or bytes array.
 
@@ -2498,7 +2553,7 @@ class BootImg4(BootImgBase):
         cls,
         stream: Union[bytes, bytearray, BufferedReader, BytesIO],
         step: int = 0x100,
-        size: int = None,
+        size: Optional[int] = None,
     ) -> BootImgBase:
         """Parse image from stream buffer or bytes array.
 
@@ -2597,7 +2652,7 @@ class KernelImg:
         self,
         address: int = 0,
         app: Optional[bytes] = None,
-        csf: Union[SegCSF, Any] = None,
+        csf: Optional[Union[SegCSF, Any]] = None,
         version: int = 0x41,
     ) -> None:
         """Initialize the IMX Kernel Image."""
@@ -2642,7 +2697,7 @@ class KernelImg:
 def parse(
     stream: Union[bytes, bytearray, BufferedReader, BytesIO],
     step: int = 0x100,
-    size: int = None,
+    size: Optional[int] = None,
 ) -> BootImgBase:
     """Common parser for all versions of i.MX boot images.
 
