@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -184,7 +184,7 @@ class CertificateImg(BaseClass):
         """Size of Certificate structure for bootable image."""
         return Header.SIZE + len(self._data)
 
-    def __init__(self, version: int = 0x40, data: bytes = None) -> None:
+    def __init__(self, version: int = 0x40, data: Optional[bytes] = None) -> None:
         """Initialize the certificate structure for bootable image."""
         super().__init__(SegTag.CRT, version)
         self._data = bytearray() if data is None else bytearray(data)
@@ -241,7 +241,7 @@ class Signature(BaseClass):
         """Size of a signature."""
         return Header.SIZE + len(self._data)
 
-    def __init__(self, version: int = 0x40, data: bytes = None) -> None:
+    def __init__(self, version: int = 0x40, data: Optional[bytes] = None) -> None:
         """Initialize the signature."""
         super().__init__(tag=SegTag.SIG, version=version)
         self._data = bytearray() if data is None else bytearray(data)
@@ -296,7 +296,7 @@ class Signature(BaseClass):
 
 
 class MAC(BaseClass):
-    """Structure that holds initial parameter for AES encryption/description.
+    """Structure that holds initial parameter for AES encryption/decryption.
 
     - nonce - initialization vector for AEAD AES128 decryption
     - mac - message authentication code to verify the decryption was successful
