@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Segments within image module."""
@@ -14,6 +14,12 @@ from struct import calcsize, pack, unpack_from
 from typing import Dict, Iterator, List, Optional, Sequence, Tuple, Union
 
 from spsdk import SPSDKError
+from spsdk.exceptions import (
+    SPSDKCorruptedException,
+    SPSDKParsingError,
+    SPSDKSyntaxError,
+    SPSDKValueError,
+)
 from spsdk.utils.misc import DebugInfo, align, align_block, extend_block, size_fmt
 
 from .bee import BEE_ENCR_BLOCK_SIZE, BeeRegionHeader
@@ -30,7 +36,7 @@ from .commands import (
     EnumWriteOps,
     parse_command,
 )
-from .header import CorruptedException, Header, Header2, SegTag, UnparsedException
+from .header import Header, Header2, SegTag
 from .secret import MAC, BaseClass
 
 logger = logging.getLogger(__name__)
@@ -128,7 +134,7 @@ class AbstractFCB(BaseSegment):
 
     @property
     def enabled(self) -> bool:
-        """Whether FCB is enabled. Note: it is not generated to output is disabled."""
+        """Whether FCB is enabled. Note: it is not generated to output if disabled."""
         return self._enabled
 
     @enabled.setter
@@ -620,7 +626,9 @@ class SegBEE(BaseSegment):
         return result
 
     @classmethod
-    def parse(cls, data: bytes, offset: int = 0, decrypt_keys: List[bytes] = None) -> "SegBEE":
+    def parse(
+        cls, data: bytes, offset: int = 0, decrypt_keys: Optional[List[bytes]] = None
+    ) -> "SegBEE":
         """De-serialization.
 
         :param data: binary data to be parsed
@@ -999,7 +1007,7 @@ class SegDCD(BaseSegment):
         self._commands.clear()
         self._header.length = self._header.size
 
-    def export_txt(self, txt_data: str = None) -> str:
+    def export_txt(self, txt_data: Optional[str] = None) -> str:
         """Export txt of Device configuration data (DCD) segment."""
         write_ops = ("WriteValue", "WriteClearBits", "ClearBitMask", "SetBitMask")
         check_ops = ("CheckAllClear", "CheckAllSet", "CheckAnyClear", "CheckAnySet")
@@ -1008,7 +1016,7 @@ class SegDCD(BaseSegment):
 
         for cmd in self._commands:
             if isinstance(cmd, CmdWriteData):
-                for (address, value) in cmd:
+                for address, value in cmd:
                     txt_data += (
                         f"{write_ops[cmd.ops]} {cmd.num_bytes} 0x{address:08X} 0x{value:08X}\n"
                     )
@@ -1067,7 +1075,7 @@ class SegDCD(BaseSegment):
         """Parse segment from bytes array.
 
         :param data: The bytes array of DCD segment
-        :raises CorruptedException: Exception caused by corrupted data
+        :raises SPSDKCorruptedException: Exception caused by corrupted data
         :return: SegDCD object
         """
         header = Header.parse(data, 0, SegTag.DCD)
@@ -1077,7 +1085,7 @@ class SegDCD(BaseSegment):
             try:
                 cmd_obj = parse_command(data, index)
             except ValueError as exc:
-                raise CorruptedException("Unknown command at position: " + hex(index)) from exc
+                raise SPSDKCorruptedException("Unknown command at position: " + hex(index)) from exc
 
             obj.append(cmd_obj)
             index += cmd_obj.size
@@ -1099,7 +1107,7 @@ class SegDcdBuilder:
 
         :param dcd_obj: result of the builder
         :param cmd: command with arguments
-        :raises SyntaxError: command is corrupted
+        :raises SPSDKError: command is corrupted
         :raises SPSDKError: When command is unsupported
         """
         # ----------------------------
@@ -1120,7 +1128,7 @@ class SegDcdBuilder:
                     self.cmd_write = None
 
                 if cmd[1] not in EnumEngine:
-                    raise SyntaxError(
+                    raise SPSDKError(
                         f"Unlock CMD: wrong engine parameter at line {self.line_cnt - 1}"
                     )
 
@@ -1133,7 +1141,7 @@ class SegDcdBuilder:
 
         elif cmd_tuple[0] == "write":
             if len(cmd) < 4:
-                raise SyntaxError(f"Write CMD: not enough arguments at line {self.line_cnt - 1}")
+                raise SPSDKError(f"Write CMD: not enough arguments at line {self.line_cnt - 1}")
 
             ops = cmd_tuple[1]
             numbytes = int(cmd[1])
@@ -1152,7 +1160,9 @@ class SegDcdBuilder:
 
         else:
             if len(cmd) < 4:
-                raise SyntaxError(f"Check CMD: not enough arguments at line {self.line_cnt - 1}")
+                raise SPSDKSyntaxError(
+                    f"Check CMD: not enough arguments at line {self.line_cnt - 1}"
+                )
 
             if self.cmd_write is not None:
                 dcd_obj.append(self.cmd_write)
@@ -1170,7 +1180,6 @@ class SegDcdBuilder:
 
         :param text: input text to import
         :return: SegDCD object
-        :raise SyntaxError: if input format is not valid
         """
         dcd_obj = SegDCD(enabled=True)
         cmd_mline = False
@@ -1432,8 +1441,8 @@ class SegCSF(BaseSegment):
 
         :param data: The bytes array of CSF segment
         :param offset: to start parsing the data
-        :raises CorruptedException: When there is unknown command
-        :raises CorruptedException: When command can not be parsed
+        :raises SPSDKCorruptedException: When there is unknown command
+        :raises SPSDKCorruptedException: When command can not be parsed
         :return: SegCSF instance
         """
         header = Header.parse(data, offset, SegTag.CSF)
@@ -1445,7 +1454,7 @@ class SegCSF(BaseSegment):
                 cmd_obj = parse_command(data, offset + index)
                 obj.append_command(cmd_obj)
             except ValueError as exc:
-                raise CorruptedException(
+                raise SPSDKCorruptedException(
                     "Failed to parse command at position: " + hex(offset + index)
                 ) from exc
             index += cmd_obj.size
@@ -1474,11 +1483,17 @@ class XMCDHeader:
         :param instance: Number of the interface instance, defaults to 0
         :param block_type: Type of XMCD data (0 - Simplified, 1 - Full), defaults to 0
         :param block_size: XMCD data block size, defaults to 4
+        :raises SPSDKValueError: If the given interface is not supported
+        :raises SPSDKValueError: If the given blovk type is not supported
         """
         self.tag = 0x0C
         self.version = 0
+        if interface not in [0, 1]:
+            raise SPSDKValueError(f"Interface not supported: {interface}")
         self.interface = interface
         self.instance = instance
+        if block_type not in [0, 1]:
+            raise SPSDKValueError(f"Block type not supported: {block_type}")
         self.block_type = block_type
         self.block_size = block_size
 
@@ -1486,10 +1501,10 @@ class XMCDHeader:
         """Export segment's header as bytes (serialization)."""
         return pack(
             self.FORMAT,
-            self.tag << 4 + self.version,
-            self.interface << 4 + self.instance,
-            self.block_size << 4 + self.block_size >> 8,
             self.block_size & 0xFF,
+            (self.block_type << 4) + (self.block_size >> 8),
+            self.interface << 4 + self.instance,
+            self.tag << 4 + self.version,
         )
 
     def info(self) -> str:
@@ -1512,12 +1527,14 @@ class XMCDHeader:
         size_low, type_size, interface_instance, tag_ver = unpack_from(cls.FORMAT, data)
         tag = (tag_ver & 0xF0) >> 4
         if tag != cls.TAG:
-            raise UnparsedException("Invalid TAG for XMCDHeader. ")
+            raise SPSDKParsingError(f"Invalid TAG for XMCDHeader {tag}. Expected: {cls.TAG}")
         version = tag_ver & 0x0F
+        if version != 0:
+            raise SPSDKParsingError(f"Invalid version {version}. Expected: 0")
         interface = (interface_instance & 0xF0) >> 4
         instance = interface_instance & 0x0F
         block_type = (type_size & 0xF0) >> 4
-        block_size = type_size & 0x0F
+        block_size = (type_size & 0x0F) << 8
         block_size += size_low
         return XMCDHeader(
             interface=interface, instance=instance, block_type=block_type, block_size=block_size
@@ -1548,7 +1565,11 @@ class SegXMCD(BaseSegment):
     def parse(cls, data: bytes) -> "SegXMCD":
         """Parse XMCD from binary data."""
         header = XMCDHeader.parse(data)
-        config_data = data[header.SIZE : header.block_size - header.SIZE]
+        if header.block_size != len(data):
+            raise SPSDKValueError(
+                f"Invalid length of data {len(data)}. Length must be equal to header value {header.block_size}"
+            )
+        config_data = data[header.SIZE : header.block_size]
         return SegXMCD(header=header, config_data=config_data)
 
     def info(self) -> str:
