@@ -1,20 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Module provides support for TrustZone configuration data."""
-import json
 import logging
 import os
 import struct
-from copy import deepcopy
 from typing import Any, Dict, List, Optional
 
 from spsdk import SPSDKError
+from spsdk.exceptions import SPSDKValueError
 from spsdk.image import TZ_SCH_FILE
+from spsdk.utils.database import Database
 from spsdk.utils.easy_enum import Enum
 from spsdk.utils.misc import format_value, load_configuration, value_to_int
 from spsdk.utils.schema_validator import ConfigTemplate, ValidationSchemas
@@ -25,9 +25,9 @@ logger = logging.getLogger(__name__)
 class TrustZoneType(Enum):
     """Enum defining various types of TrustZone types."""
 
-    ENABLED = (0x00, "TrustZone enabled with default settings")
-    CUSTOM = (0x20, "TrustZone enabled with custom settings")
-    DISABLED = (0x40, "Disabled")
+    ENABLED = (0x0, "TrustZone enabled with default settings")
+    CUSTOM = (0x1, "TrustZone enabled with custom settings")
+    DISABLED = (0x2, "Disabled")
 
 
 class TrustZone:
@@ -36,7 +36,7 @@ class TrustZone:
     PRESET_DIR = os.path.normpath(
         os.path.join(os.path.dirname(__file__), "..", "data", "tz_presets")
     )
-    CONFIG_FILE = os.path.join(PRESET_DIR, "database.json")
+    CONFIG_FILE = os.path.join(PRESET_DIR, "database.yaml")
 
     @classmethod
     def enabled(cls) -> "TrustZone":
@@ -49,7 +49,9 @@ class TrustZone:
         return cls(tz_type=TrustZoneType.DISABLED)
 
     @classmethod
-    def custom(cls, family: str, customizations: dict, revision: str = None) -> "TrustZone":
+    def custom(
+        cls, family: str, customizations: dict, revision: Optional[str] = None
+    ) -> "TrustZone":
         """Alternate constructor for CUSTOM type of TrustZone."""
         return cls(
             tz_type=TrustZoneType.CUSTOM,
@@ -59,7 +61,9 @@ class TrustZone:
         )
 
     @classmethod
-    def from_binary(cls, family: str, raw_data: bytes, revision: str = None) -> "TrustZone":
+    def from_binary(
+        cls, family: str, raw_data: bytes, revision: Optional[str] = None
+    ) -> "TrustZone":
         """Alternate constructor using existing binary data."""
         return cls(
             tz_type=TrustZoneType.CUSTOM,
@@ -86,17 +90,17 @@ class TrustZone:
     def __init__(
         self,
         tz_type: TrustZoneType = TrustZoneType.ENABLED,
-        family: str = None,
-        revision: str = None,
-        customizations: dict = None,
-        raw_data: bytes = None,
+        family: Optional[str] = None,
+        revision: Optional[str] = None,
+        customizations: Optional[dict] = None,
+        raw_data: Optional[bytes] = None,
     ) -> None:
         """Initialize the trustzone."""
         self.type = tz_type
-        self.family: Optional[str] = family
-        self.config: dict = self.load_config_file()
-        self.customs: Optional[dict] = customizations
-        self.revision: Optional[str] = revision
+        self.family = family
+        self.database = self.load_database()
+        self.customs = customizations
+        self.revision = revision
 
         if self.type == TrustZoneType.DISABLED and customizations:
             raise SPSDKError("TrustZone was disabled, can't add trust_zone_data")
@@ -128,6 +132,21 @@ class TrustZone:
                     f"{[item for item in self.customs if item not in self.presets]}"
                 )
 
+    @classmethod
+    def get_preset_data_size(cls, family: str, revision: str = "latest") -> int:
+        """Get size of preset data in binary form.
+
+        :param family: Family description.
+        :param revision: Chip revision specification, as default, latest is used.
+        :raises SPSDKValueError: Family or revision is not supported.
+        :return: Size of TZ data.
+        """
+        database = cls.load_database()
+        file_name = database.devices.get_by_name(family).revisions.get(revision).data_file
+        if not file_name:
+            raise SPSDKValueError(f"Data file for {family} does't exist!")
+        return len(load_configuration(os.path.join(TrustZone.PRESET_DIR, file_name))) * 4
+
     @classmethod
     def get_validation_schemas_family(cls) -> List[Dict[str, Any]]:
         """Create the validation schema just for supported families.
@@ -135,6 +154,7 @@ class TrustZone:
         :return: List of validation schemas for TZ supported families.
         """
         sch_cfg = ValidationSchemas.get_schema_file(TZ_SCH_FILE)
+        sch_cfg["tz_family_rev"]["properties"]["family"]["enum"] = cls.get_supported_families()
         return [sch_cfg["tz_family_rev"]]
 
     @classmethod
@@ -146,16 +166,15 @@ class TrustZone:
         :raises SPSDKError: Family or revision is not supported.
         :return: List of validation schemas.
         """
-        config_file = cls.load_config_file()
+        database = cls.load_database()
         sch_cfg = ValidationSchemas.get_schema_file(TZ_SCH_FILE)
         preset_properties = {}
 
         try:
-            if not revision or revision == "latest":
-                revision = config_file[family]["latest"]
-
+            revision_info = database.devices.get_by_name(family).revisions.get(revision)
+            assert revision_info.data_file
             presets = load_configuration(
-                os.path.join(TrustZone.PRESET_DIR, config_file[family]["revisions"][revision])
+                os.path.join(TrustZone.PRESET_DIR, revision_info.data_file)
             )
             for key, value in presets.items():
                 preset_properties[key] = {
@@ -168,7 +187,7 @@ class TrustZone:
             if "patternProperties" in sch_cfg["tz"]["properties"]["trustZonePreset"].keys():
                 sch_cfg["tz"]["properties"]["trustZonePreset"].pop("patternProperties")
             sch_cfg["tz"]["properties"]["trustZonePreset"]["properties"] = preset_properties
-
+            sch_cfg["tz_family_rev"]["properties"]["family"]["enum"] = cls.get_supported_families()
             return [sch_cfg["tz_family_rev"], sch_cfg["tz"]]
         except (KeyError, SPSDKError) as exc:
             raise SPSDKError(f"Family {family} or revision {revision} is not supported") from exc
@@ -183,11 +202,11 @@ class TrustZone:
         :return: Dictionary of individual templates (key is name of template, value is template itself).
         """
         ret: Dict[str, str] = {}
+        database = cls.load_database()
         if family in cls.get_supported_families():
             try:
-                if not revision or revision == "latest":
-                    config_file = cls.load_config_file()
-                    revision = config_file[family]["latest"]
+                revision_info = database.devices.get_by_name(family).revisions.get(revision)
+                revision = revision_info.name
             except (KeyError, SPSDKError) as exc:
                 raise SPSDKError(f"Revision {revision} is not supported") from exc
 
@@ -209,35 +228,33 @@ class TrustZone:
         return f"<TrustZone: type: {self.type} ({TrustZoneType.desc(self.type)})"
 
     @classmethod
-    def load_config_file(cls) -> dict:
+    def load_database(cls) -> Database:
         """Load data from TZ config file."""
-        with open(cls.CONFIG_FILE) as f:
-            return json.load(f)
+        return Database(cls.CONFIG_FILE)
 
-    @staticmethod
-    def get_supported_families() -> List[str]:
-        """Return list of supported families.
+    @classmethod
+    def get_supported_families(cls) -> List[str]:
+        """Return list of supported families."""
+        return Database.get_devices(cls.CONFIG_FILE).device_names
 
-        :return: List of supported families.
-        """
-        tz_sch_cfg = ValidationSchemas().get_schema_file(TZ_SCH_FILE)
-        tz_families = tz_sch_cfg["tz_family_rev"]
-
-        return deepcopy(tz_families["properties"]["family"]["enum"])
-
-    def get_families(self) -> list:
+    def get_families(self) -> List[str]:
         """Return list of supported chip families."""
-        return list(self.config.keys())
+        return self.database.devices.device_names
 
-    def get_revisions(self, family: str = None) -> list:
+    def get_revisions(self, family: Optional[str] = None) -> List[str]:
         """Return a list of revisions for given family."""
-        return list(self.config[family or self.family]["revisions"].keys())
+        actual_family = family or self.family
+        assert actual_family
+        revisions = self.database.devices.get_by_name(actual_family).revisions
+        return [revision.name for revision in revisions]
 
-    def get_latest_revision(self, family: str = None) -> str:
+    def get_latest_revision(self, family: Optional[str] = None) -> str:
         """Return latest revision for given family."""
-        return self.config[family or self.family]["latest"]
+        actual_family = family or self.family
+        assert actual_family
+        return self.database.devices.get_by_name(actual_family).revisions.get_latest().name
 
-    def sanitize_revision(self, family: str, revision: str = None) -> str:
+    def sanitize_revision(self, family: str, revision: Optional[str] = None) -> str:
         """Sanitize revision.
 
         if the 'revision' is None return the latest revision
@@ -248,24 +265,34 @@ class TrustZone:
         return revision.lower()
 
     def _get_preset_file(self) -> str:
-        return os.path.join(
-            TrustZone.PRESET_DIR, self.config[self.family]["revisions"][self.revision]
+        assert self.family
+        file_name = (
+            self.database.devices.get_by_name(self.family).revisions.get(self.revision).data_file
         )
+        assert file_name, f"Data file for {self.family} does't exist!"
+        return os.path.join(TrustZone.PRESET_DIR, file_name)
 
     def _load_presets(self) -> dict:
         """Load default TrustZone settings for given family and revision."""
-        with open(self._get_preset_file()) as preset_file:
-            return json.load(preset_file)
+        presets = load_configuration(self._get_preset_file())
+        # Unify the values format
+        return {name: format_value(value_to_int(value), 32) for name, value in presets.items()}
 
     def _parse_raw_data(self, raw_data: bytes) -> dict:
         """Parse raw data into 'customizations' format."""
-        if len(self.presets) != len(raw_data) // 4:
+        if len(self.presets) > len(raw_data) // 4:
             raise SPSDKError(
                 "Trustzone binary file has incorrect raw_data length\n"
                 f"Expected: {len(self.presets)}, Got: {len(raw_data) // 4}"
             )
 
-        registers = struct.unpack(f"<{len(raw_data) // 4}L", raw_data)
+        if len(self.presets) != len(raw_data) // 4:
+            logger.warning(
+                "Trustzone binary file has incorrect raw_data length\n"
+                f"Expected: {len(self.presets)}, Got: {len(raw_data) // 4}"
+            )
+
+        registers = struct.unpack(f"<{len(self.presets)}L", raw_data[: len(self.presets) * 4])
         customs = {name: format_value(registers[i], 32) for i, name in enumerate(self.presets)}
         return customs
 
@@ -279,10 +306,9 @@ class TrustZone:
             raise SPSDKError("Preset data not present")
         if self.customs is None:
             raise SPSDKError("Data not present")
-        logger.info(f"{len(self.presets)} registers loaded from defaults")
-        logger.debug(self.presets)
-        logger.info(f"{len(self.customs)} modifications provided")
-        logger.debug(self.customs)
+        modifications = set(self.customs.items()) - set(self.presets.items())
+        logger.info(f"{len(modifications)} modifications provided")
+        logger.debug(str(modifications).replace(", ", "\n"))
         data = self.presets
         data.update(self.customs)
         registers = [value_to_int(item) for item in data.values()]
