@@ -2,14 +2,14 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2016-2018 Martin Olejar
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Commands and responses used by MBOOT module."""
 
 from struct import pack, unpack, unpack_from
-from typing import Dict, List, Type
+from typing import Dict, List, Optional, Type
 
 from spsdk.utils.easy_enum import Enum
 
@@ -49,6 +49,7 @@ class CommandTag(Enum):
     TRUST_PROVISIONING          = (0x16, "TrustProvisioning", "Trust Provisioning")
     FUSE_READ                   = (0x17, "ReadFuse", "Read Fuse")
     UPDATE_LIFE_CYCLE           = (0x18, "UpdateLifeCycle", "Update Life Cycle")
+    ELE_MESSAGE                 = (0x19, "EleMessage", "Send EdgeLock Enclave Message")
 
     # reserved commands
     CONFIGURE_I2C = (0xC1, "ConfigureI2c", "Configure I2C")
@@ -225,7 +226,7 @@ class CmdPacket:
     SIZE = 32
     EMPTY_VALUE = 0x00
 
-    def __init__(self, tag: int, flags: int, *args: int, data: bytes = None) -> None:
+    def __init__(self, tag: int, flags: int, *args: int, data: Optional[bytes] = None) -> None:
         """Initialize the Command Packet object.
 
         :param tag: Tag identifying the command
