@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2016-2018 Martin Olejar
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -38,6 +38,7 @@ class StatusCode(Enum):
     FLASH_REGION_EXECUTE_ONLY       = (108, "FlashRegionExecuteOnly", "FLASH Driver: Region Execute Only")
     FLASH_EXEC_IN_RAM_NOT_READY     = (109, "FlashExecuteInRamFunctionNotReady", "FLASH Driver: Execute In RAM Function Not Ready")
     FLASH_COMMAND_NOT_SUPPORTED     = (111, "FlashCommandNotSupported", "FLASH Driver: Command Not Supported")
+    FLASH_ECC_ERROR                 = (116, "FlashEccError", "FLASH Driver: ECC Error")
     FLASH_OUT_OF_DATE_CFPA_PAGE     = (132, "FlashOutOfDateCfpaPage", "FLASH Driver: Out Of Date CFPA Page")
     FLASH_BLANK_IFR_PAGE_DATA       = (133, "FlashBlankIfrPageData", "FLASH Driver: Blank IFR Page Data")
     FLASH_ENCRYPTED_REGIONS_ERASE_NOT_DONE_AT_ONCE = (134, "FlashEncryptedRegionsEraseNotDoneAtOnce", "FLASH Driver: Encrypted Regions Erase Not Done At Once")
@@ -75,12 +76,18 @@ class StatusCode(Enum):
     OTFAD_INVALID_KEY           = (502, "OtfadInvalidKey", "OTFAD Driver: Invalid Key")
     OTFAD_INVALID_KEY_BLOB      = (503, "OtfadInvalidKeyBlob", "OTFAD Driver: Invalid Key Blob")
 
+    # Sending errors.
+    SENDING_OPERATION_CONDITION_ERROR = (1812, "SendOperationConditionError", "Send Operation Condition failed")
+
     # SDMMC driver errors.
 
     # FlexSPI statuses.
-    FLEXSPI_SEQUENCE_EXECUTION_TIMEOUT  = (7000, "FLEXSPI_SequenceExecutionTimeout", "FLEXSPI: Sequence Execution Timeout")
-    FLEXSPI_INVALID_SEQUENCE            = (7001, "FLEXSPI_InvalidSequence", "FLEXSPI: Invalid Sequence")
-    FLEXSPI_DEVICE_TIMEOUT              = (7002, "FLEXSPI_DeviceTimeout", "FLEXSPI: Device Timeout")
+    FLEXSPI_SEQUENCE_EXECUTION_TIMEOUT_RT5xx = (6000, "FLEXSPI_SequenceExecutionTimeout_RT5xx", "FLEXSPI: Sequence Execution Timeout")
+    FLEXSPI_INVALID_SEQUENCE_RT5xx           = (6001, "FLEXSPI_InvalidSequence_RT5xx", "FLEXSPI: Invalid Sequence")
+    FLEXSPI_DEVICE_TIMEOUT_RT5xx             = (6002, "FLEXSPI_DeviceTimeout_RT5xx", "FLEXSPI: Device Timeout")
+    FLEXSPI_SEQUENCE_EXECUTION_TIMEOUT       = (7000, "FLEXSPI_SequenceExecutionTimeout", "FLEXSPI: Sequence Execution Timeout")
+    FLEXSPI_INVALID_SEQUENCE                 = (7001, "FLEXSPI_InvalidSequence", "FLEXSPI: Invalid Sequence")
+    FLEXSPI_DEVICE_TIMEOUT                   = (7002, "FLEXSPI_DeviceTimeout", "FLEXSPI: Device Timeout")
 
     # Bootloader errors.
     UNKNOWN_COMMAND             = (10000, "UnknownCommand", "Unknown Command")
@@ -260,6 +267,9 @@ class StatusCode(Enum):
     OTP_CRC_CHECK_PASS              = (52808, "OTP_CRCCheckPass", "OTP: CRC check passed")
     OTP_VERIFY_FAIL                 = (52009, "OPT_VerifyFail", "OTP: Failed to verify OTP write")
 
+    # Security subsystem statuses.
+    SECURITY_SUBSYSTEM_ERROR  = (1515890085, "SecuritySubSystemError", "Security SubSystem Error")
+
     # TrustProvisioning statuses.
     TP_SUCCESS                  = (0,     "TP_SUCCESS", "TP: SUCCESS")
     TP_GENERAL_ERROR            = (80000, "TP_GENERAL_ERROR", "TP: General error")
