@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright (c) 2019-2022 NXP
+# Copyright (c) 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -10,6 +10,7 @@
 from .base import Interface, MBootInterface
 from .buspal_i2c import scan_buspal_i2c
 from .buspal_spi import scan_buspal_spi
+from .sdio import Sdio, scan_sdio
 from .uart import Uart, scan_uart
 from .usb import RawHid, scan_usb
-from .usbsio import UsbSioI2C, UsbSioSPI, scan_usbsio, scan_usbsio_i2c, scan_usbsio_spi
+from .usbsio import UsbSio, UsbSioI2C, UsbSioSPI, scan_usbsio, scan_usbsio_i2c, scan_usbsio_spi
