@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright (c) 2019-2022 NXP
+# Copyright (c) 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -16,10 +16,10 @@ from libusbsio.libusbsio import LIBUSBSIO
 
 from spsdk import SPSDKError, SPSDKValueError
 from spsdk.mboot.exceptions import McuBootConnectionError
+from spsdk.utils.exceptions import SPSDKTimeoutError
 from spsdk.utils.misc import value_to_int
 from spsdk.utils.usbfilter import USBDeviceFilter
 
-from .base import MBootInterface
 from .uart import Uart
 
 logger = logging.getLogger(__name__)
@@ -41,7 +41,7 @@ def _get_usbsio() -> LIBUSBSIO:
         raise SPSDKError(str(e)) from e
 
 
-def get_usbsio_devices(config: str = None) -> List[int]:
+def get_usbsio_devices(config: Optional[str] = None) -> List[int]:
     """Returns list of ports indexes of USBSIO devices.
 
     It could be filtered by standard SPSDK USB filters.
@@ -93,7 +93,7 @@ def get_usbsio_devices(config: str = None) -> List[int]:
     return port_indexes
 
 
-def scan_usbsio(config: str = None, timeout: int = 5000) -> List["UsbSio"]:
+def scan_usbsio(config: Optional[str] = None, timeout: int = 5000) -> List["UsbSio"]:
     """Scan connected USB-SIO bridge devices.
 
     :param config: Configuration string identifying spi or i2c SIO interface
@@ -148,7 +148,7 @@ def scan_usbsio(config: str = None, timeout: int = 5000) -> List["UsbSio"]:
     return devices
 
 
-def scan_usbsio_i2c(config: str = None, timeout: int = 5000) -> List["UsbSioI2C"]:
+def scan_usbsio_i2c(config: Optional[str] = None, timeout: int = 5000) -> List["UsbSioI2C"]:
     """Scan connected USB-SIO bridge devices and return just I2C devices.
 
     :param config: Configuration string identifying spi or i2c SIO interface
@@ -160,7 +160,7 @@ def scan_usbsio_i2c(config: str = None, timeout: int = 5000) -> List["UsbSioI2C"
     return [x for x in devices if isinstance(x, UsbSioI2C)]
 
 
-def scan_usbsio_spi(config: str = None, timeout: int = 5000) -> List["UsbSioSPI"]:
+def scan_usbsio_spi(config: Optional[str] = None, timeout: int = 5000) -> List["UsbSioSPI"]:
     """Scan connected USB-SIO bridge devices and return just SPI devices.
 
     :param config: Configuration string identifying spi or i2c SIO interface
@@ -185,7 +185,7 @@ class UsbSio(Uart):
         """Indicates whether interface is open."""
         return bool(self.port)
 
-    def __init__(self, dev: int = 0, config: str = None, timeout: int = 5000) -> None:
+    def __init__(self, dev: int = 0, config: Optional[str] = None, timeout: int = 5000) -> None:
         """Initialize the Interface object.
 
         :param dev: device index to be used, default is set to 0
@@ -232,7 +232,7 @@ class UsbSioSPI(UsbSio):
 
     def __init__(
         self,
-        config: str = None,
+        config: Optional[str] = None,
         dev: int = 0,
         port: int = 0,
         ssel_port: int = 0,
@@ -324,7 +324,7 @@ class UsbSioSPI(UsbSio):
         except Exception as e:
             raise McuBootConnectionError(str(e)) from e
         if result < 0 or not data:
-            raise TimeoutError()
+            raise SPSDKTimeoutError()
         logger.debug(f"<{' '.join(f'{b:02x}' for b in data)}>")
         return data
 
@@ -333,7 +333,7 @@ class UsbSioSPI(UsbSio):
 
         :param data: Data to send
         :raises McuBootConnectionError: When sending the data fails
-        :raises TimeoutError: When data could not be written
+        :raises SPSDKTimeoutError: When data could not be written
         """
         logger.debug(f"[{' '.join(f'{b:02x}' for b in data)}]")
         try:
@@ -343,7 +343,7 @@ class UsbSioSPI(UsbSio):
         except Exception as e:
             raise McuBootConnectionError(str(e)) from e
         if result < 0:
-            raise TimeoutError()
+            raise SPSDKTimeoutError()
 
 
 class UsbSioI2C(UsbSio):
@@ -351,7 +351,7 @@ class UsbSioI2C(UsbSio):
 
     def __init__(
         self,
-        config: str = None,
+        config: Optional[str] = None,
         dev: int = 0,
         port: int = 0,
         address: int = 0x10,
@@ -417,16 +417,16 @@ class UsbSioI2C(UsbSio):
 
         :param length: Number of bytes to read
         :return: Data read from the device
-        :raises TimeoutError: Time-out
+        :raises SPSDKTimeoutError: Time-out
         :raises McuBootConnectionError: When reading data from device fails
-        :raises TimeoutError: When no data received
+        :raises SPSDKTimeoutError: When no data received
         """
         try:
             (data, result) = self.port.DeviceRead(devAddr=self.i2c_address, rxSize=length)
         except Exception as e:
             raise McuBootConnectionError(str(e)) from e
         if result < 0 or not data:
-            raise TimeoutError()
+            raise SPSDKTimeoutError()
         logger.debug(f"<{' '.join(f'{b:02x}' for b in data)}>")
         return data
 
@@ -443,4 +443,4 @@ class UsbSioI2C(UsbSio):
         except Exception as e:
             raise McuBootConnectionError(str(e)) from e
         if result < 0:
-            raise TimeoutError()
+            raise SPSDKTimeoutError()
