@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -13,13 +13,16 @@ from datetime import datetime
 from typing import Iterator, List, Optional
 
 from spsdk import SPSDKError
+from spsdk.apps.utils import get_key
 from spsdk.crypto.loaders import load_certificate_as_bytes
+from spsdk.crypto.signature_provider import SignatureProvider
+from spsdk.exceptions import SPSDKUnsupportedOperation
 from spsdk.utils.crypto import CertBlockV2, Counter, crypto_backend
 from spsdk.utils.crypto.abstract import BaseClass
 from spsdk.utils.crypto.backend_internal import internal_backend
 from spsdk.utils.crypto.certificate import Certificate
 from spsdk.utils.crypto.common import calc_cypher_block_count
-from spsdk.utils.misc import find_first, load_binary, write_file
+from spsdk.utils.misc import find_first, load_text, write_file
 
 from . import sb_21_helper as elf2sb_helper21
 from . import sly_bd_parser as bd_parser
@@ -131,7 +134,7 @@ class BootImageV20(BaseClass):
         self._kek = kek
         # Set Flags value
         self._signed = signed
-        self._private_key_pem_data: Optional[bytes] = None
+        self._signature_provider: Optional[SignatureProvider] = None
         flags = 0x08 if self.signed else 0x04
         # Set private attributes
         self._dek: bytes = advanced_params.dek
@@ -183,17 +186,20 @@ class BootImageV20(BaseClass):
         return self._kek
 
     @property
-    def private_key_pem_data(self) -> Optional[bytes]:
-        """Return private key data for signed images, decrypted in PEM format."""
-        return self._private_key_pem_data
+    def signature_provider(self) -> Optional[SignatureProvider]:
+        """Signature provider instance to sign the final image.
 
-    @private_key_pem_data.setter
-    def private_key_pem_data(self, value: bytes) -> None:
-        """Setter to be used for signed images.
-
-        :param: key for signing the image; decrypted binary data in PEM format
+        None if not assigned yet.
         """
-        self._private_key_pem_data = value
+        return self._signature_provider
+
+    @signature_provider.setter
+    def signature_provider(self, value: SignatureProvider) -> None:
+        """Signature provider setter.
+
+        :param value: Signature provider to sign final image
+        """
+        self._signature_provider = value
 
     @property
     def signed(self) -> bool:
@@ -371,16 +377,26 @@ class BootImageV20(BaseClass):
             data += sect.export(dek=self.dek, mac=self.mac, counter=counter)
         # Add Signature data
         if self.signed:
-            private_key_pem_data = self.private_key_pem_data
-            if private_key_pem_data is None:
-                raise SPSDKError("Private key not assigned, cannot sign the image")
-            certificate_block = self.cert_block
-            if not (
-                (certificate_block is not None)
-                and certificate_block.verify_private_key(private_key_pem_data)
-            ):
-                raise SPSDKError("Invalid certificate block")
-            data += crypto_backend().rsa_sign(private_key_pem_data, data)
+            if self.signature_provider is None:
+                raise SPSDKError("Signature provider is not assigned, cannot sign the image.")
+            if self.cert_block is None:
+                raise SPSDKError("Certificate block is not assigned.")
+
+            public_key = self.cert_block.certificates[-1].public_key
+            try:
+                result = self.signature_provider.verify_public_key(public_key.dump())
+                if not result:
+                    raise SPSDKError(
+                        "Signature verification failed, public key does not match to certificate"
+                    )
+                logger.debug("The verification of private key pair integrity has been successful.")
+            except SPSDKUnsupportedOperation:
+                logger.warning(
+                    "Signature provider could not verify the integrity of private key pair."
+                )
+
+            data += self.signature_provider.sign(data)
+
         if len(data) != self.raw_size:
             raise SPSDKError("Invalid length of exported data")
         return data
@@ -394,11 +410,11 @@ class BootImageV20(BaseClass):
         :param offset: The offset of input data
         :param kek: The Key for unwrapping DEK and MAC keys (required)
         :return: parsed image object
-        :raise Exception: raised when header is in wrong format
-        :raise Exception: raised when there is invalid header version
-        :raise Exception: raised when signature is incorrect
+        :raises SPSDKError: raised when header is in wrong format
+        :raises SPSDKError: raised when there is invalid header version
+        :raises SPSDKError: raised when signature is incorrect
         :raises SPSDKError: Raised when kek is empty
-        :raises Exception: raised when header's nonce is not present
+        :raises SPSDKError: raised when header's nonce is not present
         """
         if not kek:
             raise SPSDKError("kek cannot be empty")
@@ -414,11 +430,11 @@ class BootImageV20(BaseClass):
         mac = key_blob_unwrap[32:]
         header_mac_data_calc = crypto_backend().hmac(mac, header_raw_data)
         if header_mac_data != header_mac_data_calc:
-            raise Exception()
+            raise SPSDKError("Invalid header MAC data")
         # Parse Header
         header = ImageHeaderV2.parse(header_raw_data)
         if header.version != "2.0":
-            raise Exception(f"Invalid Header Version: {header.version} instead 2.0")
+            raise SPSDKError(f"Invalid Header Version: {header.version} instead 2.0")
         image_size = header.image_blocks * 16
         # Initialize counter
         if not header.nonce:
@@ -447,7 +463,7 @@ class BootImageV20(BaseClass):
             if not cert_sect.cert_block.verify_data(
                 data[offset + image_size :], data[offset : offset + image_size]
             ):
-                raise Exception()
+                raise SPSDKError("Parsing Certification section failed")
         # Parse Boot Sections
         while index < (image_size + offset):
             boot_section = BootSectionV2.parse(data, index, dek=dek, mac=mac, counter=counter)
@@ -494,8 +510,8 @@ class BootImageV21(BaseClass):
         :param sections: Boot sections
         """
         self._kek = kek
-        self._private_key_pem_data: Optional[
-            bytes
+        self._signature_provider: Optional[
+            SignatureProvider
         ] = None  # this should be assigned for export, not needed for parsing
         self._dek = advanced_params.dek
         self._mac = advanced_params.mac
@@ -509,7 +525,7 @@ class BootImageV21(BaseClass):
             timestamp=advanced_params.timestamp,
         )
         self._cert_block: Optional[CertBlockV2] = None
-        self._boot_sections: List[BootSectionV2] = []
+        self.boot_sections: List[BootSectionV2] = []
         # ...
         for section in sections:
             self.add_boot_section(section)
@@ -535,20 +551,20 @@ class BootImageV21(BaseClass):
         return self._kek
 
     @property
-    def private_key_pem_data(self) -> Optional[bytes]:
-        """Return binary data of private key for signing; decrypted binary data in PEM format.
+    def signature_provider(self) -> Optional[SignatureProvider]:
+        """Signature provider instance to sign the final image.
 
-        None if not assigned yet or image not signed.
+        None if not assigned yet.
         """
-        return self._private_key_pem_data
+        return self._signature_provider
 
-    @private_key_pem_data.setter
-    def private_key_pem_data(self, value: bytes) -> None:
-        """Setter.
+    @signature_provider.setter
+    def signature_provider(self, value: SignatureProvider) -> None:
+        """Signature provider setter.
 
-        :param: key for signing the image; decrypted binary data in PEM format
+        :param value: Signature provider to sign final image
         """
-        self._private_key_pem_data = value
+        self._signature_provider = value
 
     @property
     def cert_block(self) -> Optional[CertBlockV2]:
@@ -595,26 +611,26 @@ class BootImageV21(BaseClass):
                 raise SPSDKError("Certificate block is not signed")
             size += cert_blk.signature_size
         # Boot Sections
-        for boot_section in self._boot_sections:
+        for boot_section in self.boot_sections:
             size += boot_section.raw_size
         return size
 
     def __len__(self) -> int:
-        return len(self._boot_sections)
+        return len(self.boot_sections)
 
     def __getitem__(self, key: int) -> BootSectionV2:
-        return self._boot_sections[key]
+        return self.boot_sections[key]
 
     def __setitem__(self, key: int, value: BootSectionV2) -> None:
-        self._boot_sections[key] = value
+        self.boot_sections[key] = value
 
     def __iter__(self) -> Iterator[BootSectionV2]:
-        return self._boot_sections.__iter__()
+        return self.boot_sections.__iter__()
 
     def update(self) -> None:
         """Update BootImageV21."""
-        if self._boot_sections:
-            self._header.first_boot_section_id = self._boot_sections[0].uid
+        if self.boot_sections:
+            self._header.first_boot_section_id = self.boot_sections[0].uid
             # calculate first boot tag block
             data_size = self._header.SIZE + self.HEADER_MAC_SIZE + self.KEY_BLOB_SIZE
             cert_blk = self.cert_block
@@ -632,7 +648,7 @@ class BootImageV21(BaseClass):
         )
         # Get HMAC count
         self._header.max_section_mac_count = 0
-        for boot_sect in self._boot_sections:
+        for boot_sect in self.boot_sections:
             boot_sect.is_last = True  # unified with elftosb
             self._header.max_section_mac_count += boot_sect.hmac_count
         # Update certificates block header
@@ -651,7 +667,7 @@ class BootImageV21(BaseClass):
             nfo += "::::::::::::::::::::::::::::::: CERTIFICATES BLOCK ::::::::::::::::::::::::::::::::::::\n"
             nfo += self.cert_block.info()
         nfo += "::::::::::::::::::::::::::::::::::: BOOT SECTIONS ::::::::::::::::::::::::::::::::::::\n"
-        for index, section in enumerate(self._boot_sections):
+        for index, section in enumerate(self.boot_sections):
             nfo += f"[ SECTION: {index} | UID: 0x{section.uid:08X} ]\n"
             nfo += section.info()
         return nfo
@@ -664,7 +680,7 @@ class BootImageV21(BaseClass):
         """
         if not isinstance(section, BootSectionV2):
             raise SPSDKError("Section is not instance of BootSectionV2 class")
-        self._boot_sections.append(section)
+        self.boot_sections.append(section)
 
     # pylint: disable=too-many-locals
     def export(
@@ -683,12 +699,12 @@ class BootImageV21(BaseClass):
         :raises SPSDKError: Raised when there is no debug info
         """
         # validate params
-        if not self._boot_sections:
+        if not self.boot_sections:
             raise SPSDKError("At least one Boot Section must be added")
         if self.cert_block is None:
             raise SPSDKError("Certificate is not assigned")
-        if self.private_key_pem_data is None:
-            raise SPSDKError("Private key not assigned, cannot sign the image")
+        if self.signature_provider is None:
+            raise SPSDKError("Signature provider is not assigned, cannot sign the image")
         # Update internals
         if dbg_info is not None:
             dbg_info.append("[sb_file]")
@@ -709,7 +725,7 @@ class BootImageV21(BaseClass):
         if not self._header.nonce:
             raise SPSDKError("Invalid header's nonce")
         counter = Counter(self._header.nonce, calc_cypher_block_count(bs_offset))
-        for sect in self._boot_sections:
+        for sect in self.boot_sections:
             bs_data += sect.export(
                 dek=self.dek, mac=self.mac, counter=counter, dbg_info=bs_dbg_info
             )
@@ -719,7 +735,7 @@ class BootImageV21(BaseClass):
             dbg_info.append("[header]")
             dbg_info.append(signed_data.hex())
         #  Add HMAC data
-        first_bs_hmac_count = self._boot_sections[0].hmac_count
+        first_bs_hmac_count = self.boot_sections[0].hmac_count
         hmac_data = bs_data[CmdHeader.SIZE : CmdHeader.SIZE + (first_bs_hmac_count * 32) + 32]
         hmac = crypto_backend().hmac(self.mac, hmac_data)
         signed_data += hmac
@@ -741,10 +757,20 @@ class BootImageV21(BaseClass):
         # Add SHA-256 of Bootable sections if requested
         if self.header.flags & self.FLAGS_SHA_PRESENT_BIT:
             signed_data += internal_backend.hash(bs_data)
+
         # Add Signature data
-        if not self.cert_block.verify_private_key(self.private_key_pem_data):
-            raise SPSDKError("Private key does not match certificate")
-        signature = crypto_backend().rsa_sign(self.private_key_pem_data, signed_data)
+        public_key = self.cert_block.certificates[-1].public_key
+        try:
+            result = self.signature_provider.verify_public_key(public_key.dump())
+            if not result:
+                raise SPSDKError(
+                    "Signature verification failed, public key does not match to certificate"
+                )
+            logger.debug("The verification of private key pair integrity has been successful.")
+        except SPSDKUnsupportedOperation:
+            logger.warning("Signature provider could not verify the integrity of private key pair.")
+
+        signature = self.signature_provider.sign(signed_data)
         if dbg_info:
             dbg_info.append("[signature]")
             dbg_info.append(signature.hex())
@@ -771,10 +797,10 @@ class BootImageV21(BaseClass):
         :param plain_sections: Sections are not encrypted; this is used only for debugging,
             not supported by ROM code
         :return: BootImageV21 parsed object
-        :raises Exception: raised when header is in incorrect format
-        :raises Exception: raised when signature is incorrect
+        :raises SPSDKError: raised when header is in incorrect format
+        :raises SPSDKError: raised when signature is incorrect
         :raises SPSDKError: Raised when kek is empty
-        :raises Exception: raised when header's nonce not present"
+        :raises SPSDKError: raised when header's nonce not present"
         """
         if not kek:
             raise SPSDKError("kek cannot be empty")
@@ -791,7 +817,7 @@ class BootImageV21(BaseClass):
         # Parse Header
         header = ImageHeaderV2.parse(header_raw_data)
         if header.offset_to_certificate_block != (index - offset):
-            raise Exception()
+            raise SPSDKError("Invalid offset")
         # Parse Certificate Block
         cert_block = CertBlockV2.parse(data, index)
         index += cert_block.raw_size
@@ -808,7 +834,7 @@ class BootImageV21(BaseClass):
         )
 
         if not result:
-            raise Exception()
+            raise SPSDKError("Verification failed")
         # Check flags, if 0x8000 bit is set, the SB file contains SHA-256 between
         # certificate and signature.
         if header.flags & BootImageV21.FLAGS_SHA_PRESENT_BIT:
@@ -819,7 +845,7 @@ class BootImageV21(BaseClass):
         # Not implemented yet
         # hmac_data_calc = crypto_backend().hmac(mac, data[index + CmdHeader.SIZE: index + CmdHeader.SIZE + ((2) * 32)])
         # if hmac_data != hmac_data_calc:
-        #    raise Exception()
+        #    raise SPSDKError("HMAC failed")
         if not header.nonce:
             raise SPSDKError("Header's nonce not present")
         counter = Counter(header.nonce)
@@ -855,10 +881,10 @@ class BootImageV21(BaseClass):
         return obj
 
 
-def generate_SB21(
+def generate_SB21(  # pylint: disable=invalid-name
     bd_file_path: str,
     key_file_path: str,
-    private_key_file_path: str,
+    signature_provider: SignatureProvider,
     signing_certificate_file_paths: List[str],
     root_key_certificate_paths: List[str],
     hoh_out_path: str,
@@ -868,8 +894,7 @@ def generate_SB21(
 
     :param bd_file_path: path to BD file.
     :param key_file_path: path to key file.
-    :param private_key_file_path: path to private key file for signing. This key
-        relates to last certificate from signing certificate chain.
+    :param signature_provider: Signature provider to sign final image
     :param signing_certificate_file_paths: signing certificate chain.
     :param root_key_certificate_paths: paths to root key certificate(s) for
         verifying other certificates. Only 4 root key certificates are allowed,
@@ -883,8 +908,7 @@ def generate_SB21(
     """
     # Create lexer and parser, load the BD file content and parse it for
     # further execution - the parsed BD file is a dictionary in JSON format
-    with open(str(bd_file_path)) as bd_file:
-        bd_file_content = bd_file.read()
+    bd_file_content = load_text(bd_file_path)
 
     parser = bd_parser.BDParser()
 
@@ -915,7 +939,7 @@ def generate_SB21(
         "flags", BootImageV21.FLAGS_SHA_PRESENT_BIT | BootImageV21.FLAGS_ENCRYPTED_SIGNED_BIT
     )
     if (
-        private_key_file_path is None
+        signature_provider is None
         or signing_certificate_file_paths is None
         or root_key_certificate_paths is None
     ):
@@ -950,10 +974,7 @@ def generate_SB21(
         logger.warning("No KEK key provided, using a zero KEK key")
         sb_kek = bytes.fromhex("0" * 64)
     else:
-        with open(str(key_file_path)) as kek_key_file:
-            # TODO maybe we should validate the key length and content, to make
-            # sure the key provided in the file is valid??
-            sb_kek = bytes.fromhex(kek_key_file.readline())
+        sb_kek = get_key(key_file_path, expected_size=32)
 
     # validate keyblobs and perform appropriate actions
     keyblobs = parsed_bd_file.get("keyblobs", [])
@@ -1018,7 +1039,7 @@ def generate_SB21(
     # We have our secure binary, now we attach to it the certificate block and
     # the private key content
     secure_binary.cert_block = cert_block
-    secure_binary.private_key_pem_data = load_binary(private_key_file_path)
+    secure_binary.signature_provider = signature_provider
 
     if hoh_out_path is None:
         hoh_out_path = os.path.join(os.getcwd(), "hash.bin")
