@@ -1,25 +1,35 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2021-2022 NXP
+# Copyright 2021-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module used for generation SecureBinary V3.1."""
+import logging
+import os
+from copy import deepcopy
 from datetime import datetime
-from struct import calcsize, pack
-from typing import Any, Dict, List
+from struct import calcsize, pack, unpack_from
+from typing import Any, Dict, List, Optional
 
 from spsdk import SPSDKError
-from spsdk.image import MBIMG_SCH_FILE, SB3_SCH_FILE
+from spsdk.crypto.signature_provider import SignatureProvider, get_signature_provider
+from spsdk.image import IMG_DATA_FOLDER, MBIMG_SCH_FILE
 from spsdk.sbfile.sb31.commands import CFG_NAME_TO_CLASS, CmdSectionHeader, MainCmd
 from spsdk.sbfile.sb31.functions import KeyDerivator
 from spsdk.utils.crypto import CRYPTO_SCH_FILE
 from spsdk.utils.crypto.abstract import BaseClass
 from spsdk.utils.crypto.backend_internal import internal_backend
 from spsdk.utils.crypto.cert_blocks import CertBlockV31
-from spsdk.utils.misc import align_block, load_binary, load_text, value_to_int
+from spsdk.utils.database import Database
+from spsdk.utils.misc import align_block, load_text, value_to_int
 from spsdk.utils.schema_validator import ConfigTemplate, ValidationSchemas
 
+logger = logging.getLogger(__name__)
+
+SB3_SCH_FILE: str = os.path.join(IMG_DATA_FOLDER, "sch_sb3.yaml")
+DATABASE_FILE = os.path.join(IMG_DATA_FOLDER, "database_sb31.yaml")
+
 
 ########################################################################################################################
 # Secure Boot Image Class (Version 3.1)
@@ -37,17 +47,17 @@ class SecureBinary31Header(BaseClass):
         self,
         firmware_version: int,
         curve_name: str,
-        description: str = None,
-        timestamp: int = None,
+        description: Optional[str] = None,
+        timestamp: Optional[int] = None,
         is_nxp_container: bool = False,
         flags: int = 0,
     ) -> None:
         """Initialize the SecureBinary V3.1 Header.
 
-        :param firmware_version: Firmaware version (must be bigger than current CMPA record)
+        :param firmware_version: Firmware version (must be bigger than current CMPA record)
         :param curve_name: Name of the ECC curve used for Secure binary (secp256r1/secp384r1)
         :param description: Custom description up to 16 characters long, defaults to None
-        :param timestamp: Timestap (number of seconds since Jan 1st, 200), if None use current time
+        :param timestamp: Timestamp (number of seconds since Jan 1st, 200), if None use current time
         :param is_nxp_container: NXP provisioning SB file, defaults to False
         :param flags: Flags for SB file, defaults to 0
         """
@@ -62,7 +72,7 @@ class SecureBinary31Header(BaseClass):
         self.cert_block_offset = self.calculate_cert_block_offset()
         self.description = self._adjust_description(description)
 
-    def _adjust_description(self, description: str = None) -> bytes:
+    def _adjust_description(self, description: Optional[str] = None) -> bytes:
         """Format the description."""
         if not description:
             return bytes(self.DESCRIPTION_LENGTH)
@@ -100,11 +110,21 @@ class SecureBinary31Header(BaseClass):
         info += f" Firmware version:            {self.firmware_version}\n"
         info += f" Image type:                  {self.image_type}\n"
         info += f" Timestamp:                   {self.timestamp}\n"
-        info += f" Total length of image:       {self.image_total_length}\n"
+        info += f" Total length of Block#0:     {self.image_total_length}\n"
         info += f" Certificate block offset:    {self.cert_block_offset}\n"
         info += f" Description:                 {self.description.decode('ascii')}\n"
         return info
 
+    def update(self, commands: "SecureBinary31Commands", cert_block: CertBlockV31) -> None:
+        """Updates the volatile fields in header by real commands and certification block data.
+
+        :param commands: SB3.1 Commands block
+        :param cert_block: SB3.1 Certification block.
+        """
+        self.block_count = commands.block_count
+        self.image_total_length += len(commands.final_hash) + cert_block.expected_size
+        self.image_total_length += 2 * len(commands.final_hash)
+
     def export(self) -> bytes:
         """Serialize the SB file to bytes."""
         major_format_version, minor_format_version = [
@@ -130,9 +150,43 @@ class SecureBinary31Header(BaseClass):
     def parse(cls, data: bytes, offset: int = 0) -> "SecureBinary31Header":
         """Parse binary data into SecureBinary31Header.
 
-        :raises NotImplementedError: Not yet implemented
+        :raises SPSDKError: Unable to parse SB31 Header.
         """
-        raise NotImplementedError("Not yet implemented.")
+        (
+            magic,
+            minor_version,
+            major_version,
+            flags,
+            block_count,
+            block_size,
+            timestamp,
+            firmware_version,
+            image_total_length,
+            image_type,
+            cert_block_offset,
+            description,
+        ) = unpack_from(cls.HEADER_FORMAT, data, offset=offset)
+        if magic != cls.MAGIC:
+            raise SPSDKError("Magic doesn't match")
+        if major_version != 3 and minor_version != 1:
+            raise SPSDKError(f"Unable to parse SB version {major_version}.{minor_version}")
+        if block_size not in [292, 308]:
+            raise SPSDKError(f"Unable to determine curve name from block size: {block_size}")
+
+        curve_name = "secp256r1" if block_size == 292 else "secp384r1"
+        obj = SecureBinary31Header(
+            firmware_version=firmware_version,
+            curve_name=curve_name,
+            description=description.decode("utf-8"),
+            timestamp=timestamp,
+            is_nxp_container=image_type == 7,
+            flags=flags,
+        )
+        obj.block_count = block_count
+        obj.block_size = block_size
+        obj.cert_block_offset = cert_block_offset
+        obj.image_total_length = image_total_length
+        return obj
 
     def validate(self) -> None:
         """Validate the settings of class members.
@@ -168,11 +222,12 @@ class SecureBinary31Commands(BaseClass):
 
     def __init__(
         self,
+        family: str,
         curve_name: str,
         is_encrypted: bool = True,
-        pck: bytes = None,
-        timestamp: int = None,
-        kdk_access_rights: int = None,
+        pck: Optional[bytes] = None,
+        timestamp: Optional[int] = None,
+        kdk_access_rights: Optional[int] = None,
     ) -> None:
         """Initialize container for SB3.1 commands.
 
@@ -184,6 +239,7 @@ class SecureBinary31Commands(BaseClass):
         :raises SPSDKError: Key derivation arguments are not provided if `is_encrypted` is True
         """
         super().__init__()
+        self.family = family
         self.curve_name = curve_name
         self.hash_type = self._get_hash_type(curve_name)
         self.is_encrypted = is_encrypted
@@ -217,19 +273,34 @@ class SecureBinary31Commands(BaseClass):
         """Add SB3.1 command."""
         self.commands.append(command)
 
+    def insert_command(self, index: int, command: MainCmd) -> None:
+        """Insert SB3.1 command."""
+        if index == -1:
+            self.commands.append(command)
+        else:
+            self.commands.insert(index, command)
+
     def set_commands(self, commands: List[MainCmd]) -> None:
         """Set all SB3.1 commands at once."""
         self.commands = commands.copy()
 
-    def load_from_config(self, config: List[Dict[str, Any]]) -> None:
+    def load_from_config(
+        self, config: List[Dict[str, Any]], search_paths: Optional[List[str]] = None
+    ) -> None:
         """Load configuration from dictionary.
 
         :param config: Dictionary with configuration fields.
+        :param search_paths: List of paths where to search for the file, defaults to None
         """
         for cfg_cmd in config:
             cfg_cmd_key = list(cfg_cmd.keys())[0]
             cfg_cmd_value = cfg_cmd[cfg_cmd_key]
-            self.add_command(CFG_NAME_TO_CLASS[cfg_cmd_key].load_from_config(cfg_cmd_value))
+            cfg_cmd_value["family"] = self.family
+            self.add_command(
+                CFG_NAME_TO_CLASS[cfg_cmd_key].load_from_config(
+                    cfg_cmd_value, search_paths=search_paths
+                )
+            )
 
     def get_cmd_blocks_to_export(self) -> List[bytes]:
         """Export commands as bytes."""
@@ -312,24 +383,25 @@ class SecureBinary31(BaseClass):
 
     def __init__(
         self,
+        family: str,
         curve_name: str,
         cert_block: CertBlockV31,
         firmware_version: int,
-        signing_key: bytes,
-        pck: bytes = None,
-        kdk_access_rights: int = None,
-        description: str = None,
+        signature_provider: SignatureProvider,
+        pck: Optional[bytes] = None,
+        kdk_access_rights: Optional[int] = None,
+        description: Optional[str] = None,
         is_nxp_container: bool = False,
         flags: int = 0,
-        timestamp: int = None,
+        timestamp: Optional[int] = None,
         is_encrypted: bool = True,
     ) -> None:
         """Constructor for Secure Binary v3.1 data container.
 
         :param curve_name: Name of the ECC curve used for Secure binary (secp256r1/secp384r1).
         :param cert_block: Certification block.
-        :param firmware_version: Firmaware version (must be bigger than current CMPA record).
-        :param signing_key: Key to final sign of SB3.1 image.
+        :param firmware_version: Firmware version (must be bigger than current CMPA record).
+        :param signature_provider: Signature provider for final sign of SB3.1 image.
         :param pck: Part Common Key (needed if `is_encrypted` is True), defaults to None
         :param kdk_access_rights: Key Derivation Key access rights (needed if `is_encrypted` is True), defaults to None
         :param description: Custom description up to 16 characters long, defaults to None
@@ -339,6 +411,7 @@ class SecureBinary31(BaseClass):
         :param is_encrypted: Indicate whether commands should be encrypted or not, defaults to True
         """
         # in our case, timestamp is the number of seconds since "Jan 1, 2000"
+        self.family = family
         self.timestamp = timestamp or int((datetime.now() - datetime(2000, 1, 1)).total_seconds())
         self.pck = pck
         self.cert_block: CertBlockV31 = cert_block
@@ -349,7 +422,7 @@ class SecureBinary31(BaseClass):
         self.description = description
         self.is_nxp_container = is_nxp_container
         self.flags = flags
-        self.signing_key = signing_key
+        self.signature_provider = signature_provider
 
         self.sb_header = SecureBinary31Header(
             firmware_version=self.firmware_version,
@@ -360,19 +433,58 @@ class SecureBinary31(BaseClass):
             flags=self.flags,
         )
         self.sb_commands = SecureBinary31Commands(
+            family=self.family,
             curve_name=curve_name,
             is_encrypted=self.is_encrypted,
             pck=pck,
             timestamp=self.timestamp,
             kdk_access_rights=self.kdk_access_rights,
         )
+        if self.pck:
+            logger.info(f"SB3KDK: {self.pck.hex()}")
+
+    @staticmethod
+    def _get_prv_key_length(curve_name: str) -> int:
+        """Get size of key for curve in bits."""
+        return {"secp256r1": 256, "secp384r1": 384}[curve_name]
+
+    @classmethod
+    def get_validation_schemas_family(cls) -> List[Dict[str, Any]]:
+        """Create the validation schema just for supported families.
+
+        :return: List of validation schemas for SB31 supported families.
+        """
+        sch_cfg = ValidationSchemas.get_schema_file(SB3_SCH_FILE)
+        sch_cfg["sb3_family"]["properties"]["family"]["enum"] = cls.get_supported_families()
+        return [sch_cfg["sb3_family"]]
+
+    @classmethod
+    def get_commands_validation_schemas(cls, family: str) -> List[Dict[str, Any]]:
+        """Create the list of validation schemas.
+
+        :param family: Family description.
+        :return: List of validation schemas.
+        """
+        sb3_sch_cfg = ValidationSchemas().get_schema_file(SB3_SCH_FILE)
+
+        schemas: List[Dict[str, Any]] = [deepcopy(sb3_sch_cfg["sb3_commands"])]
+
+        # remove unused command for current family
+        supported_commands = Database(DATABASE_FILE).get_device_value("supported_commands", family)
+        list_of_commands: List[Dict] = schemas[0]["properties"]["commands"]["items"]["oneOf"]
+        for command in list_of_commands:
+            if list(command["properties"].keys())[0] not in supported_commands:
+                list_of_commands.remove(command)
+
+        return schemas
 
     @classmethod
     def get_validation_schemas(
-        cls, include_test_configuration: bool = False
+        cls, family: str, include_test_configuration: bool = False
     ) -> List[Dict[str, Any]]:
         """Create the list of validation schemas.
 
+        :param family: Family description.
         :param include_test_configuration: Add also testing configuration schemas.
         :return: List of validation schemas.
         """
@@ -380,33 +492,41 @@ class SecureBinary31(BaseClass):
         crypto_sch_cfg = ValidationSchemas().get_schema_file(CRYPTO_SCH_FILE)
         sb3_sch_cfg = ValidationSchemas().get_schema_file(SB3_SCH_FILE)
 
-        ret: List[Dict[str, Any]] = []
-        ret.extend(
+        schemas: List[Dict[str, Any]] = []
+        schemas.extend(
             [
                 mbi_sch_cfg[x]
                 for x in [
                     "firmware_version",
-                    "use_isk",
-                    "signing_cert_prv_key",
-                    "signing_root_prv_key",
                     "signing_prv_key_lpc55s3x",
                     "elliptic_curves",
                 ]
             ]
         )
-        ret.extend([crypto_sch_cfg[x] for x in ["certificate_v31", "certificate_root_keys"]])
-        ret.extend([sb3_sch_cfg[x] for x in ["sb3_family", "sb3", "sb3_commands"]])
+        schemas.extend([crypto_sch_cfg[x] for x in ["certificate_v31", "certificate_root_keys"]])
+        schemas.extend([sb3_sch_cfg[x] for x in ["sb3_family", "sb3", "sb3_description"]])
+        schemas.extend(cls.get_commands_validation_schemas(family))
         if include_test_configuration:
-            ret.append(sb3_sch_cfg["sb3_test"])
-        return ret
+            schemas.append(sb3_sch_cfg["sb3_test"])
+
+        # find family
+        for schema in schemas:
+            if "properties" in schema and "family" in schema["properties"]:
+                schema["properties"]["family"]["enum"] = cls.get_supported_families()
+                break
+        return schemas
 
     @classmethod
-    def load_from_config(cls, config: Dict[str, Any]) -> "SecureBinary31":
-        """Creates instantion of SecureBinary31 from configuration.
+    def load_from_config(
+        cls, config: Dict[str, Any], search_paths: Optional[List[str]] = None
+    ) -> "SecureBinary31":
+        """Creates an instance of SecureBinary31 from configuration.
 
         :param config: Input standard configuration.
-        :return: Instantion of Secure Binary V3.1 class
+        :param search_paths: List of paths where to search for the file, defaults to None
+        :return: Instance of Secure Binary V3.1 class
         """
+        family = config["family"]
         container_keyblob_enc_key_path = config.get("containerKeyBlobEncryptionKey")
         is_nxp_container = config.get("isNxpContainer", False)
         description = config.get("description")
@@ -420,7 +540,7 @@ class SecureBinary31(BaseClass):
         if timestamp:  # re-format it
             timestamp = value_to_int(timestamp)
 
-        cert_block = CertBlockV31.from_config(config)
+        cert_block = CertBlockV31.from_config(config, search_paths=search_paths)
 
         # if use_isk is set, we use for signing the ISK certificate instead of root
         signing_key_path = (
@@ -435,16 +555,24 @@ class SecureBinary31(BaseClass):
         )
         assert curve_name and isinstance(curve_name, str)
         assert signing_key_path
-        signing_key = load_binary(signing_key_path) if signing_key_path else None
-        assert signing_key
+        signature_provider = get_signature_provider(
+            sp_cfg=config.get("signProvider"),
+            local_file_key=signing_key_path,
+            search_paths=search_paths,
+            mode="deterministic-rfc6979",
+        )
+        assert signature_provider
 
         pck = None
         if is_encrypted:
             assert container_keyblob_enc_key_path
-            pck = bytes.fromhex(load_text(container_keyblob_enc_key_path))
+            pck = bytes.fromhex(
+                load_text(container_keyblob_enc_key_path, search_paths=search_paths)
+            )
 
         # Create SB3 object
         sb3 = SecureBinary31(
+            family=family,
             pck=pck,
             cert_block=cert_block,
             curve_name=curve_name,
@@ -453,13 +581,13 @@ class SecureBinary31(BaseClass):
             description=description,
             is_nxp_container=is_nxp_container,
             flags=container_configuration_word,
-            signing_key=signing_key,
+            signature_provider=signature_provider,
             timestamp=timestamp,
             is_encrypted=is_encrypted,
         )
 
         # Add commands into the SB3 object
-        sb3.sb_commands.load_from_config(commands)
+        sb3.sb_commands.load_from_config(commands, search_paths=search_paths)
 
         return sb3
 
@@ -468,30 +596,41 @@ class SecureBinary31(BaseClass):
 
         :raises SPSDKError: Invalid configuration of SB3.1 class members.
         """
-        # TODO add checking length of signing key
-        if self.signing_key is None or not isinstance(self.signing_key, bytes):
-            raise SPSDKError(f"SB3.1 Signing Key is invalid: {self.signing_key}")
+        if self.signature_provider is None or not isinstance(
+            self.signature_provider, SignatureProvider
+        ):
+            raise SPSDKError(f"SB3.1 signature provider is invalid: {self.signature_provider}")
 
+        try:
+            prv_key_length = self._get_prv_key_length(self.curve_name)
+        except KeyError as exc:
+            raise SPSDKError(f"Invalid SB3 curve name: ({self.curve_name})") from exc
+
+        if (self.signature_provider.signature_length / 2) * 8 != prv_key_length:
+            raise SPSDKError(
+                f"Invalid length of SB3.1 signing key({self.signature_provider.signature_length} != {prv_key_length})"
+                f" for used curve: {self.curve_name}!"
+            )
         self.cert_block.validate()
         self.sb_header.validate()
         self.sb_commands.validate()
 
-    def export(self) -> bytes:
+    def export(self, cert_block: Optional[bytes] = None) -> bytes:
         """Generate binary output of SB3.1 file.
 
         :return: Content of SB3.1 file in bytes.
         """
         self.validate()
 
-        cert_block_data = self.cert_block.export()
+        if cert_block:
+            cert_block_data = cert_block
+        else:
+            cert_block_data = self.cert_block.export()
         sb3_commands_data = self.sb_commands.export()
-        self.sb_header.block_count = self.sb_commands.block_count
-        self.sb_header.image_total_length += len(self.sb_commands.final_hash) + len(cert_block_data)
-        # TODO: use proper signature len calculation
-        self.sb_header.image_total_length += 2 * len(self.sb_commands.final_hash)
 
         final_data = bytes()
         # HEADER OF SB 3.1 FILE
+        self.sb_header.update(self.sb_commands, self.cert_block)
         final_data += self.sb_header.export()
 
         # HASH OF PREVIOUS BLOCK
@@ -499,7 +638,7 @@ class SecureBinary31(BaseClass):
         final_data += cert_block_data
 
         # SIGNATURE
-        final_data += internal_backend.ecc_sign(self.signing_key, final_data)
+        final_data += self.signature_provider.sign(final_data)
 
         # COMMANDS BLOBS DATA
         final_data += sb3_commands_data
@@ -528,10 +667,7 @@ class SecureBinary31(BaseClass):
 
         :return: List of supported families.
         """
-        sb3_sch_cfg = ValidationSchemas().get_schema_file(SB3_SCH_FILE)
-        sb3_families = sb3_sch_cfg["sb3_family"]
-
-        return sb3_families["properties"]["family"]["enum"]
+        return Database(DATABASE_FILE).devices.device_names
 
     @classmethod
     def generate_config_template(cls, family: str) -> Dict[str, str]:
@@ -543,7 +679,7 @@ class SecureBinary31(BaseClass):
         ret: Dict[str, str] = {}
 
         if family in cls.get_supported_families():
-            schemas = cls.get_validation_schemas()
+            schemas = cls.get_validation_schemas(family)
             schemas.append(ValidationSchemas.get_schema_file(SB3_SCH_FILE)["sb3_output"])
             override = {}
             override["family"] = family
