@@ -1,13 +1,12 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """The shadow registers control DAT support file."""
+
 import logging
-import math
-import os
 from typing import Any, List, Optional
 
 from ruamel.yaml import YAML
@@ -18,9 +17,16 @@ from spsdk.dat.debug_mailbox import DebugMailbox
 from spsdk.dat.dm_commands import StartDebugSession
 from spsdk.debuggers.debug_probe import DebugProbe, SPSDKDebugProbeError
 from spsdk.debuggers.utils import test_ahb_access
-from spsdk.utils.misc import change_endianness, reverse_bytes_in_longs, value_to_bytes, value_to_int
+from spsdk.utils.misc import (
+    change_endianness,
+    load_configuration,
+    value_to_bytes,
+    value_to_int,
+    write_file,
+)
 from spsdk.utils.reg_config import RegConfig
 from spsdk.utils.registers import Registers, RegsRegister, SPSDKRegsErrorRegisterNotFound
+from spsdk.utils.schema_validator import ConfigTemplate
 
 logger = logging.getLogger(__name__)
 
@@ -48,7 +54,11 @@ class ShadowRegisters:
 
         self.regs = Registers(self.device)
         rev = revision or "latest"
-        self.revision = rev if rev != "latest" else config.get_latest_revision(self.device)
+        self.revision = (
+            rev
+            if rev != "latest"
+            else config.devices.get_by_name(self.device).revisions.get_latest().name
+        )
         self.regs.load_registers_from_xml(
             config.get_data_file(self.device, self.revision),
             grouped_regs=config.get_grouped_registers(self.device),
@@ -88,7 +98,8 @@ class ShadowRegisters:
             read_back = self.probe.mem_reg_read(addr)
             if read_back & verify_mask != data & verify_mask:
                 raise IoVerificationError(
-                    f"The written data 0x{data:08X} to 0x{addr:08X} address are invalid."
+                    f"The verification of written shadow register 0x{addr:08X} failed."
+                    " Maybe a READ LOCK is set for that register."
                 )
 
     def reload_registers(self) -> None:
@@ -96,14 +107,13 @@ class ShadowRegisters:
         for reg in self.regs.get_registers():
             self.reload_register(reg)
 
-    def sets_all_registers(self) -> None:
-        """Update all shadow registers in target by local values."""
+    def sets_all_registers(self, verify: bool = True) -> None:
+        """Update all shadow registers in target by local values.
+
+        :param verify: Verity write operation.
+        """
         for reg in self.regs.get_registers():
-            if reg.has_group_registers():
-                for subreg in reg.sub_regs:
-                    self.set_register(subreg.name, subreg.get_value())
-            else:
-                self.set_register(reg.name, reg.get_value())
+            self.set_register(reg.name, reg.get_value(), verify)
 
     def reload_register(self, reg: RegsRegister) -> None:
         """Reload the value in requested register.
@@ -119,12 +129,14 @@ class ShadowRegisters:
         """
         return bytes(self.get_register(reg.name))
 
-    def set_register(self, reg_name: str, data: Any) -> None:
+    def set_register(self, reg_name: str, data: Any, verify: bool = True) -> None:
         """The function sets the value of the specified register.
 
-        param reg: The register name.
-        param data: The new data to be stored to shadow register.
-        raises SPSDKDebugProbeError: The debug probe is not specified.
+        :param reg_name: The register name.
+        :param data: The new data to be stored to shadow register.
+        :param verify: Verity write operation.
+        :raises SPSDKDebugProbeError: The debug probe is not specified.
+        :raises SPSDKError: General error with write of Shadow register.
         """
 
         def write_reg(base_address: int, reg: RegsRegister) -> None:
@@ -142,13 +154,14 @@ class ShadowRegisters:
             if reg.reverse:
                 value = int.from_bytes(change_endianness(value_to_bytes(value)), "big")
             # Create verify mask
-            bitfields = reg.get_bitfields()
             verify_mask = 0
-            if bitfields:
-                for bitfield in bitfields:
-                    verify_mask = verify_mask | (((1 << bitfield.width) - 1) << bitfield.offset)
-            else:
-                verify_mask = (1 << reg.width) - 1
+            if verify:
+                bitfields = reg.get_bitfields()
+                if bitfields:
+                    for bitfield in bitfields:
+                        verify_mask = verify_mask | (((1 << bitfield.width) - 1) << bitfield.offset)
+                else:
+                    verify_mask = (1 << reg.width) - 1
 
             self._write_shadow_reg(
                 addr=base_address + reg.offset, data=value, verify_mask=verify_mask
@@ -158,11 +171,12 @@ class ShadowRegisters:
             reg = self.regs.find_reg(reg_name, include_group_regs=True)
             reg.set_value(data, raw=True)
             if reg.has_group_registers():
-                for sub_reg in reg.sub_regs:
+                for sub_reg in reg.sub_regs[:: -1 if reg.reverse_subregs_order else 1]:
                     write_reg(self.offset, sub_reg)
             else:
                 write_reg(self.offset, reg=reg)
-
+            # execute flash function handler if defined for a platfrom
+            self.flush_func_handler()
         except SPSDKError as exc:
             raise SPSDKError(f"The set shadow register failed({str(exc)}).") from exc
 
@@ -193,8 +207,7 @@ class ShadowRegisters:
 
             ret = bytearray()
             if reg.has_group_registers():
-
-                for sub_reg in reg.sub_regs:
+                for sub_reg in reg.sub_regs[:: -1 if reg.reverse_subregs_order else 1]:
                     ret.extend(read_reg(self.offset, sub_reg))
             else:
                 ret.extend(read_reg(self.offset, reg=reg))
@@ -236,10 +249,12 @@ class ShadowRegisters:
                 reg.set_value(reg.get_value())
             except SPSDKRegsErrorRegisterNotFound as exc:
                 self.fuse_mode = False
-                raise SPSDKError(f"Register {reg_name} has not found for {self.device} device.")
+                raise SPSDKError(
+                    f"Register {reg_name} has not found for {self.device} device."
+                ) from exc
 
             if reg.has_group_registers():
-                for sub_reg in reg.sub_regs:
+                for sub_reg in reg.sub_regs[:: -1 if reg.reverse_subregs_order else 1]:
                     ret += add_reg(sub_reg)
             else:
                 ret += add_reg(reg)
@@ -247,7 +262,7 @@ class ShadowRegisters:
         self.fuse_mode = False
         return ret
 
-    def create_yml_config(self, file_name: str, raw: bool = False, diff: bool = False) -> None:
+    def create_yaml_config(self, file_name: str, raw: bool = False, diff: bool = False) -> None:
         """The function creates the configuration YML file.
 
         :param file_name: The file_name (without extension) of stored configuration.
@@ -279,14 +294,9 @@ class ShadowRegisters:
             indent=2,
             diff=diff,
         )
+        write_file(ConfigTemplate.convert_cm_to_yaml(data), file_name, encoding="utf8")
 
-        if not os.path.exists(os.path.dirname(file_name)):
-            os.makedirs(os.path.dirname(file_name))
-
-        with open(file_name, "w", encoding="utf8") as out_file:
-            yaml.dump(data, out_file)
-
-    def load_yml_config(self, file_name: str, raw: bool = False) -> None:
+    def load_yaml_config(self, file_name: str, raw: bool = False) -> None:
         """The function loads the configuration from YML file.
 
         :param file_name: The file_name (without extension) of stored configuration.
@@ -295,14 +305,7 @@ class ShadowRegisters:
         """
         antipole_regs = None if raw else list(self.config.get_antipole_regs(self.device).values())
         computed_fields = None if raw else self.config.get_computed_fields(self.device)
-        try:
-            with open(file_name, "r", encoding="utf8") as yml_config_file:
-                yaml = YAML()
-                yaml.indent(sequence=4, offset=2)
-                data = yaml.load(yml_config_file)
-        except FileNotFoundError as exc:
-            raise SPSDKError("File with YML configuration doesn't exists.") from exc
-
+        data = load_configuration(file_name)
         self.regs.load_yml_config(data["registers"], antipole_regs, computed_fields)
         if not raw:
             # Just update only configured registers
@@ -336,6 +339,20 @@ class ShadowRegisters:
         new_val = val ^ 0xFFFFFFFF
         return new_val
 
+    def flush_func_handler(self) -> None:
+        """A function to determine and execute the flush-func handler.
+
+        :param self: Input Value.
+        :raises SPSDKError: Raises when the computing routine is not found.
+        """
+        flush_func = self.config.get_device_value("flush_func", device=self.device)
+        if flush_func:
+            if hasattr(self, flush_func):
+                method_ref = getattr(self, flush_func)
+                method_ref()
+            else:
+                raise SPSDKError(f"The '{flush_func}' function doesn't exists.")
+
     def reg_computed_fields_handler(self, val: bytes, context: Any) -> bytes:
         """Recalculate all fields for given register value.
 
@@ -382,8 +399,8 @@ class ShadowRegisters:
             return (crc & 0xFF) ^ 0x55
         return crc & 0xFF
 
-    # @staticmethod
-    def comalg_dcfg_cc_socu_crc8(self, val: int) -> int:
+    @staticmethod
+    def comalg_dcfg_cc_socu_crc8(val: int) -> int:
         """Function that creates the crc for DCFG_CC_SOCU.
 
         :param val: Input DCFG_CC_SOCU Value.
@@ -396,7 +413,6 @@ class ShadowRegisters:
         val |= ShadowRegisters.crc_update(in_val)
         return val
 
-    # @staticmethod
     def comalg_dcfg_cc_socu_test_en(self, val: int) -> int:
         """Function fill up the DCFG_CC_SOCU DEV_TEST_EN set to True to satisfy MCU needs.
 
@@ -408,8 +424,17 @@ class ShadowRegisters:
 
         return val | 0x80000000
 
-    # @staticmethod
-    def comalg_do_nothing(self, val: int) -> int:
+    def rw61x_update_scratch_reg(self) -> None:
+        """Function updates scracth register for RW61x, This enables the shadow register functionality.
+
+        :param self: Input Value.
+        """
+        addr = 0x5003B498
+        value = 0xA7C56B9E
+        self._write_shadow_reg(addr=addr, data=value, verify_mask=True)
+
+    @staticmethod
+    def comalg_do_nothing(val: int) -> int:
         """Function that do nothing.
 
         :param val: Input Value.
