@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -9,7 +9,7 @@
 import importlib
 import math
 from struct import pack, unpack_from
-from typing import Any, Union
+from typing import Any, Optional, Union
 
 # Used security modules
 from Crypto import Hash, Random
@@ -161,13 +161,14 @@ class Backend(BackendClass):
             raise SPSDKError(f"Integrity Check Failed: {a:016X} (expected {iv:016X})")
         return b"".join(r[1:])
 
-    @staticmethod
-    def aes_cbc_encrypt(key: bytes, plain_data: bytes, iv: bytes = None) -> bytes:
+    def aes_cbc_encrypt(
+        self, key: bytes, plain_data: bytes, iv_data: Optional[bytes] = None
+    ) -> bytes:
         """Encrypt plain data with AES in CBC mode.
 
         :param key: Key for encryption
         :param plain_data: Data to encrypt
-        :param iv: Initial vector for encryption, defaults to None
+        :param iv_data: Initial vector for encryption, defaults to None
         :return: Encrypted data
         :raises SPSDKError: Incorrect key or initialization vector size
         """
@@ -175,12 +176,33 @@ class Backend(BackendClass):
             raise SPSDKError(
                 f"The key must be a valid AES key length: {', '.join([str(k) for k in AES.key_size])}"
             )
-        init_vector = iv or bytes(AES.block_size)
+        init_vector = iv_data or bytes(AES.block_size)
         if len(init_vector) != AES.block_size:
             raise SPSDKError(f"The initial vector length must be {AES.block_size}")
         cipher = AES.new(key, mode=AES.MODE_CBC, iv=init_vector)
         return cipher.encrypt(plain_data)
 
+    def aes_cbc_decrypt(
+        self, key: bytes, encrypted_data: bytes, iv_data: Optional[bytes] = None
+    ) -> bytes:
+        """Decrypt encrypted data with AES in CBC mode.
+
+        :param key: The key for data decryption
+        :param encrypted_data: Input data
+        :param iv_data: Initialization vector data
+        :raises SPSDKError: Invalid Key or IV
+        :return: Decrypted image
+        """
+        if len(key) not in AES.key_size:
+            raise SPSDKError(
+                f"The key must be a valid AES key length: {', '.join([str(k) for k in AES.key_size])}"
+            )
+        init_vector = iv_data or bytes(AES.block_size)
+        if len(init_vector) != AES.block_size:
+            raise SPSDKError(f"The initial vector length must be {AES.block_size}")
+        cipher = AES.new(key, mode=AES.MODE_CBC, iv=init_vector)
+        return cipher.decrypt(encrypted_data)
+
     def aes_ctr_encrypt(self, key: bytes, plain_data: bytes, nonce: bytes) -> bytes:
         """Encrypt plain data with AES in CTR mode.
 
@@ -212,6 +234,28 @@ class Backend(BackendClass):
         """
         return self.aes_ctr_encrypt(key, encrypted_data, nonce)
 
+    def aes_xts_encrypt(self, key: bytes, plain_data: bytes, tweak: bytes) -> bytes:
+        """Encrypt plain data with AES in XTS mode.
+
+        :param key: The key for data encryption
+        :param plain_data: Input data
+        :param tweak: The tweak is a 16 byte value
+        :return: Encrypted data
+        :raises NotImplementedError: This backend doesn't support AES-XTS
+        """
+        raise NotImplementedError("This backend doesn't support AES-XTS")
+
+    def aes_xts_decrypt(self, key: bytes, encrypted_data: bytes, tweak: bytes) -> bytes:
+        """Decrypt encrypted data with AES in XTS mode.
+
+        :param key: The key for data decryption
+        :param encrypted_data: Input data
+        :param tweak: The tweak is a 16 byte value
+        :return: Decrypted data
+        :raises NotImplementedError: This backend doesn't support AES-XTS
+        """
+        raise NotImplementedError("This backend doesn't support AES-XTS")
+
     def rsa_sign(
         self,
         private_key: Union[RSA.RsaKey, bytes],
@@ -270,7 +314,7 @@ class Backend(BackendClass):
         return RSA.construct((modulus, exponent))
 
     def ecc_sign(
-        self, private_key: Union[ECC.EccKey, bytes], data: bytes, algorithm: str = None
+        self, private_key: Union[ECC.EccKey, bytes], data: bytes, algorithm: Optional[str] = None
     ) -> bytes:
         """Sign data using (EC)DSA.
 
@@ -287,30 +331,32 @@ class Backend(BackendClass):
 
     def ecc_verify(
         self,
-        key: Union[ECC.EccKey, bytes],  # TODO  - could we renamed abstract class to "key" only?
+        public_key: Union[ECC.EccKey, bytes],
         signature: bytes,
         data: bytes,
-        algorithm: str = None,
+        algorithm: Optional[str] = None,
     ) -> bool:
         """Verify (EC)DSA signature.
 
-        :param key: ECC private or public key, either as EccKey or bytes
+        :param public_key: ECC private or public key, either as EccKey or bytes
         :param signature: Signature to verify, r and s coordinates as bytes
         :param data: Data to validate
         :param algorithm: Hash algorithm, if None the hash length is determined from ECC curve size
         :return: True if the signature is valid
         :raises SPSDKError: Signature length is invalid
         """
-        key = key if isinstance(key, ECC.EccKey) else ECC.import_key(key)
-        hash_name = algorithm or f"sha{key.pointQ.size_in_bits()}"
-        coordinate_size = key.pointQ.size_in_bytes()
+        public_key = (
+            public_key if isinstance(public_key, ECC.EccKey) else ECC.import_key(public_key)
+        )
+        hash_name = algorithm or f"sha{public_key.pointQ.size_in_bits()}"
+        coordinate_size = public_key.pointQ.size_in_bytes()
         if len(signature) != 2 * coordinate_size:
             raise SPSDKError(
                 f"Invalid signature size: expected {2 * coordinate_size}, actual: {len(signature)}"
             )
         hasher = self._get_algorithm(name=hash_name, data=data)
         try:
-            DSS.new(key, mode="deterministic-rfc6979").verify(hasher, signature)
+            DSS.new(public_key, mode="deterministic-rfc6979").verify(hasher, signature)
             return True
         except ValueError:
             return False
