@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -18,16 +18,16 @@ from cryptography.hazmat.primitives.keywrap import aes_key_wrap
 
 from spsdk import SPSDKError
 from spsdk import version as spsdk_version
-from spsdk.apps.utils.utils import get_key
+from spsdk.apps.utils.utils import filepath_from_config, get_key
 from spsdk.exceptions import SPSDKValueError
 from spsdk.utils.database import Database
+from spsdk.utils.exceptions import SPSDKRegsErrorBitfieldNotFound
 from spsdk.utils.images import BinaryImage
 from spsdk.utils.misc import (
     align_block,
-    find_file,
     load_binary,
     reverse_bits_in_bytes,
-    reverse_bytes_in_longs,
+    split_data,
     value_to_bytes,
     value_to_int,
 )
@@ -100,7 +100,6 @@ class KeyBlob:
         key: Optional[bytes] = None,
         counter_iv: Optional[bytes] = None,
         key_flags: int = KEY_FLAG_VLD | KEY_FLAG_ADE,
-        binaries: BinaryImage = None,
         # for testing
         zero_fill: Optional[bytes] = None,
         crc: Optional[bytes] = None,
@@ -141,7 +140,6 @@ class KeyBlob:
         self.start_addr = start_addr
         self.end_addr = end_addr
         self.key_flags = key_flags
-        self._binaries = binaries
         self.zero_fill = zero_fill
         self.crc_fill = crc
 
@@ -168,7 +166,7 @@ class KeyBlob:
         result += pack("<I", self.start_addr)
         if self.end_addr or self.key_flags:
             end_addr_with_flags = (
-                (self.end_addr & ~self._KEY_FLAG_MASK) | self.key_flags | self._END_ADDR_MASK
+                ((self.end_addr - 1) & ~self._KEY_FLAG_MASK) | self.key_flags | self._END_ADDR_MASK
             )
         else:
             end_addr_with_flags = 0
@@ -196,7 +194,10 @@ class KeyBlob:
 
     # pylint: disable=invalid-name
     def export(
-        self, kek: Union[bytes, str], iv: bytes = bytes([0xA6] * 8), byte_swap_cnt: int = 0
+        self,
+        kek: Union[bytes, str],
+        iv: bytes = bytes([0xA6] * 8),
+        byte_swap_cnt: int = 0,
     ) -> bytes:
         """Creates key wrap for the key blob.
 
@@ -272,13 +273,20 @@ class KeyBlob:
         """
         return self.contains_addr(image_start) and self.contains_addr(image_end)
 
-    def encrypt_image(self, base_address: int, data: bytes, byte_swap: bool) -> bytes:
+    def encrypt_image(
+        self,
+        base_address: int,
+        data: bytes,
+        byte_swap: bool,
+        counter_value: Optional[int] = None,
+    ) -> bytes:
         """Encrypt specified data.
 
         :param base_address: of the data in target memory; must be >= self.start_addr
         :param data: to be encrypted (e.g. plain image); base_address + len(data) must be <= self.end_addr
         :param byte_swap: this probably depends on the flash device, how bytes are organized there
                 True should be used for FLASH on EVK RT6xx; False for FLASH on EVK RT5xx
+        :param counter_value: Optional counter value, if not specified start address of keyblob will be used
         :return: encrypted data
         :raises SPSDKError: If start address is not valid
         """
@@ -291,14 +299,17 @@ class KeyBlob:
         # Support dual image boot, do not raise exception
         if not self.matches_range(base_address, base_address + data_len - 1):
             logger.warning(
-                f"Image address range is not within key blob: {hex(self.start_addr)}-{hex(self.end_addr)}."
+                f"Image address range is not within key blob: "
+                f"{hex(self.start_addr)}-{hex(self.end_addr)}."
                 " Ignore this if flash remap feature is used"
             )
-
         result = bytes()
         # SPSDK-936 change base address to start address of keyblob to allow flash remap on RT5xx/RT6xx
+        if not counter_value:
+            counter_value = self.start_addr
+
         counter = Counter(
-            self._get_ctr_nonce(), ctr_value=self.start_addr, ctr_byteorder_encoding="big"
+            self._get_ctr_nonce(), ctr_value=counter_value, ctr_byteorder_encoding="big"
         )
 
         for index in range(0, data_len, 16):
@@ -336,49 +347,12 @@ class KeyBlob:
             == (self.KEY_FLAG_ADE | self.KEY_FLAG_VLD)
         )
 
-    def binary_image(
-        self,
-        plain_data: bool = False,
-        swap_bytes: bool = False,
-        join_sub_images: bool = True,
-        table_address: int = 0,
-    ) -> Optional[BinaryImage]:
-        """Get the OTFAD Key Blob Binary Image representation.
-
-        :param plain_data: Binary representation in plain data format, defaults to False
-        :param swap_bytes: For some platforms the swap bytes is needed in encrypted format, defaults to False.
-        :param join_sub_images: If it's True, all the binary sub-images are joined into one, defaults to True.
-        :param table_address: Absolute address of OTFAD table.
-        :return: OTFAD key blob data in BinaryImage.
-        """
-        if self._binaries is None:
-            return None
-        binaries: BinaryImage = deepcopy(self._binaries)
-        for binary in binaries.sub_images:
-            assert binary.binary
-            binary.binary = align_block(binary.binary, self._ENCRYPTION_BLOCK_SIZE)
-
-        binaries.validate()
-
-        if not plain_data and self.is_encrypted:
-            for binary in binaries.sub_images:
-                assert binary.binary
-                binary.binary = self.encrypt_image(
-                    table_address + binary.absolute_address, binary.binary, swap_bytes
-                )
-                binary.name += "_encrypted"
-            binaries.name += "_encrypted"
-
-        if join_sub_images:
-            binaries.join_images()
-            binaries.validate()
-
-        return binaries
-
 
 class Otfad:
     """OTFAD: On-the-Fly AES Decryption Module."""
 
+    OTFAD_DATA_UNIT = 0x400
+
     def __init__(self) -> None:
         """Constructor."""
         self._key_blobs: List[KeyBlob] = []
@@ -398,21 +372,29 @@ class Otfad:
         self._key_blobs.append(key_blob)
 
     def encrypt_image(self, image: bytes, base_addr: int, byte_swap: bool) -> bytes:
-        """Encrypt image.
+        """Encrypt image with all available keyblobs.
 
         :param image: plain image to be encrypted
         :param base_addr: where the image will be located in target processor
         :param byte_swap: this probably depends on the flash device, how bytes are organized there
                 True should be used for FLASH on EVK RT6xx; False for FLASH on EVK RT5xx
         :return: encrypted image
-        :raises SPSDKError: If address range does not match to any key blob
         """
-        image_end = base_addr + len(image) - 1
-        for key_blob in self._key_blobs:
-            if key_blob.matches_range(base_addr, image_end):
-                return key_blob.encrypt_image(base_addr, image, byte_swap)
+        encrypted_data = bytearray(image)
+        addr = base_addr
+        for block in split_data(image, self.OTFAD_DATA_UNIT):
+            for key_blob in self._key_blobs:
+                if key_blob.matches_range(addr, addr + len(block)):
+                    logger.debug(
+                        f"Encrypting {hex(addr)}:{hex(len(block) + addr)}"
+                        f" with keyblob: \n {key_blob.info()}"
+                    )
+                    encrypted_data[
+                        addr - base_addr : len(block) + addr - base_addr
+                    ] = key_blob.encrypt_image(addr, block, byte_swap, counter_value=addr)
+            addr += len(block)
 
-        raise SPSDKError("The image address range does not match to key blob")
+        return bytes(encrypted_data)
 
     def get_key_blobs(self) -> bytes:
         """Get key blobs.
@@ -429,8 +411,8 @@ class Otfad:
     def encrypt_key_blobs(
         self,
         kek: Union[bytes, str],
-        key_scramble_mask: int = None,
-        key_scramble_align: int = None,
+        key_scramble_mask: Optional[int] = None,
+        key_scramble_align: Optional[int] = None,
         byte_swap_cnt: int = 0,
     ) -> bytes:
         """Encrypt key blobs with specified key.
@@ -495,9 +477,10 @@ class OtfadNxp(Otfad):
         family: str,
         kek: Union[bytes, str],
         table_address: int = 0,
-        key_blobs: List[KeyBlob] = None,
-        key_scramble_mask: int = None,
-        key_scramble_align: int = None,
+        key_blobs: Optional[List[KeyBlob]] = None,
+        key_scramble_mask: Optional[int] = None,
+        key_scramble_align: Optional[int] = None,
+        binaries: Optional[BinaryImage] = None,
     ) -> None:
         """Constructor.
 
@@ -511,7 +494,7 @@ class OtfadNxp(Otfad):
             ('key_scramble_mask' must be defined also)
         :raises SPSDKValueError: Unsupported family
         """
-        self._key_blobs: List[KeyBlob] = []
+        super().__init__()
 
         if family not in self.get_supported_families():
             raise SPSDKValueError(f"Unsupported family{family} by OTFAD")
@@ -535,6 +518,7 @@ class OtfadNxp(Otfad):
             "keyblob_byte_swap_cnt", device=family
         )
         assert self.keyblob_byte_swap_cnt in [0, 2, 4, 8, 16]
+        self.binaries = binaries
 
         if key_blobs:
             for key_blob in key_blobs:
@@ -575,8 +559,8 @@ class OtfadNxp(Otfad):
         fuses = Registers(family, base_endianness="little")
         grouped_regs = database.get_device_value("grouped_registers", device=family, default=None)
         fuses.load_registers_from_xml(xml_fuses, grouped_regs=grouped_regs)
-        reg_omk = fuses.find_reg(f"OTP_MASTER_KEY")
-        reg_oks = fuses.find_reg(f"OTFAD_KEK_SEED")
+        reg_omk = fuses.find_reg("OTP_MASTER_KEY")
+        reg_oks = fuses.find_reg("OTFAD_KEK_SEED")
         reg_omk.set_value(otp_master_key)
         reg_oks.set_value(otfad_key_seed)
         ret = (
@@ -621,25 +605,41 @@ class OtfadNxp(Otfad):
         xml_fuses = os.path.join(OTFAD_DATA_FOLDER, xml_fuses)
 
         fuses = Registers(self.family, base_endianness="little")
+        self.database.index = index
+
         grouped_regs = self.database.get_device_value(
             "grouped_registers", device=self.family, default=None
         )
-        if grouped_regs:
-            for reg in grouped_regs:
-                reg["name"] = reg["name"].replace("{index}", str(index))
+
         fuses.load_registers_from_xml(xml_fuses, filter_out_list, grouped_regs)
 
         scramble_enabled = (
             self.key_scramble_mask is not None and self.key_scramble_align is not None
         )
 
-        fuses.find_reg(f"OTFAD{index}_KEY").set_value(self.kek)
-        otfad_cfg = fuses.find_reg(f"OTFAD{index}_CFG4")
-        otfad_cfg.find_bitfield("OTFAD_ENABLE").set_value(1)
+        otfad_key_fuse = self.database.get_device_value("otfad_key_fuse", self.family)
+        otfad_cfg_fuse = self.database.get_device_value("otfad_cfg_fuse", self.family)
+
+        fuses.find_reg(otfad_key_fuse).set_value(self.kek)
+        otfad_cfg = fuses.find_reg(otfad_cfg_fuse)
+
+        try:
+            otfad_cfg.find_bitfield(
+                self.database.get_device_value("otfad_enable_bitfield", self.family)
+            ).set_value(1)
+        except SPSDKRegsErrorBitfieldNotFound:
+            logger.debug(f"Bitfield for OTFAD ENABLE not found for {self.family}")
+
         if scramble_enabled:
-            otfad_cfg.find_bitfield("OTFAD_SCRAMBLE_ENABLE").set_value(1)
-            otfad_cfg.find_bitfield("OTFAD_SCRAMBLE_ALIGN").set_value(self.key_scramble_align)
-            fuses.find_reg(f"OTFAD{index}_KEY_SCRAMBLE").set_value(self.key_scramble_mask)
+            otfad_cfg.find_bitfield(
+                self.database.get_device_value("otfad_scramble_enable_bitfield", self.family)
+            ).set_value(1)
+            otfad_cfg.find_bitfield(
+                self.database.get_device_value("otfad_scramble_align_bitfield", self.family)
+            ).set_value(self.key_scramble_align)
+            fuses.find_reg(
+                self.database.get_device_value("otfad_scramble_key", self.family)
+            ).set_value(self.key_scramble_mask)
 
         ret = (
             f"# BLHOST OTFAD{index} KEK fuses programming script\n"
@@ -648,7 +648,7 @@ class OtfadNxp(Otfad):
         )
 
         ret += f"# OTP KEK (Big Endian): {self.kek.hex()}\n\n"
-        for reg in fuses.find_reg(f"OTFAD{index}_KEY").sub_regs:
+        for reg in fuses.find_reg(otfad_key_fuse).sub_regs:
             ret += f"# {reg.name} fuse.\n"
             ret += f"efuse-program-once {reg.offset} 0x{reg.get_bytes_value().hex()} --no-verify\n"
 
@@ -658,17 +658,74 @@ class OtfadNxp(Otfad):
         ret += f"efuse-program-once {otfad_cfg.offset} 0x{otfad_cfg.get_bytes_value().hex()} --no-verify\n"
 
         if scramble_enabled:
-            scramble = fuses.find_reg(f"OTFAD{index}_KEY_SCRAMBLE")
+            scramble = fuses.find_reg(
+                self.database.get_device_value("otfad_scramble_key", self.family)
+            )
             ret += f"\n# {scramble.name} fuse.\n"
             ret += f"efuse-program-once {scramble.offset} 0x{scramble.get_bytes_value().hex()} --no-verify\n"
 
         return ret
 
-    def binary_image(self, plain_data: bool = False, data_alignment: int = 16) -> BinaryImage:
+    def export_image(
+        self,
+        plain_data: bool = False,
+        swap_bytes: bool = False,
+        join_sub_images: bool = True,
+        table_address: int = 0,
+    ) -> Optional[BinaryImage]:
+        """Get the OTFAD Key Blob Binary Image representation.
+
+        :param plain_data: Binary representation in plain data format, defaults to False
+        :param swap_bytes: For some platforms the swap bytes is needed in encrypted format, defaults to False.
+        :param join_sub_images: If it's True, all the binary sub-images are joined into one, defaults to True.
+        :param table_address: Absolute address of OTFAD table.
+        :return: OTFAD key blob data in BinaryImage.
+        """
+        if self.binaries is None:
+            return None
+        binaries: BinaryImage = deepcopy(self.binaries)
+        for binary in binaries.sub_images:
+            if binary.binary:
+                binary.binary = align_block(binary.binary, KeyBlob._ENCRYPTION_BLOCK_SIZE)
+            for segment in binary.sub_images:
+                if segment.binary:
+                    segment.binary = align_block(segment.binary, KeyBlob._ENCRYPTION_BLOCK_SIZE)
+
+        binaries.validate()
+
+        if not plain_data:
+            for binary in binaries.sub_images:
+                if binary.binary:
+                    binary.binary = self.encrypt_image(
+                        binary.binary,
+                        table_address + binary.absolute_address,
+                        swap_bytes,
+                    )
+                for segment in binary.sub_images:
+                    if segment.binary:
+                        segment.binary = self.encrypt_image(
+                            segment.binary,
+                            segment.absolute_address + table_address,
+                            swap_bytes,
+                        )
+
+        if join_sub_images:
+            binaries.join_images()
+            binaries.validate()
+
+        return binaries
+
+    def binary_image(
+        self,
+        plain_data: bool = False,
+        data_alignment: int = 16,
+        otfad_table_name: str = "OTFAD_Table",
+    ) -> BinaryImage:
         """Get the OTFAD Binary Image representation.
 
         :param plain_data: Binary representation in plain format, defaults to False
         :param data_alignment: Alignment of data part key blobs.
+        :param otfad_table_name: name of the output file that contains OTFAD table
         :return: OTFAD in BinaryImage.
         """
         otfad = BinaryImage("OTFAD", offset=self.table_address)
@@ -685,7 +742,7 @@ class OtfadNxp(Otfad):
         )
         otfad.add_image(
             BinaryImage(
-                "OTFAD_Table",
+                otfad_table_name,
                 size=self.key_blob_rec_size * self.blobs_max_cnt,
                 offset=0,
                 description=f"OTFAD description table for {self.family}",
@@ -693,15 +750,12 @@ class OtfadNxp(Otfad):
                 alignment=256,
             )
         )
-        for key_blob in self._key_blobs:
-            binaries = key_blob.binary_image(
-                plain_data=plain_data, swap_bytes=self.byte_swap, table_address=self.table_address
-            )
-            if binaries:
-                binaries.alignment = data_alignment
-                binaries.validate()
-                otfad.add_image(binaries)
+        binaries = self.export_image(table_address=self.table_address)
 
+        if binaries:
+            binaries.alignment = data_alignment
+            binaries.validate()
+            otfad.add_image(binaries)
         return otfad
 
     @staticmethod
@@ -711,7 +765,7 @@ class OtfadNxp(Otfad):
         :return: List of supported families.
         """
         database = Database(OTFAD_DATABASE_FILE)
-        return database.get_devices()
+        return database.devices.device_names
 
     @staticmethod
     def get_validation_schemas(family: str) -> List[Dict[str, Any]]:
@@ -767,22 +821,25 @@ class OtfadNxp(Otfad):
         return {}
 
     @staticmethod
-    def load_from_config(config: Dict[str, Any], search_paths: List[str] = None) -> "OtfadNxp":
+    def load_from_config(
+        config: Dict[str, Any], config_dir: str, search_paths: Optional[List[str]] = None
+    ) -> "OtfadNxp":
         """Converts the configuration option into an OTFAD image object.
 
         "config" content array of containers configurations.
 
-        :raises SPSDKValueError: if the count of AHAB containers is invalid.
-        :param config: array of AHAB containers configuration dictionaries.
+        :param config: array of OTFAD configuration dictionaries.
+        :param config_dir: directory where the config is located
         :param search_paths: List of paths where to search for the file, defaults to None
         :return: initialized OTFAD object.
         """
         otfad_config: List[Dict[str, Any]] = config["key_blobs"]
         family = config["family"]
         database = Database(OTFAD_DATABASE_FILE)
-        kek = get_key(find_file(config["kek"], search_paths=search_paths), 16)
+        kek = get_key(config["kek"], expected_size=16, search_paths=search_paths)
         logger.debug(f"Loaded KEK: {kek.hex()}")
         table_address = value_to_int(config["otfad_table_address"])
+        start_address = min([value_to_int(addr["start_address"]) for addr in otfad_config])
 
         key_scramble_mask = None
         key_scramble_align = None
@@ -792,12 +849,38 @@ class OtfadNxp(Otfad):
                 key_scramble_mask = value_to_int(key_scramble["key_scramble_mask"])
                 key_scramble_align = value_to_int(key_scramble["key_scramble_align"])
 
+        data_blobs: Optional[List[Dict]] = config.get("data_blobs")
+        binaries = None
+        if data_blobs:
+            # pylint: disable-next=nested-min-max
+            start_address = min(
+                min([value_to_int(addr["address"]) for addr in data_blobs]),
+                start_address,
+            )
+            binaries = BinaryImage(
+                filepath_from_config(
+                    config, "encrypted_name", "encrypted_blobs", config_dir, config["output_folder"]
+                ),
+                offset=start_address - table_address,
+            )
+            for data_blob in data_blobs:
+                data = load_binary(data_blob["data"], search_paths=search_paths)
+                address = value_to_int(data_blob["address"])
+
+                binary = BinaryImage(
+                    os.path.basename(data_blob["data"]),
+                    offset=address - table_address - binaries.offset,
+                    binary=data,
+                )
+                binaries.add_image(binary)
+
         otfad = OtfadNxp(
             family=family,
             kek=kek,
             table_address=table_address,
             key_scramble_align=key_scramble_align,
             key_scramble_mask=key_scramble_mask,
+            binaries=binaries,
         )
 
         for i, key_blob_cfg in enumerate(otfad_config):
@@ -816,27 +899,12 @@ class OtfadNxp(Otfad):
             if read_only:
                 flags |= KeyBlob.KEY_FLAG_READ_ONLY
 
-            data_blobs: Optional[List[Dict]] = key_blob_cfg.get("data_blobs")
-            binaries = None
-            if data_blobs:
-                binaries = BinaryImage(f"KeyBlob{i}_data", offset=start_addr - table_address)
-                for j, data_blob in enumerate(data_blobs):
-                    data = load_binary(data_blob["data"], search_paths=search_paths)
-                    address = value_to_int(data_blob.get("address", start_addr))
-                    binary = BinaryImage(
-                        f"KeyBlob{i}_data{j}",
-                        offset=address - table_address - binaries.offset,
-                        binary=data,
-                    )
-                    binaries.add_image(binary)
-
             otfad[i] = KeyBlob(
                 start_addr=start_addr,
                 end_addr=end_addr,
                 key=aes_key,
                 counter_iv=aes_ctr,
                 key_flags=flags,
-                binaries=binaries,
                 zero_fill=bytes([0] * 4),
             )
 
