@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2022 NXP
+# Copyright 2022-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module to keep additional utilities for binary images."""
@@ -10,7 +10,7 @@ import logging
 import math
 import os
 import textwrap
-from typing import TYPE_CHECKING, Any, Dict, List, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Optional
 
 import colorama
 
@@ -22,7 +22,6 @@ from spsdk.utils.misc import (
     align_block,
     find_file,
     format_value,
-    load_binary,
     size_fmt,
     write_file,
 )
@@ -32,7 +31,7 @@ if TYPE_CHECKING:
     # bincopy will be loaded lazily as needed, this is just to satisfy type-hint checkers
     import bincopy
 
-BINARY_SCH_FILE = os.path.join(SPSDK_DATA_FOLDER, "image", "sch_binary.yml")
+BINARY_SCH_FILE = os.path.join(SPSDK_DATA_FOLDER, "image", "sch_binary.yaml")
 
 logger = logging.getLogger(__name__)
 
@@ -60,7 +59,7 @@ class ColorPicker:
         """Constructor of ColorPicker."""
         self.index = len(self.COLORS)
 
-    def get_color(self, unwanted_color: str = None) -> str:
+    def get_color(self, unwanted_color: Optional[str] = None) -> str:
         """Get new color from list.
 
         :param unwanted_color: Color that should be omitted.
@@ -84,11 +83,11 @@ class BinaryImage:
         name: str,
         size: int = 0,
         offset: int = 0,
-        description: str = None,
-        binary: bytes = None,
-        pattern: BinaryPattern = None,
+        description: Optional[str] = None,
+        binary: Optional[bytes] = None,
+        pattern: Optional[BinaryPattern] = None,
         alignment: int = 1,
-        parent: "BinaryImage" = None,
+        parent: Optional["BinaryImage"] = None,
     ) -> None:
         """Binary Image class constructor.
 
@@ -104,15 +103,26 @@ class BinaryImage:
         self.name = name
         self.description = description
         self.offset = offset
-        self._size = size
+        self._size = align(size, alignment)
         self.binary = binary
         self.pattern = pattern
         self.alignment = alignment
         self.parent = parent
+
         if parent:
             assert isinstance(parent, BinaryImage)
         self.sub_images: List["BinaryImage"] = []
 
+    @property
+    def size(self) -> int:
+        """Size property."""
+        return self._size
+
+    @size.setter
+    def size(self, value: int) -> None:
+        """Size property setter."""
+        self._size = align(value, self.alignment)
+
     def add_image(self, image: "BinaryImage") -> None:
         """Add new sub image information.
 
@@ -155,7 +165,7 @@ class BinaryImage:
         """Returns aligned start address.
 
         :param alignment: The alignment value, defaults to 4.
-        :returns: Floor alignment address.
+        :return: Floor alignment address.
         """
         return math.floor(self.absolute_address / alignment) * alignment
 
@@ -163,7 +173,7 @@ class BinaryImage:
         """Returns aligned length for erasing purposes.
 
         :param alignment: The alignment value, defaults to 4.
-        :returns: Ceil alignment length.
+        :return: Ceil alignment length.
         """
         end_address = self.absolute_address + len(self)
         aligned_end = math.ceil(end_address / alignment) * alignment
@@ -203,7 +213,7 @@ class BinaryImage:
             begin = image.offset
             end = begin + len(image) - 1
             # Check if it fits inside the parent image
-            if end > len(self):
+            if end >= len(self):
                 raise SPSDKOverlapError(
                     f"The image {image.name} doesn't fit into {self.name} parent image."
                 )
@@ -230,7 +240,7 @@ class BinaryImage:
         """
         widths = [
             self.MINIMAL_DRAW_WIDTH,
-            len(f"+--0x0000_0000--{self.name}--+"),
+            len(f"+==-0x0000_0000= {self.name} =+"),
             len(f"|Size: {size_fmt(len(self), False)}|"),
         ]
         if include_sub_images:
@@ -245,6 +255,7 @@ class BinaryImage:
         color: str = "",
         no_color: bool = False,
     ) -> str:
+        # fmt: off
         """Draw the image into the ASCII graphics.
 
         :param include_sub_images: Include also sub images into, defaults to True
@@ -254,22 +265,23 @@ class BinaryImage:
         :raises SPSDKValueError: In case of invalid width.
         :return: ASCII art representation of image.
         """
-        # +--0x0000_0000--Title1---------------+
+        # +==0x0000_0000==Title1===============+
         # |            Size: 2048B             |
         # |           Description1             |
         # |       Description1 2nd line        |
-        # |+--0x0000_0000--Title11------------+|
+        # |+==0x0000_0000==Title11============+|
         # ||           Size: 512B             ||
         # ||           Description11          ||
         # ||       Description11 2nd line     ||
-        # |+--0x0000_01FF---------------------+|
+        # |+==0x0000_01FF=====================+|
         # |                                    |
-        # |+--0x0000_0210--Title12------------+|
+        # |+==0x0000_0210==Title12============+|
         # ||           Size: 512B             ||
         # ||           Description12          ||
         # ||       Description12 2nd line     ||
-        # |+--0x0000_041F---------------------+|
-        # +--0x0000_07FF-----------------------+
+        # |+==0x0000_041F=====================+|
+        # +==0x0000_07FF=======================+
+        # fmt: on
         def _get_centered_line(text: str) -> str:
             text_len = len(text)
             spaces = width - text_len - 2
@@ -306,8 +318,8 @@ class BinaryImage:
             )
 
         # - Title line
-        header = f"+--{format_value(self.absolute_address, 32)}--{self.name}--"
-        block += color + f"{header}{'-'*(width-len(header)-1)}+\n"
+        header = f"+=={format_value(self.absolute_address, 32)}= {self.name} ="
+        block += color + f"{header}{'='*(width-len(header)-1)}+\n"
         # - Size
         block += _get_centered_line(f"Size: {size_fmt(len(self), False)}")
         # - Description
@@ -336,8 +348,8 @@ class BinaryImage:
                 block += wrap_block(inner_block)
 
         # - Closing line
-        footer = f"+--{format_value(self.absolute_address + len(self) - 1, 32)}--"
-        block += color + f"{footer}{'-'*(width-len(footer)-1)}+\n"
+        footer = f"+=={format_value(self.absolute_address + len(self) - 1, 32)}=="
+        block += color + f"{footer}{'='*(width-len(footer)-1)}+\n"
 
         if self.parent is None:
             block += "\n" + "" if no_color else colorama.Fore.RESET
@@ -389,7 +401,9 @@ class BinaryImage:
         return [ValidationSchemas.get_schema_file(BINARY_SCH_FILE)]
 
     @staticmethod
-    def load_from_config(config: Dict[str, Any], search_paths: List[str] = None) -> "BinaryImage":
+    def load_from_config(
+        config: Dict[str, Any], search_paths: Optional[List[str]] = None
+    ) -> "BinaryImage":
         """Converts the configuration option into an Binary Image object.
 
         :param config: Description of binary image.
@@ -405,10 +419,17 @@ class BinaryImage:
             for i, region in enumerate(regions):
                 binary_file: Dict = region.get("binary_file")
                 if binary_file:
-                    binary = load_binary(binary_file["path"], search_paths=search_paths)
                     offset = binary_file["offset"]
                     name = binary_file.get("name", binary_file["path"])
-                    ret.add_image(BinaryImage(name, len(binary), offset, binary=binary))
+                    ret.add_image(
+                        BinaryImage.load_binary_image(
+                            binary_file["path"],
+                            name=name,
+                            offset=offset,
+                            pattern=pattern,
+                            search_paths=search_paths,
+                        )
+                    )
                 binary_block: Dict = region.get("binary_block")
                 if binary_block:
                     size = binary_block["size"]
@@ -481,12 +502,13 @@ class BinaryImage:
     @staticmethod
     def load_binary_image(
         path: str,
-        name: str = None,
+        name: Optional[str] = None,
         size: int = 0,
         offset: int = 0,
-        description: str = None,
-        pattern: BinaryPattern = None,
-        search_paths: List[str] = None,
+        description: Optional[str] = None,
+        pattern: Optional[BinaryPattern] = None,
+        search_paths: Optional[List[str]] = None,
+        alignment: int = 1,
     ) -> "BinaryImage":
         # pylint: disable=missing-param-doc
         r"""Load binary data file.
@@ -498,6 +520,7 @@ class BinaryImage:
         :param description: Text description of image, defaults to None
         :param pattern: Optional binary pattern.
         :param search_paths: List of paths where to search for the file, defaults to None
+        :param alignment: Optional alignment of result image
         :raises SPSDKError: The binary file cannot be loaded.
         :return: Binary data represented in BinaryImage class.
         """
@@ -533,6 +556,7 @@ class BinaryImage:
             offset=offset,
             description=img_descr,
             pattern=pattern,
+            alignment=alignment,
         )
         if len(bin_file.segments) == 0:
             raise SPSDKError(f"Load of {path} failed, can't be decoded.")
@@ -546,6 +570,7 @@ class BinaryImage:
                     pattern=pattern,
                     binary=segment.data,
                     parent=bin_image,
+                    alignment=alignment,
                 )
             )
         # Optimize offsets in image
