@@ -1,20 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module to handle registers descriptions with support for XML files."""
 
 import logging
+import re
 import xml.etree.ElementTree as ET
-from typing import Any, Callable, Dict, List, Mapping, Union
+from typing import Any, Callable, Dict, List, Mapping, Optional, Tuple, Union
 from xml.dom import minidom
 
 from jinja2 import Environment, FileSystemLoader
 from ruamel.yaml.comments import CommentedMap as CM
 
 from spsdk import SPSDK_YML_INDENT, SPSDKError, utils
+from spsdk.exceptions import SPSDKValueError
 from spsdk.utils.exceptions import (
     SPSDKRegsError,
     SPSDKRegsErrorBitfieldNotFound,
@@ -23,7 +25,7 @@ from spsdk.utils.exceptions import (
     SPSDKRegsErrorRegisterNotFound,
 )
 from spsdk.utils.images import BinaryImage, BinaryPattern
-from spsdk.utils.misc import format_value, value_to_bool, value_to_bytes, value_to_int
+from spsdk.utils.misc import format_value, value_to_bool, value_to_bytes, value_to_int, write_file
 
 HTMLDataElement = Mapping[str, Union[str, dict, list]]
 HTMLData = List[HTMLDataElement]
@@ -60,7 +62,7 @@ class RegsEnum:
         :return: The instance of this class.
         :raises SPSDKRegsError: Error during enum XML parsing.
         """
-        name = xml_element.attrib["name"] if "name" in xml_element.attrib else "N/A"
+        name = xml_element.attrib.get("name", "N/A")
         if "value" not in xml_element.attrib:
             raise SPSDKRegsError(f"Missing Enum Value Key for {name}.")
 
@@ -70,9 +72,9 @@ class RegsEnum:
         except (TypeError, ValueError, SPSDKError) as exc:
             raise SPSDKRegsError(f"Invalid Enum Value: {raw_val}") from exc
 
-        descr = xml_element.attrib["description"] if "description" in xml_element.attrib else "N/A"
+        description = xml_element.attrib.get("description", "N/A")
 
-        return cls(name, value, descr, maxwidth)
+        return cls(name, value, description, maxwidth)
 
     def get_value_int(self) -> int:
         """Method returns Integer value of enum.
@@ -111,6 +113,120 @@ class RegsEnum:
         return output
 
 
+class ConfigProcessor:
+    """Base class for processing configuration data."""
+
+    NAME = "NOP"
+
+    def __init__(self, description: str = "") -> None:
+        """Initialize the processor."""
+        self.description = description
+
+    def pre_process(self, value: int) -> int:
+        """Pre-process value coming from config file."""
+        return value
+
+    def post_process(self, value: int) -> int:
+        """Post-process value going to config file."""
+        return value
+
+    def width_update(self, value: int) -> int:
+        """Update bit-width of value going to config file."""
+        return value
+
+    @classmethod
+    def get_method_name(cls, config_string: str) -> str:
+        """Return config processor method name."""
+        return config_string.split(":")[0]
+
+    @classmethod
+    def get_params(cls, config_string: str) -> Dict[str, int]:
+        """Return config processor method parameters."""
+
+        def split_params(param: str) -> Tuple[str, str]:
+            """Split key=value pair into a tuple."""
+            parts = param.split("=")
+            if len(parts) != 2:
+                raise SPSDKRegsError(
+                    f"Invalid param setting: '{param}'. Expected format '<name>=<value>'"
+                )
+            return (parts[0], parts[1])
+
+        parts = config_string.split(";", maxsplit=1)[0].split(":")
+        if len(parts) == 1:
+            return dict()
+        params = parts[1].split(",")
+        params_dict: Dict[str, str] = dict(split_params(p) for p in params)
+        return {key.lower(): value_to_int(value) for key, value in params_dict.items()}
+
+    @classmethod
+    def get_description(cls, config_string: str) -> str:
+        """Return extra description for config processor."""
+        parts = config_string.partition(";")
+        return parts[2].replace("DESC=", "")
+
+    @classmethod
+    def from_str(cls, config_string: str) -> "ConfigProcessor":
+        """Create config processor instance from configuration string."""
+        return cls(config_string)
+
+    @classmethod
+    def from_xml(cls, element: ET.Element) -> Optional["ConfigProcessor"]:
+        """Create config processor from XML data entry."""
+        processor_node = element.find("alias[@type='CONFIG_PREPROCESS']")
+        if processor_node is None:
+            return None
+        if "value" not in processor_node.attrib:
+            raise SPSDKRegsError("CONFIG_PREPROCESS alias node doesn't have a value")
+        config_string = processor_node.attrib["value"]
+        method_name = cls.get_method_name(config_string=config_string)
+        for klass in cls.__subclasses__():
+            if klass.NAME == method_name:
+                return klass.from_str(config_string=config_string)
+        return None
+
+
+class ShiftRightConfigProcessor(ConfigProcessor):
+    """Config processor performing the right-shift operation."""
+
+    NAME = "SHIFT_RIGHT"
+
+    def __init__(self, count: int, description: str = "") -> None:
+        """Initialize the right-shift config processor.
+
+        :param count: Count of bit for shift operation
+        :param description: Extra description for config processor, defaults to ""
+        """
+        super().__init__(
+            description=description or f"Actual binary value is shifted by {count} bits to right."
+        )
+        self.count = count
+
+    def pre_process(self, value: int) -> int:
+        """Pre-process value coming from config file."""
+        return value >> self.count
+
+    def post_process(self, value: int) -> int:
+        """Post-process value going to config file."""
+        return value << self.count
+
+    def width_update(self, value: int) -> int:
+        """Update bit-width of value going to config file."""
+        return value + self.count
+
+    @classmethod
+    def from_str(cls, config_string: str) -> "ShiftRightConfigProcessor":
+        """Create config processor instance from configuration string."""
+        name = cls.get_method_name(config_string=config_string)
+        if name != cls.NAME:
+            raise SPSDKRegsError(f"Invalid method name '{name}' expected {cls.NAME}")
+        params = cls.get_params(config_string=config_string)
+        if "count" not in params:
+            raise SPSDKRegsError(f"{cls.NAME} requires the COUNT parameter")
+        description = cls.get_description(config_string=config_string)
+        return cls(count=value_to_int(params["count"]), description=description)
+
+
 class RegsBitField:
     """Storage for register bitfields."""
 
@@ -120,10 +236,11 @@ class RegsBitField:
         name: str,
         offset: int,
         width: int,
-        description: str = None,
+        description: Optional[str] = None,
         reset_val: Any = "0",
         access: str = "RW",
         hidden: bool = False,
+        config_processor: Optional[ConfigProcessor] = None,
     ) -> None:
         """Constructor of RegsBitField class. Used to store bitfield information.
 
@@ -145,6 +262,8 @@ class RegsBitField:
         self.access = access
         self.hidden = hidden
         self._enums: List[RegsEnum] = []
+        self.config_processor = config_processor or ConfigProcessor()
+        self.config_width = self.config_processor.width_update(width)
         self._update_reset_value()
         self.set_value(self.reset_value, raw=True)
 
@@ -156,23 +275,22 @@ class RegsBitField:
         :param parent: Reference to parent RegsRegister object.
         :return: The instance of this class.
         """
-        name = xml_element.attrib["name"] if "name" in xml_element.attrib else "N/A"
-        offset = value_to_int(xml_element.attrib["offset"]) if "offset" in xml_element.attrib else 0
-        width = value_to_int(xml_element.attrib["width"]) if "width" in xml_element.attrib else 0
-        descr = xml_element.attrib["description"] if "description" in xml_element.attrib else "N/A"
-        access = xml_element.attrib["access"] if "access" in xml_element.attrib else "R/W"
-        reset_value = (
-            value_to_int(xml_element.attrib["reset_value"])
-            if "reset_value" in xml_element.attrib
-            else 0
-        )
+        name = xml_element.attrib.get("name", "N/A")
+        offset = value_to_int(xml_element.attrib.get("offset", 0))
+        width = value_to_int(xml_element.attrib.get("width", 0))
+        description = xml_element.attrib.get("description", "N/A")
+        access = xml_element.attrib.get("access", "R/W")
+        reset_value = value_to_int(xml_element.attrib.get("reset_value", 0))
         hidden = xml_element.tag != "bit_field"
-        bitfield = cls(parent, name, offset, width, descr, reset_value, access, hidden)
+        config_processor = ConfigProcessor.from_xml(xml_element)
+
+        bitfield = cls(
+            parent, name, offset, width, description, reset_value, access, hidden, config_processor
+        )
+
+        for xml_enum in xml_element.findall("bit_field_value"):
+            bitfield.add_enum(RegsEnum.from_xml_element(xml_enum, width))
 
-        if xml_element.text:
-            xml_enums = xml_element.findall("bit_field_value")
-            for xml_enum in xml_enums:
-                bitfield.add_enum(RegsEnum.from_xml_element(xml_enum, width))
         return bitfield
 
     def has_enums(self) -> bool:
@@ -205,6 +323,7 @@ class RegsBitField:
         value = reg_val >> self.offset
         mask = (1 << self.width) - 1
         value = value & mask
+        value = self.config_processor.post_process(value)
         return value
 
     def get_reset_value(self) -> int:
@@ -222,6 +341,7 @@ class RegsBitField:
         :raises SPSDKError: The input value is out of range.
         """
         new_val_int = value_to_int(new_val)
+        new_val_int = self.config_processor.pre_process(new_val_int)
         if new_val_int > 1 << self.width:
             raise SPSDKError("The input value is out of bitfield range")
         reg_val = self.parent.get_value()
@@ -233,7 +353,7 @@ class RegsBitField:
 
     def _update_reset_value(self) -> None:
         """Updates the reset value of the bitfield in register."""
-        reg_val = self.parent.get_value()
+        reg_val = self.parent.get_reset_value()
         mask = ((1 << self.width) - 1) << self.offset
         reg_val = reg_val & ~mask
         value = (self.reset_value << self.offset) & mask
@@ -249,12 +369,12 @@ class RegsBitField:
         """
         try:
             val_int = self.get_enum_constant(new_val)
-        except SPSDKRegsErrorEnumNotFound as exc:
+        except SPSDKRegsErrorEnumNotFound:
             # Try to decode standard input
             try:
                 val_int = value_to_int(new_val)
             except TypeError:
-                raise exc  # pylint: disable=raise-missing-from
+                raise SPSDKRegsErrorEnumNotFound  # pylint: disable=raise-missing-from
         self.set_value(val_int, raw)
 
     def get_enum_value(self) -> Union[str, int]:
@@ -263,11 +383,20 @@ class RegsBitField:
         :return: Current value of bitfield.
         """
         value = self.get_value()
-        if len(self._enums) > 0:
-            for enum in self._enums:
-                if enum.get_value_int() == value:
-                    return enum.name
-        return value
+        for enum in self._enums:
+            if enum.get_value_int() == value:
+                return enum.name
+        # return value
+        return self.get_hex_value()
+
+    def get_hex_value(self) -> str:
+        """Get the value of register in string hex format.
+
+        :return: Hexadecimal value of register.
+        """
+        fmt = f"0{self.config_width // 4}X"
+        val = f"0x{format(self.get_value(), fmt)}"
+        return val
 
     def get_enum_constant(self, enum_name: str) -> int:
         """Returns constant representation of enum by its name.
@@ -351,10 +480,13 @@ class RegsRegister:
         name: str,
         offset: int,
         width: int,
-        description: str = None,
+        description: Optional[str] = None,
         reverse: bool = False,
-        access: str = None,
+        access: Optional[str] = None,
         config_as_hexstring: bool = False,
+        otp_index: Optional[int] = None,
+        reverse_subregs_order: bool = False,
+        base_endianness: str = "big",
     ) -> None:
         """Constructor of RegsRegister class. Used to store register information.
 
@@ -362,9 +494,12 @@ class RegsRegister:
         :param offset: Byte offset of register.
         :param width: Bit width of register.
         :param description: Text description of register.
-        :param reverse: Multi  register value is stored in reverse order.
+        :param reverse: Multi byte register value could be printed in reverse order.
         :param access: Access type of register.
         :param config_as_hexstring: Config is stored as a hex string.
+        :param otp_index: Index of OTP fuse.
+        :param reverse_subregs_order: Reverse order of sub registers.
+        :param base_endianness: Base endianness for bytes import/export of value.
         """
         self.name = name
         self.offset = offset
@@ -377,6 +512,9 @@ class RegsRegister:
         self._value = 0
         self._reset_value = 0
         self.config_as_hexstring = config_as_hexstring
+        self.otp_index = otp_index
+        self.reverse_subregs_order = reverse_subregs_order
+        self.base_endianness = base_endianness
 
         # Grouped register members
         self.sub_regs: List["RegsRegister"] = []
@@ -390,16 +528,31 @@ class RegsRegister:
         :param xml_element: Input XML subelement with register data.
         :return: The instance of this class.
         """
-        name = xml_element.attrib["name"] if "name" in xml_element.attrib else "N/A"
-        offset = value_to_int(xml_element.attrib["offset"]) if "offset" in xml_element.attrib else 0
-        width = value_to_int(xml_element.attrib["width"]) if "width" in xml_element.attrib else 0
-        descr = xml_element.attrib["description"] if "description" in xml_element.attrib else "N/A"
-        reverse = (
-            xml_element.attrib["reversed"] if "reversed" in xml_element.attrib else "False"
-        ) == "True"
-        access = xml_element.attrib["access"] if "access" in xml_element.attrib else "N/A"
+        name = xml_element.attrib.get("name", "N/A")
+        offset = value_to_int(xml_element.attrib.get("offset", 0))
+        width = value_to_int(xml_element.attrib.get("width", 0))
+        description = xml_element.attrib.get("description", "N/A")
+        reverse_subregs_order = (xml_element.attrib.get("reverse_subregs_order", "False")) == "True"
+        reverse = (xml_element.attrib.get("reversed", "False")) == "True"
+        access = xml_element.attrib.get("access", "N/A")
+        otp_index_raw = xml_element.attrib.get("otp_index")
+        otp_index = None
+        if otp_index_raw:
+            otp_index = value_to_int(otp_index_raw)
+        reg = cls(
+            name,
+            offset,
+            width,
+            description,
+            reverse,
+            access,
+            otp_index=otp_index,
+            reverse_subregs_order=reverse_subregs_order,
+        )
+        value = xml_element.attrib.get("value")
+        if value:
+            reg.set_value(value)
 
-        reg = cls(name, offset, width, descr, reverse, access)
         if xml_element.text:
             xml_bitfields = xml_element.findall("bit_field")
             xml_bitfields.extend(xml_element.findall("reserved_bit_field"))
@@ -438,8 +591,6 @@ class RegsRegister:
             else:
                 self._sub_regs_width_init = True
                 self._sub_regs_width = reg.width
-            if reg.reverse:
-                self.reverse = True
             if self.access == "RW":
                 self.access = reg.access
         else:
@@ -468,10 +619,7 @@ class RegsRegister:
                 raise SPSDKRegsErrorRegisterGroupMishmash(
                     f"The register {reg.name} has different access type."
                 )
-
-        if self.reverse:
-            reg.reverse = True
-
+        reg.base_endianness = self.base_endianness
         self.sub_regs.append(reg)
 
     def add_et_subelement(self, parent: ET.Element) -> None:
@@ -485,10 +633,12 @@ class RegsRegister:
         element.set("name", self.name)
         element.set("reversed", str(self.reverse))
         element.set("description", self.description)
+        if self.otp_index:
+            element.set("otp_index", str(self.otp_index))
         for bitfield in self._bitfields:
             bitfield.add_et_subelement(element)
 
-    def get_html_data_element(self, exclude: List[str] = None) -> HTMLDataElement:
+    def get_html_data_element(self, exclude: Optional[List[str]] = None) -> HTMLDataElement:
         """Returns HTML element of register.
 
         :return: HTML element.
@@ -514,6 +664,10 @@ class RegsRegister:
         """
         try:
             value = value_to_int(val)
+            if value >= 1 << self.width:
+                raise SPSDKError(
+                    f"Input value {value} doesn't fit into register of width {self.width}."
+                )
             if not raw and len(self._set_value_hooks) > 0:
                 for hook in self._set_value_hooks:
                     value = hook[0](value, hook[1])
@@ -524,10 +678,11 @@ class RegsRegister:
                 # Update also values in sub registers
                 subreg_width = self.sub_regs[0].width
                 for index, sub_reg in enumerate(self.sub_regs, start=1):
-                    # sub_reg.set_value((value >> (index * subreg_width)) & ((1 << subreg_width) - 1))
-                    sub_reg.set_value(
-                        (value >> (self.width - index * subreg_width)) & ((1 << subreg_width) - 1)
-                    )
+                    if self.reverse_subregs_order:
+                        bit_pos = (index - 1) * subreg_width
+                    else:
+                        bit_pos = self.width - index * subreg_width
+                    sub_reg.set_value((value >> bit_pos) & ((1 << subreg_width) - 1), raw=raw)
 
         except SPSDKError as exc:
             raise SPSDKError(f"Loaded invalid value {str(val)}") from exc
@@ -553,19 +708,25 @@ class RegsRegister:
             subreg_width = self.sub_regs[0].width
             sub_regs_value = 0
             for index, sub_reg in enumerate(self.sub_regs, start=1):
-                sub_regs_value |= sub_reg.get_value() << (self.width - index * subreg_width)
+                if self.reverse_subregs_order:
+                    bit_pos = (index - 1) * subreg_width
+                else:
+                    bit_pos = self.width - index * subreg_width
+                sub_regs_value |= sub_reg.get_value() << (bit_pos)
             if sub_regs_value != self._value:
                 self.set_value(sub_regs_value, raw=True)
 
         return self._value
 
-    def get_bytes_value(self) -> bytes:
+    def get_bytes_value(self, reverse_off: bool = False) -> bytes:
         """Get the bytes value of register.
 
-        The value endianness is returned by 'reversed' member.
+        :param reverse_off: The value indianness is returned by 'reversed' member if True,
+            the reverse member is ignored otherwise.
         :return: Register value in bytes.
         """
-        endianness = "little" if not self.reverse else "big"
+        rev_endianness = "little" if self.base_endianness == "big" else "big"
+        endianness = self.base_endianness if (not self.reverse or reverse_off) else rev_endianness
         return value_to_bytes(
             self.get_value(),
             align_to_2n=False,
@@ -573,10 +734,31 @@ class RegsRegister:
             endianness=endianness,  # type: ignore[arg-type]
         )
 
+    def set_bytes_value(self, value: bytes, reverse_off: bool = False) -> None:
+        """Set register value from the bytes value.
+
+        :param value: The input value in raw bytes.
+        :param reverse_off: The value indianness is loaded by 'reversed' member if True,
+            the reverse member is ignored otherwise.
+        :raises SPSDKValueError: Invalid input value.
+        """
+        if len(value) < self.width // 8:
+            raise SPSDKValueError(f"Invalid length of inputs bytes to set {self.name} register.")
+        rev_endianness = "little" if self.base_endianness == "big" else "big"
+        endianness = self.base_endianness if (not self.reverse or reverse_off) else rev_endianness
+        self.set_value(
+            int.from_bytes(value[: self.width // 8], byteorder=endianness),  # type: ignore[arg-type]
+            raw=True,
+        )
+
     def get_hex_value(self) -> str:
-        """Get the value of register in string hex format."""
-        use_prefix = not self.config_as_hexstring
-        return format_value(self.get_value(), self.width, delimiter="", use_prefix=use_prefix)
+        """Get the value of register in string hex format.
+
+        :return: Hexadecimal value of register.
+        """
+        fmt = f"0{self.width // 4}X"
+        val = f"{'' if self.config_as_hexstring else '0x'}{format(self.get_value(), fmt)}"
+        return val
 
     def get_reset_value(self) -> int:
         """Returns reset value of the register.
@@ -592,7 +774,7 @@ class RegsRegister:
         """
         self._bitfields.append(bitfield)
 
-    def get_bitfields(self, exclude: List[str] = None) -> List[RegsBitField]:
+    def get_bitfields(self, exclude: Optional[List[str]] = None) -> List[RegsBitField]:
         """Returns register bitfields.
 
         Method allows exclude some bitfields by their names.
@@ -608,7 +790,7 @@ class RegsRegister:
             ret.append(bitf)
         return ret
 
-    def get_bitfield_names(self, exclude: List[str] = None) -> List[str]:
+    def get_bitfield_names(self, exclude: Optional[List[str]] = None) -> List[str]:
         """Returns list of the bitfield names.
 
         :param exclude: Exclude list of bitfield names if needed.
@@ -620,8 +802,8 @@ class RegsRegister:
         """Returns the instance of the bitfield by its name.
 
         :param name: The name of the bitfield.
-        :return: The bitfield instance.
-        :raises SPSDKRegsErrorBitfieldNotFound: The register doesn't exists.
+        :return: Instance of the bitfield.
+        :raises SPSDKRegsErrorBitfieldNotFound: The bitfield doesn't exist.
         """
         for bitfield in self._bitfields:
             if name == bitfield.name:
@@ -629,7 +811,7 @@ class RegsRegister:
 
         raise SPSDKRegsErrorBitfieldNotFound(f" The {name} is not found in register {self.name}.")
 
-    def add_setvalue_hook(self, hook: Callable, context: Any = None) -> None:
+    def add_setvalue_hook(self, hook: Callable, context: Optional[Any] = None) -> None:
         """Set the value hook for write operation.
 
         :param hook: Callable hook for set value operation.
@@ -648,6 +830,8 @@ class RegsRegister:
         output += f"Width:  {self.width} bits\n"
         output += f"Access:   {self.access}\n"
         output += f"Description: \n {self.description}\n"
+        if self.otp_index:
+            output += f"OTP Word: \n {self.otp_index}\n"
 
         i = 0
         for bitfield in self._bitfields:
@@ -660,18 +844,20 @@ class RegsRegister:
 class Registers:
     """SPSDK Class for registers handling."""
 
-    def __init__(self, device_name: str) -> None:
+    def __init__(self, device_name: str, base_endianness: str = "big") -> None:
         """Initialization of Registers class."""
+        assert base_endianness in ["little", "big"]
         self._registers: List[RegsRegister] = []
         self.dev_name = device_name
+        self.base_endianness = base_endianness
 
     def find_reg(self, name: str, include_group_regs: bool = False) -> RegsRegister:
         """Returns the instance of the register by its name.
 
         :param name: The name of the register.
         :param include_group_regs: The algorithm will check also group registers.
-        :return: The register instance.
-        :raises SPSDKRegsErrorRegisterNotFound: The register doesn't exists.
+        :return: Instance of the register.
+        :raises SPSDKRegsErrorRegisterNotFound: The register doesn't exist.
         """
         for reg in self._registers:
             if name == reg.name:
@@ -698,6 +884,8 @@ class Registers:
         if reg.name in self.get_reg_names():
             raise SPSDKRegsError(f"Cannot add register with same name: {reg.name}.")
 
+        # update base endianness for all registers in group
+        reg.base_endianness = self.base_endianness
         self._registers.append(reg)
 
     def remove_register(self, reg: RegsRegister) -> None:
@@ -721,7 +909,7 @@ class Registers:
                 self._registers.remove(reg)
 
     def get_registers(
-        self, exclude: List[str] = None, include_group_regs: bool = False
+        self, exclude: Optional[List[str]] = None, include_group_regs: bool = False
     ) -> List[RegsRegister]:
         """Returns list of the registers.
 
@@ -744,7 +932,7 @@ class Registers:
         return regs
 
     def get_reg_names(
-        self, exclude: List[str] = None, include_group_regs: bool = False
+        self, exclude: Optional[List[str]] = None, include_group_regs: bool = False
     ) -> List[str]:
         """Returns list of the register names.
 
@@ -754,7 +942,7 @@ class Registers:
         """
         return [x.name for x in self.get_registers(exclude, include_group_regs)]
 
-    def run_hooks(self, exclude: List[str] = None) -> None:
+    def run_hooks(self, exclude: Optional[List[str]] = None) -> None:
         """The method run hooks on all regular registers.
 
         :param exclude: The list of register names to be excluded.
@@ -762,7 +950,7 @@ class Registers:
         for reg in self.get_registers(exclude):
             reg.set_value(reg.get_value(), False)
 
-    def reset_values(self, exclude: List[str] = None) -> None:
+    def reset_values(self, exclude: Optional[List[str]] = None) -> None:
         """The method reset values in registers.
 
         :param exclude: The list of register names to be excluded.
@@ -791,11 +979,10 @@ class Registers:
         for reg in self._registers:
             reg.add_et_subelement(xml_root)
 
-        with open(file_name, "w", encoding="utf-8") as xml_file:
-            no_pretty_data = minidom.parseString(
-                ET.tostring(xml_root, encoding="unicode", short_empty_elements=False)
-            )
-            xml_file.write(no_pretty_data.toprettyxml())
+        no_pretty_data = minidom.parseString(
+            ET.tostring(xml_root, encoding="unicode", short_empty_elements=False)
+        )
+        write_file(no_pretty_data.toprettyxml(), file_name, encoding="utf-8")
 
     def image_info(
         self, size: int = 0, pattern: BinaryPattern = BinaryPattern("zeros")
@@ -874,8 +1061,8 @@ class Registers:
         self,
         heading1: str,
         heading2: str,
-        regs_exclude: List[str] = None,
-        fields_exclude: List[str] = None,
+        regs_exclude: Optional[List[str]] = None,
+        fields_exclude: Optional[List[str]] = None,
     ) -> str:
         """Generate describing HTML file with registers.
 
@@ -905,13 +1092,16 @@ class Registers:
 
     # pylint: disable=dangerous-default-value
     def load_registers_from_xml(
-        self, xml: str, filter_reg: List[str] = None, grouped_regs: List[dict] = None
+        self,
+        xml: str,
+        filter_reg: Optional[List[str]] = None,
+        grouped_regs: Optional[List[dict]] = None,
     ) -> None:
         """Function loads the registers from the given XML.
 
         :param xml: Input XML data in string format.
         :param filter_reg: List of register names that should be filtered out.
-        :param grouped_regs: List of register prefixes names to be grouped int one.
+        :param grouped_regs: List of register prefixes names to be grouped into one.
         :raises SPSDKRegsError: XML parse problem occurs.
         """
 
@@ -919,7 +1109,8 @@ class Registers:
             """Help function to recognize if the register should be part of group."""
             if grouped_regs:
                 for group in grouped_regs:
-                    if reg.startswith(group["name"]):
+                    # pylint: disable=anomalous-backslash-in-string  # \d is a part of the regex pattern
+                    if re.fullmatch(f"{group['name']}\d+", reg) is not None:
                         return group
             return None
 
@@ -943,9 +1134,10 @@ class Registers:
                         description=group.get(
                             "description", f"Group of {group['name']} registers."
                         ),
-                        reverse=value_to_bool(group.get("reverse", False)),
+                        reverse=value_to_bool(group.get("reversed", False)),
                         access=group.get("access", None),
                         config_as_hexstring=group.get("config_as_hexstring", False),
+                        reverse_subregs_order=group.get("reverse_subregs_order", False),
                     )
 
                     self.add_register(group_reg)
@@ -956,8 +1148,8 @@ class Registers:
     def load_yml_config(
         self,
         yml_data: Any,
-        exclude_regs: List[str] = None,
-        exclude_fields: Dict[str, Dict[str, str]] = None,
+        exclude_regs: Optional[List[str]] = None,
+        exclude_fields: Optional[Dict[str, Dict[str, str]]] = None,
     ) -> None:
         """The function loads the configuration from YML file.
 
@@ -972,12 +1164,22 @@ class Registers:
             register = self.find_reg(reg_name, include_group_regs=True)
             if "value" in reg_dict.keys():
                 raw_val = reg_dict["value"]
-                val = int(raw_val, 16) if register.config_as_hexstring else value_to_int(raw_val)
+                val = (
+                    int(raw_val, 16)
+                    if register.config_as_hexstring and isinstance(raw_val, str)
+                    else value_to_int(raw_val)
+                )
                 register.set_value(val, True)
             elif "bitfields" in reg_dict.keys():
                 for bitfield_name in reg_dict["bitfields"]:
                     bitfield_val = reg_dict["bitfields"][bitfield_name]
-                    bitfield = register.find_bitfield(bitfield_name)
+                    try:
+                        bitfield = register.find_bitfield(bitfield_name)
+                    except SPSDKRegsErrorBitfieldNotFound:
+                        logger.error(
+                            f"The {bitfield_name} is not found in register {register.name}."
+                        )
+                        continue
                     if (
                         exclude_fields
                         and reg_name in exclude_fields.keys()
@@ -985,7 +1187,20 @@ class Registers:
                     ):
                         continue
 
-                    bitfield.set_enum_value(bitfield_val, True)
+                    try:
+                        bitfield.set_enum_value(bitfield_val, True)
+                    except SPSDKError:
+                        # New versions of register data do not contain register and bitfield value in enum
+                        old_bitfield = bitfield_val
+                        bitfield_val = bitfield_val.replace(bitfield.name + "_", "").replace(
+                            register.name + "_", ""
+                        )
+                        # Some bitfield were renamed from ENABLE to ALLOW
+                        bitfield_val = "ALLOW" if bitfield_val == "ENABLE" else bitfield_val
+                        logger.warning(
+                            f"Bitfield {old_bitfield} not found, trying backward compatibility mode with {bitfield_val}"
+                        )
+                        bitfield.set_enum_value(bitfield_val, True)
             else:
                 logger.error(f"There are no data for {reg_name} register.")
 
@@ -1002,11 +1217,13 @@ class Registers:
         def new_line(comment: str) -> str:
             return f"\n{' '*indent}# {comment}"
 
-        description = f"Width: {bitfield.width}b[0-{(1<<bitfield.width)-1}]"
+        description = f"Width: {bitfield.config_width}b"
         if bitfield.description not in ("", "."):
             description += ", Description: " + bitfield.description.replace(
                 "&#10;", f"\n{' '*indent}# "
             )
+        if bitfield.config_processor.description:
+            description += ". NOTE: " + bitfield.config_processor.description
         if bitfield.has_enums():
             for enum in bitfield.get_enums():
                 descr = enum.description if enum.description != "." else enum.name
@@ -1017,7 +1234,7 @@ class Registers:
         return description
 
     def create_yml_config_white_list(
-        self, white_list: Dict[str, Any] = None, diff: bool = False, indent: int = 0
+        self, white_list: Optional[Dict[str, Any]] = None, diff: bool = False, indent: int = 0
     ) -> Any:
         """The function creates the configuration YML file.
 
@@ -1077,9 +1294,9 @@ class Registers:
     # pylint: disable=(useless-param-doc, useless-type-doc)
     def create_yml_config(
         self,
-        exclude_regs: List[str] = None,
-        exclude_fields: Dict[str, Dict[str, str]] = None,
-        ignored_fields: List[str] = None,
+        exclude_regs: Optional[List[str]] = None,
+        exclude_fields: Optional[Dict[str, Dict[str, str]]] = None,
+        ignored_fields: Optional[List[str]] = None,
         diff: bool = False,
         indent: int = 0,
     ) -> Any:
